@@ -1,7 +1,11 @@
 /*
-#ident	"@(#)smail/compat:RELEASE-3_2_0_111:dummy.c,v 1.3 1997/08/08 03:44:06 woods Exp"
+#ident	"@(#)smail/compat:RELEASE-3_2_0_112:dummy.c,v 1.4 2000/09/03 16:32:08 woods Exp"
  */
 
+#include "defs.h"
+#include "../src/smail.h"
+#include "compat.h"
+
 /*
  * Provide a dummy reference so there is at least one item in
  * compat.a.  This will keep ld from complaining about empty libraries
