@@ -1,6 +1,6 @@
 #! /bin/sh
 :
-#ident	"@(#)smail/conf/lib:RELEASE-3_2_0_111:mkdefs.sh,v 1.57 1999/06/26 16:48:01 woods Exp"
+#ident	"@(#)smail/conf/lib:RELEASE-3_2_0_112:mkdefs.sh,v 1.59 2000/09/11 01:09:53 woods Exp"
 
 #    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
 #    Copyright (C) 1992  Ronald S. Karr
@@ -105,7 +105,7 @@ VARS="$VARS\
  SOCKET_INCLUDES\
 "
 
-# list OS/ARCH vars, for these variables, the EDITME versions are appended
+# list OS & ARCH vars, for these variables, the EDITME versions are appended
 LISTS="\
  CFLAGS\
  CPPFLAGS\
@@ -136,7 +136,7 @@ NLISTS="\
 HAVE_REQUIRE="\
 "
 
-# list vars that are set below, and *not* in EDITME/OS/ARCH, etc.
+# list vars that are set below, and *not* in EDITME, OS, & ARCH, etc.
 # but are set by this script from derrived values....
 NOEDVARS="\
  PATH_ALIASES_FILE\
@@ -151,7 +151,7 @@ NOEDVARS="\
 sep=":$IFS"
 save_IFS="$IFS"
 
-# clear all EDITME/OS/ARCH variables
+# clear all EDITME, OS, & ARCH variables
 for v in $VARS $NOEDVARS $LISTS $NLISTS; do
 	eval "case \"$v\" in ?*) $v=;; esac"
 done
@@ -291,8 +291,8 @@ for v in $VARS $LISTS $NLISTS; do
 	     "
 done
 
-# Fill in defaults.  OS/ARCH files can override these.  EDITME file
-# can override the defaults or the OS/ARCH files.
+# Fill in defaults.  OS & ARCH files can override these.  EDITME file
+# can override the defaults or the OS & ARCH files.
 #
 CC=cc
 TMP_DIR='$SPOOL_DIRS/tmp'
@@ -369,7 +369,7 @@ for v in $LISTS; do
 		listsep=:
 		;;
 	*)
-	      listsep=' '
+		listsep=' '
 		;;
 	esac
 	eval "
@@ -663,21 +663,23 @@ esac
 
 cat <<EOF
 # DO NOT EDIT THIS FILE DIRECTLY, IT IS CREATED AUTOMATICALLY
-# FROM THE SCRIPT $ROOT/conf/lib/mkdefs.sh.  IF YOU MAKE
-# CHANGES TO THIS FILE THEY ARE LIKELY TO GO AWAY.  SEE THE FILE
-# $EDITME FOR INFORMATION ON HOW TO CONTROL
-# THE CONTENTS OF THIS FILE.
-
+# BY THE SCRIPT $ROOT/conf/lib/mkdefs.sh.
+#
+# IF YOU MAKE CHANGES TO THIS FILE THEY ARE LIKELY TO GO AWAY.
+#
+# SEE THE FILE $EDITME
+# FOR INFORMATION ON HOW TO CONTROL THE CONTENTS OF THIS FILE.
+#
 FROM_ROOT='$FROM_ROOT'
-DEFS_DEPEND=\
-'defs.sh defs.h defs.sed: $RELATIVE_EDITME
+DEFS_DEPEND='
+defs.sh defs.h defs.sed: $RELATIVE_EDITME
 defs.sh defs.h defs.sed: \${ROOT}/conf/lib/arch.h
 defs.sh defs.h defs.sed: \${ROOT}/conf/lib/mkdefs.sh
 defs.sh defs.h defs.sed: \${ROOT}/conf/os/$OS_TYPE
 defs.sh defs.h defs.sed: \${ROOT}/conf/arch/$ARCH_TYPE
 version.sh version.h version.sed: \${ROOT}/conf/lib/mkversion.sh
 version.sh version.h version.sed: \${ROOT}/level
-$MISC_DEFS_DEPEND'
+$MISC_DEFS_DEPEND' 
 EOF
 
 # Write out the simple variables
@@ -748,9 +750,11 @@ cat <<EOF
 /*
  * DO NOT EDIT THIS FILE DIRECTLY, IT IS CREATED AUTOMATICALLY
  * BY THE SCRIPT $ROOT/conf/lib/mkdefs.sh.
- * IF YOU MAKE CHANGES TO THIS FILE THEY ARE LIKELY TO GO AWAY.  SEE
- * THE FILE $EDITME FOR INFORMATION ON HOW TO CONTROL THE CONTENTS OF
- * THIS FILE.
+ *
+ * IF YOU MAKE CHANGES TO THIS FILE THEY ARE LIKELY TO GO AWAY.
+ *
+ * SEE THE FILE $EDITME
+ * FOR INFORMATION ON HOW TO CONTROL THE CONTENTS OF THIS FILE.
  */
 
 #define MAILBOX_DIR		"$MAILBOX_DIR"
@@ -760,12 +764,13 @@ $DEFINE_LOCAL_MAIL_COMMAND
 #define LOCAL_MAIL_FILE		"$LOCAL_MAIL_FILE"
 $LOCKING_PROTOCOL
 $DECLARE_STRINGS
+
 EOF
 
 # Handle string variables which should be turned off with a NO_ prefix
 cat <<EOF | sed -n \
-		-e 's/\([A-Z0-9_]*\)=\(..*\)/#define \1		"\2"/p' \
-		-e 's/\([A-Z0-9_]*\)=$/#define NO_\1/p'
+	-e 's/\([A-Z0-9_]*\)=\(..*\)/#define \1		"\2"/p' \
+	-e 's/\([A-Z0-9_]*\)=$/#define NO_\1/p'
 ALIASES_FILE=$ALIASES_FILE
 PATH_ALIASES_FILE=$PATH_ALIASES_FILE
 ALIASES_PROTO=$ALIASES_TYPE
@@ -781,6 +786,7 @@ FORCE_SMTP_PROTO=$FORCE_SMTP_TYPE
 REWRITE_FILE=$REWRITE_FILE
 PATH_REWRITE_FILE=$PATH_REWRITE_FILE
 REWRITE_PROTO=$REWRITE_TYPE
+
 EOF
 
 # Handle string variables which should revert to defaults if undefined
@@ -808,6 +814,7 @@ cat <<EOF | sed -n \
 		-e 's/\([A-Z0-9_]*\)=$/#undef \1/p'
 RETRY_INTERVAL=$RETRY_INTERVAL
 RETRY_DURATION=$RETRY_DURATION
+
 EOF
 
 # Handle NULL or string variables
@@ -842,6 +849,7 @@ cat <<EOF | sed -n \
 		-e 's/\([A-Z0-9_]*\)=$/#define \1		FALSE/p'
 LOCK_BY_NAME=$LOCK_BY_NAME
 FLOCK_MAILBOX=$FLOCK_MAILBOX
+
 EOF
 
 # Handle on or off variables
@@ -854,6 +862,7 @@ UUWHO_USE_DBM=$UUWHO_USE_DBM
 UUCP_ZONE=$UUCP_ZONE
 ALIASES_REMOVE_SENDER=$ALIASES_REMOVE_SENDER
 LISTS_REMOVE_SENDER=$LISTS_REMOVE_SENDER
+
 EOF
 
 # write out the positibe boolean list variables
@@ -878,13 +887,21 @@ EOF
 	done
 )
 
-# write the extensible set of variables that may have values
+# write the extensible set of variables that may have values.
+# 
+# NOTE: we first undef these values and then define them again as
+# this is necessary to allow EDITME settings to override OS & ARCH
+# settings.
+# 
+# WARNING:  this may hide unintentional conflicts with system defines!
 (
 	IFS="$sep"
 	for i in $MISC_DEFINES; do
 		echo "$i"
-	done | sed -n -e 's/^\(.[^=]*\)$/#define \1/p' \
-		-e 's/^\(.[^=]*\)=\(.*\)/#define \1		\2/p'
+	done | sed -n -e 's/^\(.[^=]*\)$/#undef \1\
+#define \1/p'\
+		-e 's/^\(.[^=]*\)=\(.*\)/#undef \1\
+#define \1		\2/p'
 )
 
 if [ -z "$SOCKET_INCLUDES" ]; then
@@ -896,18 +913,23 @@ cat <<EOF
 #define ALIGNED_TYPE	$ALIGNED_TYPE
 #define BITS_PER_CHAR	$BITS_PER_CHAR
 #define BYTES_PER_ALIGN	$BYTES_PER_ALIGN
+
 $MISC_C_DEFINES
+
 #if defined(__STDC__) && !defined(NO_ANSI_C)
 # ifndef ANSI_C
 #  define ANSI_C 1
 # endif
 #endif
+
 #include "$RELATIVE_ROOT/conf/lib/arch.h"
+
 #ifdef NEED_SOCKETS
 #ifdef HAVE_BSD_NETWORKING
 $SOCKET_INCLUDES
 #endif	/* HAVE_BSD_NETWORKING */
 #endif	/* NEED_SOCKETS */
+
 EOF
 
 # Build the sed change script
@@ -927,6 +949,7 @@ case "$MISC_SED_DEFINES" in
 ?*)
 	cat <<EOF
 $MISC_SED_DEFINES
+
 EOF
 	;;
 esac
@@ -942,6 +965,7 @@ s|X_OLD_LOGDIR_X|$OLD_LOGDIR|g
 s|X_MAIN_SPOOL_DIR_X|$MAIN_SPOOL_DIR|g
 s|X_EDITME_X|$EDITME|g
 s|X_OS_TYPE_X|$OS_TYPE|g
+
 EOF
 
 # write out the simple variables.  Ignore variables with newlines or | chars
