@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:addr.c,v 1.34 2000/02/14 05:30:16 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:addr.c,v 1.37 2000/09/29 21:18:00 woods Exp"
  */
 
 /*
@@ -20,7 +20,9 @@
  *			     insert_addr_list, addr_sort, split_addr_list
  */
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <stdio.h>
+#include <errno.h>
 #include <ctype.h>
 
 #define NEED_SOCKETS			/* Required for IP address validation */
@@ -29,15 +31,22 @@
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
 #endif
-#include "config.h"
 #include "smail.h"
+#include "main.h"
+#include "config.h"
+#include "parse.h"
 #include "addr.h"
+#include "field.h"
+#include "route.h"
+#include "direct.h"
+#include "transport.h"
+#include "alloc.h"
 #include "dys.h"
 #include "exitcodes.h"
-#ifndef DEPEND
-# include "debug.h"
-# include "extern.h"
-#endif
+#include "smailconf.h"
+#include "bindlib.h"
+#include "debug.h"
+#include "extern.h"
 
 #ifndef INADDR_NONE
 # define INADDR_NONE		((unsigned long) (-1)) /* XXX 64bit too??? */
@@ -46,17 +55,11 @@
 # define INADDR_LOOPBACK	((unsigned long) 0x7f000001) /* XXX not 64-bit clean */
 #endif
 
-#ifdef HAVE_BSD_NETWORKING
-# ifndef OBSOLETE_RESOLVER
-extern int h_errno;			/* import h_errno; many systems don't define it in <netdb.h> */
-# endif
-#endif
-
 /* functions local to this file */
-static int check_target_and_remainder();
-static char *escaped();
-static char *internal_build_uucp_route();
-static int addrcmp();
+static int check_target_and_remainder __P((char **, char **));
+static char *escaped __P((char *, char *));
+static char *internal_build_uucp_route __P((char *, char **, int, int));
+static int addrcmp __P((const void *, const void *));
 
 
 /*
@@ -1398,8 +1401,8 @@ alloc_addr()
     /* preset the proper values */
     bzero((char *)addr, sizeof(*addr));	/* XXX hope NULL pointers are all zeros! */
     addr->match_count = -1;
-    addr->uid = BOGUS_USER;		/* the identity is not known yet */
-    addr->gid = BOGUS_GROUP;		/* the identity is not known yet */
+    addr->uid = (unsigned int) BOGUS_USER;	/* the identity is not known yet */
+    addr->gid = (unsigned int) BOGUS_GROUP;	/* the identity is not known yet */
 
     return addr;
 }
@@ -1419,10 +1422,10 @@ insert_addr_list(in, out, error)
 {
     struct addr *next;
 
-    DEBUG(DBG_ADDR_MID, "insert_addr_list() called:\n");
+    DEBUG(DBG_ADDR_HI, "insert_addr_list() called:\n");
 #ifndef NODEBUG
     if (error) {
-	DEBUG2(DBG_ADDR_MID, "\tERR%ld: %s\n",
+	DEBUG2(DBG_ADDR_HI, "\tERR%ld: %s\n",
 	       error->info & ERR_MASK, error->message);
     }
 #endif	/* NODEBUG */
@@ -1430,7 +1433,7 @@ insert_addr_list(in, out, error)
     for (; in; in = next) {
 	next = in->succ;
 
-	DEBUG1(DBG_ADDR_MID, "\t%s\n", in->in_addr);
+	DEBUG1(DBG_ADDR_HI, "\t%s\n", in->in_addr);
 	if (error) {
 	    in->error = error;		/* set the error message, if given */
 	}
@@ -1439,6 +1442,12 @@ insert_addr_list(in, out, error)
     }
 }
 
+
+/*
+ * offset passed through the heap to the compare function....
+ */
+static int sort_offset;
+
 /*
  * addr_sort - sort an input list of addrs and return the new sorted list
  *
@@ -1448,14 +1457,13 @@ insert_addr_list(in, out, error)
  * where tag_name is the (char *) element name in the addr structure to
  * sort on.
  */
-static int sort_offset;			/* pass offset to compare function */
 struct addr *
 addr_sort(in, offset)
     struct addr *in;
     int offset;
 {
     struct addr **addrv;		/* array of addresses */
-    register int addrc;			/* count of addresses */
+    register unsigned int addrc;	/* count of addresses */
     register struct addr **addrp;	/* temp addr pointer */
     register struct addr *a;		/* address list or current address */
 
@@ -1496,10 +1504,13 @@ addr_sort(in, offset)
  * addrcmp - compare two addr structures based on a field at sort_offset.
  */
 static int
-addrcmp(a, b)
-    char **a;
-    char **b;
+addrcmp(x, y)
+    const void *x;
+    const void *y;
 {
+    char **a = (char **) x;
+    char **b = (char **) y;
+
     return strcmp(*(char **)(*a + sort_offset), *(char **)(*b + sort_offset));
 }
 
