@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:addr.h,v 1.11 1996/12/11 21:04:05 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:addr.h,v 1.13 2000/09/03 15:43:54 woods Exp"
  */
 
 /*
@@ -40,8 +40,8 @@ struct addr {
     char *next_addr;		/* address to give next-hop host */
     struct transport *transport; /* transport to use for remote delivery */
     char *home;			/* home directory associated with login */
-    int uid;			/* user for pipes/files perms or BOGUS_USER */
-    int gid;			/* gid for pipes/files perms or BOGUS_GROUP */
+    unsigned int uid;		/* user for pipes/files perms or BOGUS_USER */
+    unsigned int gid;		/* gid for pipes/files perms or BOGUS_GROUP */
     struct error *error;	/* error message associated */
     struct transport_hints *tphint_list; /* transport hints from the router */
 };
@@ -113,6 +113,7 @@ struct transport_hints {
 #define ADDR_SHADOW	0x00800000	/* using shadow transport */
 #define ADDR_PARTLOCAL	0x01000000	/* partially matched local host */
 #define ADDR_RETRY_FILE	0x02000000	/* on failure, touch retry file */
+#define ADDR_VRFY_ONLY	0x04000000	/* don't do any transforms in directors */
 #define ADDR_FORM_MASK	0x0000000f	/* form from parse_address */
 /*
  * NOTE:
@@ -151,3 +152,20 @@ struct transport_hints {
 
 /* flag values for the parse_address flagp argument */
 #define FOUND_MAILBOX	0x0001		/* found user@host address form */
+
+/* external functions defined in addr.c */
+extern char *preparse_address __P((char *, char **));
+extern char *preparse_address_1 __P((char *, char **, char **));
+extern int parse_address __P((char *, char **, char **, int *));
+extern char *address_token __P((char *));
+extern char *back_address_token __P((char *, char *));
+extern int mixed_address __P((char *));
+extern char *build_uucp_route __P((char *, char **, int));
+extern char *build_partial_uucp_route __P((char *, char **, int));
+extern void split_addr_list __P((char *, struct addr **));
+extern void strip_rfc822_comments __P((char *));
+extern void strip_rfc822_whitespace __P((char *));
+extern struct addr *alloc_addr __P((void));
+extern void insert_addr_list __P((struct addr *, struct addr **, struct error *));
+extern struct addr *addr_sort __P((struct addr *, int));
+extern struct error *note_error __P((long, char *));
