@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:alloc.h,v 1.10 1999/02/03 06:03:29 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:alloc.h,v 1.11 2000/09/03 16:29:01 woods Exp"
  */
 
 /*
@@ -70,3 +70,14 @@ struct bseg {
 /* use these macros to panic code which should not generate a panic */
 #define X_NO_PANIC() (x_dont_panic = TRUE)
 #define X_PANIC_OKAY() (x_dont_panic = FALSE)
+
+/* external functions defined in alloc.c */
+extern char *xmalloc __P((unsigned int));
+extern char *xrealloc __P((char *, unsigned int));
+extern void xfree __P((char *));
+extern char *bmalloc __P((unsigned int, struct block *));
+extern char *brealloc __P((char *, unsigned int, struct block *));
+extern void bfree __P((char *, struct block *));
+extern struct block *malloc_block __P((void));
+extern void realloc_block __P((char *, struct block *, struct block *));
+extern void free_block __P((struct block *));
