@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:bindlib.c,v 1.64 2000/02/05 19:08:02 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:bindlib.c,v 1.69 2000/11/03 23:30:51 woods Exp"
  */
 
 /*
@@ -240,10 +240,11 @@
  */
 
 #define NEED_SOCKETS
-#include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/param.h>
+#include <stdio.h>
+#include <errno.h>
 #include "defs.h"
 
 /*
@@ -253,20 +254,20 @@
 #ifdef HAVE_BIND
 
 #include "smail.h"
+#include "main.h"
 #include "log.h"
-#include "alloc.h"
+#include "parse.h"
 #include "addr.h"
 #include "route.h"
 #include "transport.h"
 #include "bindlib.h"
 #include "bindsmtpth.h"
 #include "lookup.h"
+#include "alloc.h"
 #include "dys.h"
-#ifndef DEPEND
-# include "extern.h"
-# include "debug.h"
-# include "error.h"
-#endif
+#include "extern.h"
+#include "debug.h"
+#include "error.h"
 
 /* Additional return code
  * I need a negative match, which is a stronger statement than DB_NOMATCH
@@ -312,12 +313,12 @@
  */
 enum rr_sect { SECT_AN, SECT_NS, SECT_AR };
 typedef struct rr {
-    enum rr_sect rr_sect;               /* resource record section */
-    char  *rr_dname;                    /* domain name */
-    short  rr_class;                    /* class number */
-    short  rr_type;                     /* type number */
-    int    rr_size;                     /* size of data area */
-    char  *rr_data;                     /* pointer to data */
+    enum rr_sect rr_sect;              /* resource record section */
+    char *rr_dname;                    /* domain name */
+    short rr_class;                    /* class number */
+    short rr_type;                     /* type number */
+    unsigned int rr_size;              /* size of data area */
+    char *rr_data;                     /* pointer to data */
 } RR;
 
 /* structure for iterating over RR's with getnextrr() */
@@ -352,61 +353,47 @@ struct mx_work_area {
 
 static struct bindlib_private generic_bindattr = BIND_TEMPLATE_ATTRIBUTES;
 
-/*
- * import h_errno; many systems don't define it in <netdb.h>
- */
-
-#ifndef OBSOLETE_RESOLVER
-extern int h_errno;
-#endif
 
 /* functions local to this file */
 
-#ifdef ANSI_C
-# define P_(x) x
-#else
-# define P_(x) ()
-#endif
-
-static int bind_addr_work P_((char*,long,struct bindlib_private*,char*,
+static int bind_addr_work __P((char*,long,struct bindlib_private*,char*,
                        struct rt_info*,struct error **));
-static int full_mx_lookup P_((char *, struct mx_work_area *, int,
+static int full_mx_lookup __P((char *, struct mx_work_area *, int,
                        struct bindlib_private *));
-static int find_mx_records P_((char *, struct mx_work_area *, int));
-static int decode_cnames P_((struct mx_work_area *));
-static int handle_null_mxs P_((struct mx_work_area *, struct bindlib_private *));
-static int make_mx_hints P_((struct mx_work_area *));
-static int filter_mx_hints P_((struct mx_work_area *));
-static int get_addr_hints P_((struct mx_work_area *));
-static void make_rt_entry P_((struct rt_info *, char *, char *, struct transport_hints *));
-static int route_by_greybook P_((struct mx_work_area *));
-static int do_straight_lookup P_((char *, struct mx_work_area *));
-static struct mx_work_area * mx_work_area_constructor 
-    P_((struct bindlib_private *, struct error **, char *, long, int *, int *));
-static void mx_work_area_destructor P_((struct mx_work_area *, int));
-static char * check_if_gatewayed P_((char *, char *));
-static char *strip_dots P_((char*));
-static void flip P_((char *));   /* Flip address between UK/world order */
-static char *rewrite_header P_((char *, char *, char *));
-static int get_records P_((char*,int,HEADER*,int*,char **));
-static RR *getnextrr P_((struct rr_iterator*,int));
-static void rewindrr P_((struct rr_iterator*,HEADER*,int));
-static void do_header_rewrites P_((char *, char *));
-static int find_a_records P_((struct mx_transport_hint*,char*,char **));
-static struct error *not_a_canonical_host P_((char*));
-static struct error *server_failure P_((char*,char*));
-static struct error *packet_error P_((char*,char*,char*));
-static struct error *matched_local_host P_((char*,char*));
-static struct error *no_transport_error P_((char*,char*));
-static struct error *open_failed P_((char*, char*, char*));
-static struct error *cname_loop_error P_((char*,char*,int));
-static struct error *res_init_error P_((char*));
-static int decode_mx_rr P_((RR*,struct rr_iterator*,char **,int*));
-static struct transport_hints * new_mx_hint P_((int,char*,int));
-static void add_mx_hint P_((struct transport_hints **,int,char*,int));
-static void free_mx_hint P_((struct transport_hints **));
-static void add_a_hint P_((struct mx_transport_hint*,char*,char*));
-static char * match_full_or_end_domain P_((char * domains, char * target));
+static int find_mx_records __P((char *, struct mx_work_area *, int));
+static int decode_cnames __P((struct mx_work_area *));
+static int handle_null_mxs __P((struct mx_work_area *, struct bindlib_private *));
+static int make_mx_hints __P((struct mx_work_area *));
+static int filter_mx_hints __P((struct mx_work_area *));
+static int get_addr_hints __P((struct mx_work_area *));
+static void make_rt_entry __P((struct rt_info *, char *, char *, struct transport_hints *));
+static int route_by_greybook __P((struct mx_work_area *));
+static int do_straight_lookup __P((char *, struct mx_work_area *));
+static struct mx_work_area *mx_work_area_constructor  __P((struct bindlib_private *, struct error **, char *, long, int *, int *));
+static void mx_work_area_destructor __P((struct mx_work_area *, int));
+static char * check_if_gatewayed __P((char *, char *));
+static char *strip_dots __P((char*));
+static void flip __P((char *));   /* Flip address between UK/world order */
+static char *rewrite_header __P((char *, char *, char *));
+static int get_records __P((char*,int,HEADER*,int*,char **));
+static RR *getnextrr __P((struct rr_iterator*,int));
+static void rewindrr __P((struct rr_iterator*,HEADER*,int));
+static void do_header_rewrites __P((char *, char *));
+static int find_a_records __P((struct mx_transport_hint*,char*,char **));
+static struct error *not_a_canonical_host __P((char*));
+static struct error *server_failure __P((char*,char*));
+static struct error *packet_error __P((char*,char*,char*));
+static struct error *matched_local_host __P((char*,char*));
+static struct error *no_transport_error __P((char*,char*));
+static struct error *open_failed __P((char*, char*, char*));
+static struct error *cname_loop_error __P((char*,char*,int));
+static struct error *res_init_error __P((char*));
+static int decode_mx_rr __P((RR*,struct rr_iterator*,char **,int*));
+static struct transport_hints * new_mx_hint __P((int,char*,int));
+static void add_mx_hint __P((struct transport_hints **,int,char*,int));
+static void free_mx_hint __P((struct transport_hints **));
+static void add_a_hint __P((struct mx_transport_hint*,char*,char*));
+static char * match_full_or_end_domain __P((char * domains, char * target));
 
 
 
@@ -420,8 +407,6 @@ static char * match_full_or_end_domain P_((char * domains, char * target));
 char *
 bind_compute_domain ()
 {
-    
-#ifdef RES_INIT
     /*  This initialises the resolver.  If the resolver is so old that RES_INIT
      *  isn't defined then the routines will initialise themselves - that is
      *  if the thing will compile at all!
@@ -434,10 +419,6 @@ bind_compute_domain ()
     }
 
     return(COPY_STRING(_res.defdname));
-#else
-    /* Not sure whats likely to be possible in a system this old! */
-    return(NULL);
-#endif
 }
 
 
@@ -464,7 +445,7 @@ bind_compute_domain ()
  *			notify postmaster.
  */
 
-#define CANONICAL_HOST_CHECK	"canonical host check"
+#define CANONICAL_HOST_CHECK	"bind"	/* caller's are explicit enough */
 
 int
 bind_check_if_canonical_host(hostnm, inet, error_p)
@@ -512,9 +493,9 @@ bind_check_if_canonical_host(hostnm, inet, error_p)
     result = get_records(hostnm, T_A, a_rrs, &a_size, &error_msg);
 
     if (result != DB_SUCCEED) {
-        xfree((char *) a_rrs);
+	xfree((char *) a_rrs);
 	*error_p = server_failure(CANONICAL_HOST_CHECK, error_msg);
-        return result;
+	return result;
     }
 
     result = DB_NOMATCH;		/* assume no data, or no A RRs, in response */
@@ -527,16 +508,17 @@ bind_check_if_canonical_host(hostnm, inet, error_p)
 	    if ((a_rr->rr_type == T_A) && (memcmp(a_rr->rr_data, (char *) &inet, sizeof(inet)) == 0)) {
 		result = DB_SUCCEED;	/* Oh!  I guess there was a matching A RR! */
 	    } else if (a_rr->rr_type == T_CNAME) {
-		*error_p = not_a_canonical_host(xprintf("the domain name '%s' is a CNAME not a hostname (it must be a domain name resolving to an A record)", hostnm));
+		*error_p = not_a_canonical_host(xprintf("the domain name '%s' is a CNAME not a hostname\\\n\t(it must be a domain name resolving to an A record)", hostnm));
 	    } /* we don't care about any other kind of RRs */
 	}
     }
     if (result != DB_SUCCEED && !*error_p) {
-	*error_p = not_a_canonical_host(xprintf("no A records for the hostname '%s' have a target address matching the source address [%s]",
+	/* XXX we probably don't need the actual source address in here... */
+	*error_p = not_a_canonical_host(xprintf("no DNS A records for the hostname '%s' have a\\\n\ttarget address matching the source address [%s]",
 						hostnm,
 						inet_ntoa(*((struct in_addr *) &inet))));
     } else if (nothing) {
-	*error_p = not_a_canonical_host(xprintf("A DNS query did not return any records matching the hostname '%s'", hostnm));
+	*error_p = not_a_canonical_host(xprintf("A DNS query did not return any A records matching the hostname '%s'", hostnm));
     }
     xfree((char *) a_rrs);
     _res.options = save_res_options;
@@ -617,28 +599,28 @@ bind_check_if_local_mxs(target, local_precedence_p, error_p)
 	mx_work_area_destructor(mxdat, FALSE);
 	_res.options = save_res_options;
 
-        return ret;
+	return ret;
     }
     if ((ret = decode_cnames(mxdat)) != DB_SUCCEED) {
 	DEBUG1(DBG_DRIVER_LO, "problem decoding cnames for %s.\n", strip_target);
 	mx_work_area_destructor(mxdat, FALSE);
 	_res.options = save_res_options;
 
-        return ret;
+	return ret;
     }
     if ((ret = make_mx_hints(mxdat)) != DB_SUCCEED) {
 	DEBUG1(DBG_DRIVER_LO, "problem finding MX hints for %s.\n", strip_target);
 	mx_work_area_destructor(mxdat, FALSE);
 	_res.options = save_res_options;
 
-        return ret;
+	return ret;
     }
     if ((ret = get_addr_hints(mxdat)) != DB_SUCCEED) {
 	DEBUG1(DBG_DRIVER_LO, "problem filling in address hints for %s.\n", strip_target);
 	mx_work_area_destructor(mxdat, FALSE);
 	_res.options = save_res_options;
 
-        return ret;
+	return ret;
     }
     ret = DB_SUCCEED;
     *local_precedence_p = mxdat->local_precedence;
@@ -654,6 +636,90 @@ bind_check_if_local_mxs(target, local_precedence_p, error_p)
 
 
 
+/*
+ * gather up the TXT records for a domain
+ */
+char *
+bind_lookup_txt_rr(target, error_p)
+    char *target;
+    struct error **error_p;
+{
+    char *error_msg = NULL;		/* pointer for error string from get_records() */
+    HEADER *txt_rrs;
+    int txt_size;
+    struct rr_iterator txt_it;
+    RR *txt_rr;
+    int result;
+    char *text = NULL;
+
+    DEBUG1(DBG_DRIVER_MID, "bind_lookup_txt_rr(%s) called.\n", target);
+
+    if (error_p) {
+	*error_p = (struct error *) NULL;
+    }
+
+#ifdef RES_INIT
+    /*  This initialises the resolver.  If the resolver is so old that RES_INIT
+     *  isn't defined then the routines will initialise themselves - that is
+     *  if the thing will compile at all!
+     */
+    if ((_res.options & RES_INIT)  == 0) {
+	if (res_init() == EOF) {
+	    DEBUG(DBG_DRIVER_LO, "ERROR: res_init() failed - cannot do TXT lookup\n");
+	    if (error_p) {
+		*error_p = res_init_error(LOCALHOST_MX_CHECK);
+	    }
+	    return (char *) NULL;
+	}
+    }
+#endif
+    txt_rrs = (HEADER *) xmalloc(MAXPACKET);
+    result = get_records(target, T_TXT, txt_rrs, &txt_size, &error_msg);
+    if (result != DB_SUCCEED) {
+	xfree((char *) txt_rrs);
+	if (error_p) {
+	    *error_p = server_failure(CANONICAL_HOST_CHECK, error_msg);
+	}
+	return (char *) NULL;
+    }
+    rewindrr(&txt_it, txt_rrs, txt_size);
+    while ((txt_rr = getnextrr(&txt_it, TRUE)) != 0) {
+	if (txt_rr->rr_type == T_TXT) {
+	    char *tp, *cp, *cp2;
+
+	    if (!text) {
+		text = xmalloc((txt_rr->rr_size * 2) + 1);
+		tp = text;
+	    } else {
+		int curlen = strlen(text);
+
+		text = xrealloc(text, curlen + (txt_rr->rr_size * 2) + 1);
+		tp = text + curlen;
+	    }
+	    cp = txt_rr->rr_data;
+	    cp2 = txt_rr->rr_data + txt_rr->rr_size;
+	    while (cp < cp2) {
+		int n;
+
+		if ((n = (unsigned char) *cp++)) {
+		    for (; n > 0 && cp < cp2; n--) {
+			*tp++ = *cp++;
+		    }
+		}
+		if (cp < cp2) {
+		    *tp++ = '\n';		/* ???? */
+		}
+	    }
+	    *tp = '\0';
+	}
+    }
+    xfree((char *) txt_rrs);
+
+    return text;
+}
+
+
+
 /*
  * bind_addr - lookup a host through the domain system
  *
@@ -1709,14 +1775,14 @@ static void mx_work_area_destructor (mxdat, do_not_deallocate_mxhints)
     /* Deallocate storage */
     if (mxdat->matched_target != NULL) xfree(mxdat->matched_target);
     if (mxdat->full_target != NULL)    xfree(mxdat->full_target);
-    if (mxdat->mx_rrs != NULL)         xfree(mxdat->mx_rrs);
+    if (mxdat->mx_rrs != NULL)         xfree((char *) mxdat->mx_rrs);
 
     if (!do_not_deallocate_mxhints) {
         while(mxdat->mx_hints) {
             free_mx_hint(&(mxdat->mx_hints));
         }
     }
-    xfree(mxdat);
+    xfree((char *) mxdat);
 }
 
 
@@ -1821,8 +1887,8 @@ rewrite_header(text, oldtarget, newtarget)
     char *newtarget;
 {
     char *p = text;
-    int oldlen = strlen(oldtarget);
-    int newlen = strlen(newtarget);
+    unsigned int oldlen = strlen(oldtarget);
+    unsigned int newlen = strlen(newtarget);
 
     while (*p)
         {
@@ -1895,25 +1961,10 @@ get_records(qname, qtype, answer, pack_size, error)
     int *pack_size;                     /* store answer packet size here */
     char **error;                       /* store error message here */
 {
-#ifdef OBSOLETE_RESOLVER
-    char msgbuf[MAXPACKET];
-    int msglen;
-#endif
     int anslen;
 
     *pack_size = 0;                     /* Ensure packet size is zero unless set */
     *error = NULL;
-#ifdef OBSOLETE_RESOLVER
-    msglen = res_mkquery(QUERY, qname, C_IN, qtype, (char *)NULL, 0,
-                         (struct rrec *)NULL, msgbuf, MAXPACKET);
-
-    anslen = res_send(msgbuf, msglen,  (char *)answer, MAXPACKET);
-    if (anslen < 0) {
-	/* XXX cast of strerror() is to explicitly discard const */
-	*error = (char *) strerror(errno);
-        return FILE_NOMATCH;
-    }
-#else                                   /* not OBSOLETE_RESOLVER */
     anslen = res_search(qname, C_IN, qtype, (u_char *) answer, MAXPACKET);
     if (anslen < 0) {
 	/* XXX cast of hstrerror() is to explicitly discard const */
@@ -1950,7 +2001,6 @@ get_records(qname, qtype, answer, pack_size, error)
             return FILE_AGAIN;
         }
     }
-#endif /* not OBSOLETE_RESOLVER */
     *pack_size = anslen;
 
     answer->qdcount = ntohs(answer->qdcount);
@@ -1961,29 +2011,6 @@ get_records(qname, qtype, answer, pack_size, error)
     switch (answer->rcode) {
     case NOERROR:
         return DB_SUCCEED;
-
-#ifdef OBSOLETE_RESOLVER
-    case FORMERR:
-        *error = "DNS resolver: Format error in packet";
-        return DB_FAIL;
-
-    case SERVFAIL:
-        *error = "DNS resolver: Server failure";
-        return DB_AGAIN;
-
-    case NXDOMAIN:
-	*error = "DNS resolver: no such domain name";
-        return DB_NOMATCH;
-
-    case NOTIMP:
-        *error = "DNS resolver: Unimplemented request";
-        return DB_FAIL;
-
-    case REFUSED:
-        *error = "DNS resolver: Query refused";
-        return FILE_AGAIN;
-#endif /* OBSOLETE_RESOLVER */
-
     default:
         *error = "DNS resolver: Unknown response code";
         return DB_FAIL;
@@ -2171,8 +2198,6 @@ server_failure(what, error)
 {
     char *error_text;
 
-    extern int errno;
-
     /*
      * ERR_164 - failure talking to DNS server
      *
