@@ -1,4 +1,6 @@
-/* @(#)src/bindlib.h bindlib.h,v 1.9 1997/12/02 00:30:40 woods Exp */
+/*
+#ident	"@(#)src/bindlib.h:RELEASE-3_2_0_112:bindlib.h,v 1.10 2000/09/03 15:53:01 woods Exp"
+*/
 
 /*
  *    Copyright (C) 1992  Ronald S. Karr
@@ -52,7 +54,7 @@ struct bindlib_private {
     char *uk_greybook_hosts_file;       /* UK: file of hosts I can gb to */
     char *uk_greybook_hosts_proto;      /* UK: proto for above */
     int uk_greybook_host_retries;	/* UK: retries for above */
-    int uk_greybook_host_interval;	/* UK: interval for above */
+    unsigned int uk_greybook_host_interval; /* UK: interval for above */
     char *uk_suffix;                    /* UK: Suffix for UK addresses */
     int  rewrite_headers;               /* Amount of header rewriting to do */
     int  cname_limit;                   /* Depth of CNAMES allowed */
@@ -133,4 +135,13 @@ struct bindlib_private {
      NULL                               /* char *fallback_gateway */		\
    }
 
+#ifdef HAVE_BIND
+/* external functions defined in bindlib.c */
+extern int bind_check_if_canonical_host __P((char *, unsigned long, struct error **));
+extern int bind_check_if_local_mxs __P((char *, int *, struct error **));
+extern char *bind_lookup_txt_rr __P((char *, struct error **));
+extern int bind_addr __P((char *, long, struct bindlib_private *, char *, struct rt_info *, struct error **));
+extern char * bind_compute_domain __P((void));
+#endif /* HAVE_BIND */
+
 #endif /* not BINDLIB_H */
