@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:config.h,v 1.68 2000/02/11 03:32:37 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:config.h,v 1.75 2001/02/05 23:27:10 woods Exp"
  */
 
 /*
@@ -161,8 +161,7 @@
 /*
  * STRONG SUGGESTIONS
  *
- * The following suggested values should probably not be changed,
- * except to make them less restrictive.
+ * The following suggested values should probably not be changed.
  */
 
 /*
@@ -172,7 +171,28 @@
  * NOTE: This can be set with the config file value "max_message_size"
  */
 #ifndef	MAX_MESSAGE_SIZE
-# define MAX_MESSAGE_SIZE	(100*1024)
+# define MAX_MESSAGE_SIZE (1024*1024)	/* RFC-1123 s.5.3.8 suggests 1mb */
+#endif
+
+/*
+ * The maximum accepted length of an SMTP command line.
+ *
+ * 512 bytes was initially specified in RFC-821 section 4.5.3, page 42
+ *
+ * RFC-1869 section 4.1.2 (page 3) says :
+ *
+ *	This limit is hereby amended to only apply to command lines without any
+ *	parameters.  Each specification that defines new MAIL FROM or RCPT TO
+ *	parameters must also specify maximum parameter value lengths for each
+ *	parameter so that implementors of some set of extensions know how much
+ *	buffer space must be allocated. The maximum command length that must be
+ *	supported by an SMTP implementation with extensions is 512 plus the sum
+ *	of all the maximum parameter lengths for all the extensions supported.
+ *
+ * so we add a generous 512 additional bytes.....
+ */
+#ifndef MAX_SMTP_CMD_LINE_LEN
+# define MAX_SMTP_CMD_LINE_LEN	1024
 #endif
 
 /*
@@ -719,11 +739,7 @@ ${if and{{origin:local} {def:owner}}\
  * SMTP startup banner message
  */
 #ifndef SMTP_BANNER
-# ifdef HAVE_EHLO
-#  define SMTP_BANNER "$primary_name Smail-$version (#$compile_num $compile_date) ready at $date\nESMTP supported"
-# else
-#  define SMTP_BANNER "$primary_name Smail-$version (#$compile_num $compile_date) ready at $date"
-# endif
+# define SMTP_BANNER "$primary_name Smail-$version (#$compile_num $compile_date) ready at $date"
 #endif
 
 /*
@@ -780,6 +796,7 @@ ${if and{{origin:local} {def:owner}}\
  * The default list of networks from which to allow remote relay and
  * which do not need/want RCPT TO address to be verified.
  */
+
 #ifndef SMTP_RECIPIENT_NO_VERIFY
 # define SMTP_RECIPIENT_NO_VERIFY	NULL
 #endif
@@ -787,15 +804,25 @@ ${if and{{origin:local} {def:owner}}\
 /*
  * The default list of networks from which to allow remote relay.
  *
- * NOTE: the keyword "localnet" is replaced by the result of
+ * NOTE: the keyword "localnet" is effectively replaced by the result of
  * inet_netof(local_sa.sin_addr) [where local_sa is filled by
  * getsockname(fileno(in)) for each connection].  This gives effectively
  * a wildcard pattern that matches a classic IP network for the local
  * address of the incoming connection.
+ *
+ * NOTE: the keyword "whoson" causes a WHOSON lookup to be done and a
+ * positive match is made if the WHOSON server lists the client address.
+ *
+ * #define SMTP_REMOTE_ALLOW		"whoson:localnet" (* special *)
+ *
  */
 
 #ifndef SMTP_REMOTE_ALLOW
-# define SMTP_REMOTE_ALLOW		"localnet" /* special */
+# if defined(HAVE_LIBWHOSON)
+#   define SMTP_REMOTE_ALLOW		"whoson" /* special */
+# else
+#  define SMTP_REMOTE_ALLOW		NULL
+# endif
 #endif
 
 /*
@@ -814,6 +841,22 @@ ${if and{{origin:local} {def:owner}}\
 # define SMTP_HELLO_BROKEN_ALLOW	NULL 	/* none by default */
 #endif
 
+/*
+ * Whether or not we reject if PTR names don't have matching addresses
+ */
+
+#ifndef SMTP_HELLO_REJECT_DNS_PARANOID
+# define SMTP_HELLO_REJECT_DNS_PARANOID	1 	/* definitely! */
+#endif
+
+/*
+ * A list of hosts/networks which we deny outright!
+ */
+
+#ifndef SMTP_HELLO_REJECT_HOSTS
+# define SMTP_HELLO_REJECT_HOSTS	NULL 	/* none by default */
+#endif
+
 /*
  * Whether or not we verify HELO/EHLO domain literal has PTR RR
  */
@@ -830,6 +873,25 @@ ${if and{{origin:local} {def:owner}}\
 # define SMTP_HELLO_VERIFY_PTR		0 	/* XXX not many will yet... */
 #endif
 
+/*
+ * The default list of RBLs
+ */
+#ifndef SMTP_RBL_DOMAINS
+# define SMTP_RBL_DOMAINS		NULL
+#endif
+
+/*
+ * The default list of networks which will not trigger an RBL lookup.
+ */
+
+#ifndef SMTP_RBL_EXCEPT
+# if defined(HAVE_LIBWHOSON)
+#  define SMTP_RBL_EXCEPT		"localnet:whoson" /* special */
+# else
+#  define SMTP_RBL_EXCEPT		"localnet" /* special */
+# endif
+#endif
+
 /*
  * The default list of networks from which do not need/want the MAIL FROM:
  * address to be verified.
@@ -1039,7 +1101,7 @@ ${if and{{origin:local} {def:owner}}\
 /*
  * Set HOST_LOCK_TIMEOUT to the default timeout, in seconds, for
  * use when trying to lock a retry file for exclusive delivery
- * to a given host.
+ * to a given host.  Used only on the initial delivery, not by runq.
  */
 
 #ifndef HOST_LOCK_TIMEOUT
@@ -1465,7 +1527,10 @@ ${if and{{origin:local} {def:owner}}\
  * #define ALIASES_FILE	"/usr/smail/lib/aliases" (* alternate *)
  */
 #if	!defined(ALIASES_FILE) && !defined(NO_ALIASES_FILE)
-# define ALIASES_FILE "/usr/lib/aliases" /* sendmail compatible name */
+# define ALIASES_FILE	"/usr/lib/aliases" /* sendmail compatible name */
+#endif
+#if	!defined(ALIASES_OWNER) && !defined(NO_ALIASES_OWNER)
+# define ALIASES_OWNER	"owner-${lc:user}" /* sendmail compatible prefix */
 #endif
 /* NOTE: this is derrived from ALIASES_TYPE in conf/EDITME by conf/lib/mkdefs.sh */
 #ifndef	ALIASES_PROTO
@@ -1491,9 +1556,7 @@ ${if and{{origin:local} {def:owner}}\
  * host may be down.  To enable this behavior, define the name below.
  */
 #if	!defined(ALIASES_TRYAGAIN) && !defined(NO_ALIASES_TRYAGAIN)
-# if defined(HAVE_YP) || defined(HAVE_NISPLUS)
-#  define ALIASES_TRYAGAIN		/*  */
-# endif
+# define ALIASES_TRYAGAIN		/*  */
 #endif
 
 /*
@@ -1516,7 +1579,7 @@ ${if and{{origin:local} {def:owner}}\
  * .forward files found in these places.
  */
 #ifndef REMOTE_HOMES
-# define REMOTE_HOMES "~nuucp:~uucp:~ftp:/tmp:/usr/tmp"
+# define REMOTE_HOMES "~nuucp:~uucp:~ftp:/tmp:/usr/tmp:/var/tmp"
 #endif
 
 /*
@@ -1570,14 +1633,14 @@ ${if and{{origin:local} {def:owner}}\
  * relative to the smail_lib_dir directory.
  *
  * #define LIST_FILENAME "/usr/smail/lists/${lc:user}" (* alternative *)
- * #define LIST_OWNER_FILENAME "/usr/smail/lists/owner-${lc:user}" (* alternative *)
+ * #define LIST_OWNER    "/usr/smail/lists/owner-${lc:user}" (* alternative *)
  */
 #if	!defined(LIST_FILENAME) && !defined(NO_LIST_FILENAME)
-# define LIST_FILENAME "lists/${lc:user}"	/* suggested pathname */
-# define LIST_OWNER_FILENAME "owner-${lc:user}"	/* suggested pathname */
+# define LIST_FILENAME	"lists/${lc:user}"	/* suggested pathname */
+# define LIST_OWNER	"owner-${lc:user}"	/* suggested prefix */
 #endif
-#if defined(LIST_FILENAME) && !defined(LIST_OWNER_FILENAME)
-# include "CONFIG_ERROR: LIST_OWENR_FILENAME must be defined with LIST_FILENAME."
+#if defined(LIST_FILENAME) && !defined(LIST_OWNER)
+# include "CONFIG_ERROR: LIST_OWNER must be defined with LIST_FILENAME."
 #endif
 
 /*
@@ -1867,7 +1930,7 @@ ${if and{{origin:local} {def:owner}}\
 #endif
 
 /*
- * The force_paths file is always considered optional, so that if
+ * The force_paths file is usually considered optional, so that if
  * the file does not exist, it is assumed to be empty.
  */
 #if	!defined(FORCE_PATHS_OPTIONAL) && !defined(NO_FORCE_PATHS_OPTIONAL)
@@ -1881,7 +1944,7 @@ ${if and{{origin:local} {def:owner}}\
  * host may be down.  To enable this behavior, define the name below.
  */
 #if	!defined(FORCE_PATHS_TRYAGAIN) && !defined(NO_FORCE_PATHS_TRYAGAIN)
-/* # define FORCE_PATHS_TRYAGAIN	(* you probably don't want this! *) */
+# define FORCE_PATHS_TRYAGAIN			/*  */
 #endif
 
 /*
@@ -1925,7 +1988,7 @@ ${if and{{origin:local} {def:owner}}\
 #endif
 
 /*
- * The force_smtp file is always considered optional, so that if
+ * The force_smtp file is usually considered optional, so that if
  * the file does not exist, it is assumed to be empty.
  */
 #if	!defined(FORCE_SMTP_OPTIONAL) && !defined(NO_FORCE_SMTP_OPTIONAL)
@@ -1939,7 +2002,7 @@ ${if and{{origin:local} {def:owner}}\
  * host may be down.  To enable this behavior, define the name below.
  */
 #if	!defined(FORCE_SMTP_TRYAGAIN) && !defined(NO_FORCE_SMTP_TRYAGAIN)
-/* # define FORCE_SMTP_TRYAGAIN		(* you probably don't want this! *) */
+# define FORCE_SMTP_TRYAGAIN			/*  */
 #endif
 
 /*
@@ -1983,11 +2046,11 @@ ${if and{{origin:local} {def:owner}}\
 #endif
 
 /*
- * The rewrite file is always considered optional, so that if
+ * The rewrite file is usuall considered optional, so that if
  * the file does not exist, it is assumed to be empty.
  */
 #if	!defined(REWRITE_OPTIONAL) && !defined(NO_REWRITE_OPTIONAL)
-# define REWRITE_OPTIONAL		/*  */
+# define REWRITE_OPTIONAL			/*  */
 #endif
 
 /*
@@ -1997,39 +2060,5 @@ ${if and{{origin:local} {def:owner}}\
  * host may be down.  To enable this behavior, define the name below.
  */
 #if	!defined(REWRITE_TRYAGAIN) && !defined(NO_REWRITE_TRYAGAIN)
-/* # define REWRITE_TRYAGAIN		(* you probably don't want this! *) */
+# define REWRITE_TRYAGAIN			/*  */
 #endif
-
-#ifdef HAVE_LIBWRAP
-
-# include <syslog.h>			/* XXX gross nested include hack */
-
-# ifndef LIBWRAP_DEFAULT_FACILITY
-#   define LIBWRAP_DEFAULT_FACILITY	LOG_MAIL /* for openlog() */
-#  endif
-
-# ifndef LIBWRAP_ALLOW_FACILITY
-#   define LIBWRAP_ALLOW_FACILITY	LOG_AUTH
-#  endif
-
-#  ifndef LIBWRAP_ALLOW_SEVERITY
-#   define LIBWRAP_ALLOW_SEVERITY	LOG_INFO
-#  endif
-
-#  ifndef LIBWRAP_DENY_FACILITY
-#   define LIBWRAP_DENY_FACILITY	LOG_AUTH
-#  endif
-
-#  ifndef LIBWRAP_DENY_SEVERITY
-#   define LIBWRAP_DENY_SEVERITY	LOG_NOTICE
-#  endif
-
-/*
- * Used to set the undocumented hosts_access_verbose logging flag.
- * "2" causes matches to be syslog'ed at LOG_DEBUG with file & line numbers
- */
-# ifndef LIBWRAP_MATCH_VERBOSE
-#  define LIBWRAP_MATCH_VERBOSE		2
-# endif
-
-#endif /* HAVE_LIBWRAP */
