@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:debug.h,v 1.13 1999/09/19 23:15:16 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:debug.h,v 1.16 2000/09/29 21:18:39 woods Exp"
  */
 
 /*
@@ -49,8 +49,38 @@
 # define MALLOC_VERIFY(fullcheck)	mal_verify(fullcheck)
 #endif
 
+/*
+ * The Vmalloc package builds with a debugging version that must be linked with
+ * '-lvmalloc' prior to including '-lc'.
+ *
+ * Add '-DVMFL' to enable printing of source filenames and line numbers in
+ * trace and debug output.
+ *
+ * Debugging is turned on by setting the environment variable "VMETHOD=Vmdebug".
+ *
+ * Debugging can cause an abort() if "VMDEBUG=a" is set.  See vmalloc(3) for
+ * more info.
+ *
+ * Full tracing can be turned on by setting "VMTRACE=/tmp/vmalloc.trace.%p"
+ * (where the '%p' is replaced by the process-ID of the running process)
+ *
+ * Full profiling can be turned on by setting "VMETHOD=Vmprofile" and
+ * "VMPROFILE=/tmp/vmalloc.profile.%p" (where the '%p' is replaced by the
+ * process-ID of the running process)
+ */
+#ifdef USE_VMALLOC
+# define MALLOC_DEBUG_OFF		0
+# define MALLOC_DEBUG_NORMAL		0
+# define MALLOC_DEBUG_FULL		0
+# define MALLOC_DEBUG_EXTREME		0
+# define MALLOC_DEBUG(level)		/*  */
+# define MALLOC_VERIFY_NORMAL		0
+# define MALLOC_VERIFY_FULLCHECK	0
+# define MALLOC_VERIFY(fullcheck)	vmdbcheck(Vmheap)
+#endif
+
 /* XXX could also use "ifndef MALLOC_DEBUG_OFF" or some such... */
-#if !defined(USE_SUNOS4_MALLOC_DEBUG) && !defined(USE_MORAES_MALLOC_DEBUG)
+#if !defined(USE_SUNOS4_MALLOC_DEBUG) && !defined(USE_MORAES_MALLOC_DEBUG) && !defined(USE_VMALLOC)
 # define MALLOC_DEBUG_OFF		0
 # define MALLOC_DEBUG_NORMAL		0
 # define MALLOC_DEBUG_FULL		0
@@ -70,9 +100,9 @@
 #endif
 
 /*
- * define a macro hook to ease insertion of verifies in everywhere
+ * define a macro hook to ease insertion of verifies in every DEBUG*() macro
  */
-#ifdef MALLOC_DEBUG_EVERY_DEBUG
+#ifdef MALLOC_VERIFY_EVERY_DEBUG
 # define MALLOC_DEBUG_HOOK(fullcheck)	MALLOC_VERIFY(fullcheck)
 #else
 # define MALLOC_DEBUG_HOOK(fullcheck)	/* No-OP */
@@ -210,7 +240,7 @@
 /* debug levels for functions in spool.c */
 #define DBG_SPOOL_HI	50
 #define DBG_SPOOL_MID	5
-#define DBG_SPOOL_LO	2
+#define DBG_SPOOL_LO	1
 
 /* debug levels for functions in header.c */
 #define DBG_HEADER_HI	50
@@ -230,7 +260,7 @@
 /* debug levels for main code in main.c */
 #define DBG_MAIN_HI	50
 #define DBG_MAIN_MID	8
-#define DBG_MAIN_LO	2
+#define DBG_MAIN_LO	1
 
 /* debug levels for notification code in notify.c */
 #define DBG_NOTIFY_HI	50
