@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:default.c,v 1.50 1999/02/03 05:25:29 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:default.c,v 1.54 2001/02/05 23:27:11 woods Exp"
  */
 
 /*
@@ -19,13 +19,19 @@
  * by `config.h' or the `startup' file.
  */
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <stdio.h>
 #include "defs.h"
 #include "config.h"
 #include "smail.h"
+#include "main.h"
+#include "parse.h"
+#include "addr.h"
 #include "direct.h"
 #include "route.h"
 #include "transport.h"
+#include "alloc.h"
+#include "dys.h"
 #if defined(HAVE_BIND) && defined(USE_BIND)
 # include "bindlib.h"
 #endif
@@ -42,6 +48,7 @@
 #include "routers/pathalias.h"
 #include "routers/smarthost.h"
 #include "routers/reroute.h"
+#include "routers/rewrite.h"
 #include "transports/pipe.h"
 #include "transports/appendfile.h"
 #include "transports/tcpsmtp.h"
@@ -127,7 +134,7 @@ static struct director list_director = {
     SENDER_OKAY |
 #endif
 	CAUTION_DIRECTOR,
-    LIST_OWNER_FILENAME,		/* normal address owner owners */
+    LIST_OWNER,				/* normal address owner owners */
     NULL,				/* no default user */
     NULL,				/* no default group */
     NULL,				/* no default home director */
@@ -391,7 +398,7 @@ static struct director aliases_director = {
 	ALIAS_TRYAGAIN |
 #endif
 	NOBODY_DIRECTOR,		/* secure source of addresses */
-    NULL,				/* no address owner */
+    ALIASES_OWNER,			/* default owner */
     NULL,				/* no default user */
     NULL,				/* no default group */
     NULL,				/* no default home director */
@@ -461,7 +468,7 @@ static struct director aliasinclude_director = {
     SUCC,				/* point to next director */
     COPY_SECURE | COPY_OWNERS |		/* restrictions from aliases */
 	NOBODY_DIRECTOR,		/*  */
-    NULL,				/* no address owner */
+    ALIASES_OWNER,			/* default owner */
     NULL,				/* no default user */
     NULL,				/* no default group */
     NULL,				/* no default home director */
@@ -572,8 +579,8 @@ static struct uuname_private uuname_router_priv = {
     NULL,				/* internal - cached output from cmd */
     NULL,				/* internal - end of cached output */
     NULL,				/* internal - processing error */
-    NULL,				/* internal - st_ino from stat */
-    NULL,				/* internal - st_mtime from stat */
+    0,					/* internal - st_ino from stat */
+    0,					/* internal - st_mtime from stat */
 };
 
 static struct router uuname_router = {
@@ -763,29 +770,13 @@ static struct router reroute_router = {
  * defined in config.h, if a rewrite database is configured.
  */
 
-static struct pathalias_private rewrite_router_priv = {
+static struct rewrite_private rewrite_router_priv = {
     REWRITE_FILE,			/* file attribute */
     REWRITE_PROTO,			/* file access method */
     NULL,				/* default domain names */
     NULL,				/* no required domain names */
-#ifdef	HAVE_RENAME
-    /*
-     * if we have an atomic rename call, retries should not be required
-     * since the file should always exist, even when being rebuilt
-     *
-     * if the file is not found, the pathalias director is ignored.
-     */
-    0,					/* no retries */
-#else	/* HAVE_RENAME */
-    /*
-     * without an atomic rename call, there will be short periods
-     * of time when the file does not exist, after it is has been
-     * rebuilt between when it is removed and the new one is put
-     * into place.  (Assuming one doesn't just edit it in place!)
-     */
     2,					/* two retries on open */
-#endif	/* HAVE_RENAME */
-    2,					/* two second intervals for retries */
+    10,					/* ten second intervals for retries */
     NULL,				/* internal - open database */
     NULL,				/* internal - error text from open */
 };
@@ -796,12 +787,12 @@ static struct router rewrite_router = {
     SUCC,				/* point to next router */
     USE_ALWAYS |			/* we generally want to force rewrite on a match */
 #ifdef REWRITE_OPTIONAL
-	PA_OPTIONAL |			/* the rewrite file is optional */
+	RW_OPTIONAL |			/* the rewrite file is optional */
 #endif
 #ifdef REWRITE_TRYAGAIN
-	PA_TRYAGAIN |			/* defer address on open failure */
+	RW_TRYAGAIN |			/* defer address on open failure */
 #endif
-	PA_REOPEN,			/* always reopen database to search */
+	RW_REOPEN,			/* always reopen database to search */
     METHOD_TABLE,			/* use method table to find tport */
     NULL,				/* no default transport */
     (char *) &rewrite_router_priv,	/* rewrite private configuration */
