@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:direct.c,v 1.28 2000/01/31 17:04:04 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:direct.c,v 1.31 2000/09/03 17:00:12 woods Exp"
  */
 
 /*
@@ -20,27 +20,28 @@
  *			    find_direct_driver, read_director_file.
  *                          dump_director_config
  */
-#include <stdio.h>
-#include <pwd.h>
-#include <grp.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <stdio.h>
+#include <errno.h>
+#include <pwd.h>
+#include <grp.h>
 #include "defs.h"
 #include "config.h"
 #include "smail.h"
+#include "main.h"
+#include "parse.h"
+#include "addr.h"
 #include "direct.h"
 #include "transport.h"
 #include "log.h"
-#include "addr.h"
-#include "main.h"
+#include "alloc.h"
 #include "dys.h"
 #include "exitcodes.h"
 #include "smailconf.h"
-#ifndef DEPEND
-# include "debug.h"
-# include "extern.h"
-# include "error.h"
-#endif
+#include "debug.h"
+#include "extern.h"
+#include "error.h"
 
 /* enums local to this file */
 enum local_form {
@@ -59,13 +60,15 @@ enum other_form_op {
 int cached_directors = FALSE;		/* TRUE if cache_directors() called */
 
 /* functions local to this file */
-static void implicit_redirect();
-static enum other_form_op finish_direct_other_form();
-static void finish_direct_special_form();
-static void cache_director_data();
-static void premunge_local_addrs();
-static enum local_form addr_type();
-static char *director_driv_function();
+static void implicit_redirect __P((struct addr*, struct addr **, char *));
+static enum other_form_op finish_direct_other_form __P((struct director *, struct addr *));
+static void finish_direct_special_form __P((struct addr *, struct director *, enum local_form,
+					    struct addr **, struct addr **, struct addr **,
+					    struct addr **, struct addr **));
+static void cache_director_data __P((struct director *));
+static void premunge_local_addrs __P((struct addr *));
+static enum local_form addr_type __P((char *));
+static char *director_driv_function __P((char *, struct attribute *));
 
 /* Configuration template information */
 static struct attr_table director_generic[] = {
@@ -184,12 +187,14 @@ direct_local_addrs(in, out, retry, defer, fail)
 		}
 	    }
 
-	    /* call the driver if any left */
+	    /* call the driver if any input addresses are left to direct */
 	    if (in != NULL) {
 		in = (*driver->driver)(dp, in, out, &newaddrs, defer, fail);
 	    }
 
-	    /* put back any that were held back */
+	    /* put back any that were held back because they did not match the
+	     * director's domain list
+	     */
 	    if (other_domains != NULL) {
 		if (in == NULL) {
 		    in = other_domains;
@@ -198,7 +203,9 @@ direct_local_addrs(in, out, retry, defer, fail)
 		}
             }
 
-	    /* step through newaddrs to verify them */
+	    /* step through newaddrs to verify them (newaddrs will be left as
+	     * NULL if the input address(es) had ADDR_VRFY_ONLY set)
+	     */
 	    for (cur = newaddrs; cur; cur = next) {
 		enum local_form form = addr_type(cur->work_addr);
 
@@ -408,11 +415,14 @@ finish_direct_other_form(dp, addr)
     char *temp_work_addr;
     int form;
 
+    if (!addr->parent) {
+	panic(EX_SOFTWARE, "finish_direct_other_form(): director %s: no parent address for %s.", dp->name, addr->in_addr);
+	/* NOTREACHED */
+    }
     /* not an address form we need to be worried about */
     if (EQIC(addr->work_addr, addr->parent->remainder) ||
 	(addr->work_addr[0] == '\\' &&
-	 EQIC(addr->work_addr + 1, addr->parent->remainder)))
-    {
+	 EQIC(addr->work_addr + 1, addr->parent->remainder))) {
 	/* for the form \parent-addr drop the \ */
 	if (addr->work_addr[0] == '\\') {
 	    escaped = 1;
@@ -711,6 +721,7 @@ finish_direct_special_form(addr, dp, form, new, out, defer, fail, includes)
 	panic(EX_SOFTWARE,
 	      "director %s, impossible situation:  reached OTHER_FORM",
 	      dp->name);
+	/* NOTREACHED */
 	break;
     }
 }
@@ -1018,8 +1029,6 @@ read_director_file()
 	NULL,				/* cache - x_set_home */
     };
 
-    extern int errno;
-
     /*
      * try to open directory file, stat file if possible
      */
