@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:direct.h,v 1.8 1998/11/20 01:01:24 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:direct.h,v 1.9 2000/09/03 16:29:01 woods Exp"
  */
 
 /*
@@ -44,12 +44,18 @@ struct director {
 
 /* compiled in director drivers */
 struct direct_driver {
-    char *name;				/* name of director driver */
-    void (*cache)();			/* function to cache director info */
-    struct addr *(*driver)();		/* function to perform directing */
-    void (*finish)();			/* function to free resources */
-    char *(*builder)();			/* fun to read from director file */
-    void (*dumper)();			/* fun to dump configuration */
+    char *name;					/* name of director driver */
+    void (*cache) __P((struct director *));	/* function to cache director info */
+    struct addr *(*driver) __P((struct director *,
+				struct addr *,
+				struct addr **,
+				struct addr **,
+				struct addr **,
+				struct addr **)); /* function to perform directing */
+    void (*finish) __P((struct director *));	/* function to free resources */
+    char *(*builder) __P((struct director *,
+			  struct attribute *)); /* fun to read from director file */
+    void (*dumper) __P((FILE *, struct director *)); /* fun to dump configuration */
 };
 
 /* flags for the director.flags field */
@@ -60,4 +66,14 @@ struct direct_driver {
 #define IGNORE_ALIAS_MATCH	0x0010	/* ignore aliases that match parent */
 #define USER_IGNORE_CASE	0x0020	/* lower user b4 matching w/mailbox */
 
+/* external functions defined in direct.c */
+extern void direct_local_addrs __P((struct addr *, struct addr **, struct addr **, struct addr **, struct addr **));
+extern void director_user_info __P((struct addr *));
+extern struct director *find_director __P((char *));
+extern struct direct_driver *find_direct_driver __P((char *));
+extern void cache_directors __P((void));
+extern void finish_directors __P((void));
+extern char *read_director_file __P((void));
+extern void dump_director_config __P((FILE *));
+
 #endif	/* DIRECT_H */
