@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/directors:RELEASE-3_2_0_111:aliasfile.c,v 1.22 1999/07/22 05:15:22 woods Exp"
+#ident	"@(#)smail/src/directors:RELEASE-3_2_0_112:aliasfile.c,v 1.25 2000/09/03 17:00:21 woods Exp"
  */
 
 /*
@@ -68,20 +68,24 @@
 #include <stdio.h>
 #include <pwd.h>
 #include <grp.h>
+#include <errno.h>
 #include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include "defs.h"
 #include "../smail.h"
-#include "../dys.h"
-#include "../smailconf.h"
+#include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../field.h"
 #include "../log.h"
 #include "../direct.h"
+#include "../transport.h"
 #include "../exitcodes.h"
 #include "../lookup.h"
+#include "../alloc.h"
+#include "../dys.h"
+#include "../smailconf.h"
 #include "aliasfile.h"
 #ifndef DEPEND
 # include "../extern.h"
@@ -90,7 +94,7 @@
 #endif
 
 /* functions local to this file */
-static int alias_secure();
+static int alias_secure __P((struct aliasfile_private *, struct stat *));
 
 static struct attr_table aliasfile_attributes[] = {
     { "file", t_string, NULL, NULL, OFFSET(aliasfile_private, file) },
@@ -285,7 +289,7 @@ dtd_aliasfile(dp, in, out, new, defer, fail)
 	    struct addr *next_list;
 
 	case DB_NOMATCH:
-	    DEBUG2(DBG_DRIVER_MID, "director <%s> did not match name <%s>\n",
+	    DEBUG2(DBG_DRIVER_MID, "director %s: did not match name <%s>\n",
 		   dp->name, cur->remainder);
 	    cur->succ = pass;
 	    pass = cur;
@@ -345,7 +349,7 @@ dtd_aliasfile(dp, in, out, new, defer, fail)
 					cur->remainder)) ||
 		!(preparsed_owner = preparse_address(owner, &error)))
 	    {
-		DEBUG1(DBG_DRIVER_HI, "dtd_aliasfile: no owner set for %s.\n", value);
+		DEBUG1(DBG_DRIVER_HI, "dtd_aliasfile: no owner set for %s.\n", cur->remainder);
 		owner = NULL;
 	    } else {
 		/* 
@@ -361,43 +365,46 @@ dtd_aliasfile(dp, in, out, new, defer, fail)
          
 		if (strcmpic(preparsed_owner, "postmaster") != 0) { 
 		    struct addr *new2 = alloc_addr();
-		    struct addr *defer2 = alloc_addr();
-		    struct addr *fail2_safe = alloc_addr();
-		    struct addr *fail2;
+		    struct addr *defer2 = NULL;
+		    struct addr *fail2 = NULL;
 		    struct addr *okay = NULL;
-		    int was_reopen = (dp->flags & ALIAS_REOPEN);
+		    int was_reopen = (dp->flags & ALIAS_REOPEN); /* remember... */
 
 		    DEBUG1(DBG_DRIVER_MID, "dtd_aliasfile: owner is not postmaster (is '%s'), will attempt to verify.\n", preparsed_owner);
 
-		    dp->flags &= ~ALIAS_REOPEN;	/* make sure db not closed! needed again! */
+		    dp->flags &= ~ALIAS_REOPEN;	/* make sure db is not closed! needed again! */
             
-		    fail2 = fail2_safe;	/* verify_addr_list can clobber its
-					 * last arg and we need to save it so
-					 * we can free it when we're done
-					 */
 		    new2->in_addr = preparsed_owner;
 		    new2->work_addr = preparsed_owner;
             
 		    /* disable use of smartuser driver */
-		    new2->flags = ADDR_SMARTUSER;
+		    new2->flags = ADDR_SMARTUSER | ADDR_VRFY_ONLY;
             
 		    resolve_addr_list(new2, &okay, &defer2, &fail2, FALSE);
 		    if (!okay) {
 			xfree(owner);
 			owner = NULL;
 		    }
-		    xfree(new2);
-		    xfree(defer2);
-		    xfree(fail2_safe);
+		    xfree((char *) new2);
 		    if (was_reopen) {
 			dp->flags |= ALIAS_REOPEN;
 		    }
 		}     
 	    }
-            DEBUG4(DBG_DRIVER_LO,
-		   "director %s: matched %s, aliased to %s, \n  owner %s\n",
+            DEBUG5(DBG_DRIVER_LO,
+		   "director %s: matched %s, aliased to %s, \n  owner %s%s\n",
 		   dp->name, cur->remainder, value,
-		   owner ? owner : "(none)");
+		   owner ? owner : "(none)", cur->flags & ADDR_VRFY_ONLY ? " (VRFY_ONLY)" : "");
+	    /*
+	     * if we are only verifying deliverability then we don't want to
+	     * actually transform anything just yet so push this one onto the
+	     * output list and go on to the next one....
+	     */
+	    if (cur->flags & ADDR_VRFY_ONLY) {
+		cur->succ = *out;
+		*out = cur;
+		break;
+	    }
 	    /*
 	     * Now we can check the process_field results...
 	     */
