@@ -1,4 +1,6 @@
-/* @(#) aliasfile.h,v 1.3 1992/07/11 11:48:29 tron Exp */
+/*
+#ident	"@(#)smail/src/directors:RELEASE-3_2_0_112:aliasfile.h,v 1.4 2000/09/03 16:06:30 woods Exp"
+ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -28,8 +30,17 @@ struct aliasfile_private {
     char *owners;			/* ownership restrictions */
     char *owngroups;			/* group ownership restrictions */
     int retries;			/* max retries on open */
-    int interval;			/* retry interval (seconds) */
+    unsigned int interval;		/* retry interval (seconds) */
     int flags_set;			/* temp - bits to set in addr.flags */
     char *database;			/* temp - open database */
     char *error_text;			/* temp - error from open */
 };
+
+extern struct addr *dtd_aliasfile __P((struct director *,
+				       struct addr *,
+				       struct addr **,
+				       struct addr **,
+				       struct addr **,
+				       struct addr **));
+extern char *dtb_aliasfile __P((struct director *, struct attribute *));
+extern void dtp_aliasfile __P((FILE *, struct director *));
