@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/directors:RELEASE-3_2_0_111:altuser.c,v 1.7 1998/11/20 01:00:02 woods Exp"
+#ident	"@(#)smail/src/directors:RELEASE-3_2_0_112:altuser.c,v 1.10 2000/09/03 17:00:21 woods Exp"
  */
 
 /*
@@ -65,12 +65,12 @@
  *	    tryagain:  if set, then if the open fails, try again on a
  *		later spool directory queue run.
  */
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <stdio.h>
 #include <pwd.h>
 #include <grp.h>
 #include <ctype.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include "defs.h"
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
@@ -84,21 +84,22 @@
 #endif	/* not POSIX_OS */
 
 #include "../smail.h"
-#include "../dys.h"
-#include "../smailconf.h"
+#include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../field.h"
 #include "../log.h"
 #include "../direct.h"
+#include "../transport.h"
+#include "../alloc.h"
+#include "../dys.h"
+#include "../smailconf.h"
 #include "../exitcodes.h"
 #include "../lookup.h"
 #include "altuser.h"
-#ifndef DEPEND
-# include "../extern.h"
-# include "../error.h"
-# include "../debug.h"
-#endif
+#include "../extern.h"
+#include "../error.h"
+#include "../debug.h"
 
 #undef P_
 #ifdef ANSI_C
@@ -290,7 +291,7 @@ dtd_altuser(dp, in, out, new, defer, fail)
 	switch (ret) {
 
 	case DB_NOMATCH:
-	    DEBUG2(DBG_DRIVER_MID, "director <%s> did not match name <%s>\n",
+	    DEBUG2(DBG_DRIVER_MID, "director %s: did not match name %s\n",
 		   dp->name, cur->remainder);
 	    cur->succ = pass;
 	    pass = cur;
@@ -363,15 +364,21 @@ dtd_altuser(dp, in, out, new, defer, fail)
 		    *defer = cur;
 		    continue;
 		}
-
-		cur->flags |= ADDR_ISUSER;
-		cur->flags &= ~ADDR_NOTUSER;
-		cur->uid = altuser_uid;
-		cur->gid = altuser_gid;
-		cur->home = altuser_home;
-		cur->succ = *out;
 		cur->director = dp;		/* set the director which matched */
-		cur->next_addr = COPY_STRING(cur->remainder);
+
+		/*
+		 * if we are only verifying deliverability then we don't want
+		 * to actually transform anything just yet...
+		 */
+		if (!(cur->flags & ADDR_VRFY_ONLY)) {
+		    cur->flags |= ADDR_ISUSER;
+		    cur->flags &= ~ADDR_NOTUSER;
+		    cur->uid = altuser_uid;
+		    cur->gid = altuser_gid;
+		    cur->home = altuser_home;
+		    cur->next_addr = COPY_STRING(cur->remainder);
+		}
+		cur->succ = *out;
 		*out = cur;
 	    } else {
 		/*
