@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/directors:RELEASE-3_2_0_111:altuser.h,v 1.1 1996/05/29 18:48:57 woods Exp"
+#ident	"@(#)smail/src/directors:RELEASE-3_2_0_112:altuser.h,v 1.2 2000/09/03 16:07:11 woods Exp"
  */
 
 /*
@@ -29,9 +29,18 @@ struct altuser_private {
     char *owners;			/* ownership restrictions */
     char *owngroups;			/* group ownership restrictions */
     int retries;			/* max retries on open */
-    int interval;			/* retry interval (seconds) */
+    unsigned int interval;		/* retry interval (seconds) */
     int flags_set;			/* temp - bits to set in addr.flags */
     char *database;			/* temp - open database */
     char *transport;			/* name of the transport */
     char *error_text;			/* temp - error from open */
 };
+
+extern struct addr *dtd_altuser __P((struct director *,
+				     struct addr *,
+				     struct addr **,
+				     struct addr **,
+				     struct addr **,
+				     struct addr **));
+extern char *dtb_altuser __P((struct director *, struct attribute *));
+extern void dtp_altuser __P((FILE *, struct director *));
