@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/directors:RELEASE-3_2_0_111:fwdfile.c,v 1.32 1999/11/26 03:38:52 woods Exp"
+#ident	"@(#)smail/src/directors:RELEASE-3_2_0_112:fwdfile.c,v 1.36 2000/09/03 17:00:21 woods Exp"
  */
 
 /*
@@ -75,38 +75,48 @@
  *	       `checkowner' is set, no checks are made for ownership
  *	       restrictions.
  */
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <stdio.h>
+#include <errno.h>
 #include <ctype.h>
 #include <pwd.h>
 #include <grp.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include "defs.h"
 #ifdef UNIX_BSD
 # include <sys/file.h>
 #endif
+#ifdef HAVE_STDLIB_H
+# include <stdlib.h>
+#endif
 #include "../smail.h"
-#include "../smailconf.h"
+#include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../field.h"
 #include "../log.h"
 #include "../direct.h"
+#include "../alloc.h"
 #include "../dys.h"
 #include "../exitcodes.h"
-#include "../main.h"
+#include "../smailconf.h"
 #include "fwdfile.h"
-#ifndef DEPEND
-# include "../extern.h"
-# include "../error.h"
-# include "../debug.h"
+#include "../extern.h"
+#include "../error.h"
+#include "../debug.h"
+
+#ifndef HAVE_STDLIB_H
+long atol __P((char *));
 #endif
 
-long atol();
-
-static int finish_forward();
-static int forward_secure();
-static int secure_match();
+static int finish_forward __P((struct director *,
+			       struct addr *,
+			       struct addr **,
+			       struct addr **,
+			       struct addr **,
+			       struct addr **));
+static int forward_secure __P((struct director *, struct addr *, struct stat *, char *));
+static int secure_match __P((char *, char *, char *));
 
 static struct attr_table forwardfile_attributes[] = {
     { "file", t_string, NULL, NULL, OFFSET(forwardfile_private, file) },
@@ -172,7 +182,7 @@ dtd_forwardfile(dp, in, out, new, defer, fail)
 	director_user_info(cur);
 
 	/* finish up */
-	if (finish_forward(dp, cur, new, defer, fail) == FAIL) {
+	if (finish_forward(dp, cur, out, new, defer, fail) == FAIL) {
 	    /* if there was no forward file, pass it to the next director */
 	    DEBUG2(DBG_DRIVER_MID, "director %s did not match name %s\n",
 		   dp->name, cur->remainder);
@@ -240,9 +250,10 @@ dtp_forwardfile(f, dp)
  * be properly filled in.
  */
 static int
-finish_forward(dp, addr, new, defer, fail)
+finish_forward(dp, addr, out, new, defer, fail)
     struct director *dp;		/* director entry */
     struct addr *addr;			/* specific addr structure */
+    struct addr **out;			/* output resolved addrs */
     struct addr **new;			/* output new addrs to resolve */
     struct addr **defer;		/* addrs to defer to a later time */
     struct addr **fail;			/* unresolvable addrs */
@@ -263,12 +274,13 @@ finish_forward(dp, addr, new, defer, fail)
     char *owner = NULL;                 /* owner address */
     char *preparsed_owner = NULL;       /* preparsed version */
 
-    if (!(priv = (struct forwardfile_private *)dp->private))
+    if (!(priv = (struct forwardfile_private *)dp->private)) {
 	priv = &forwardfile_template;
+    }
 
     /* remove prefix and suffix from the user name */
     if (priv->prefix) {
-	int len = strlen(priv->prefix);
+	unsigned int len = strlen(priv->prefix);
 
 	if (strncmpic(priv->prefix, remainder, len) == 0) {
 	    remainder += len;
@@ -304,12 +316,7 @@ finish_forward(dp, addr, new, defer, fail)
 	       dp->name, priv->file);
 	return FAIL;
     }
-    if (fn[0] == '/') {
-	fn = COPY_STRING(fn);
-    } else {
-	fn = make_lib_fn(fn);
-    }
-
+    fn = make_lib_fn(fn);
     if (fn == NULL) {
 	/*
 	 * ERR_114 - forward file not an absolute path, and no
@@ -358,6 +365,21 @@ finish_forward(dp, addr, new, defer, fail)
 	xfree(fn);
 	return FAIL;
     }
+    /*
+     * if we are only verifying deliverability then we don't want to actually
+     * transform anything just yet and local config errors are not important
+     * just yet so just push this one onto the "resolved addresses" list and be
+     * done.
+     */
+    if (addr->flags & ADDR_VRFY_ONLY) {
+	/* matched a forward file */
+	DEBUG2(DBG_DRIVER_LO, "director %s: matched %s (VRFY_ONLY)\n", dp->name, addr->remainder);
+	(void) fclose(fp);
+	xfree(fn);
+	addr->succ = *out;
+	*out = addr;
+	return SUCCEED;
+    }
 
     /* set security bits appropriately */
     flags |= forward_secure(dp, addr, &statbuf, fn);
@@ -386,18 +408,22 @@ finish_forward(dp, addr, new, defer, fail)
 	    STR_FREE(&fwdtobuf);
 	    return FAIL;
 	}
-	(void) fclose(fp);
+
 	STR_NEXT(&fwdtobuf, '\0');
 	if (strncmpic(STR(&fwdtobuf), "forward to ",
 		      sizeof("forward to ") - 1) != 0) {
 	    STR_FREE(&fwdtobuf);
+	    (void) fclose(fp);
 	    xfree(fn);
 	    return FAIL;
 	}
 
 	buf = STR(&fwdtobuf) + sizeof("forward to ") - 1;
     } else {
-	buf = xmalloc(statbuf.st_size + 1); /* leave room for trailing nul */
+	unsigned int fsize;
+
+	fsize = statbuf.st_size;
+	buf = xmalloc(fsize + 1); /* leave room for trailing nul */
 
 	if (fread(buf, (size_t) statbuf.st_size, 1, fp) < 1) {
 	    write_log(WRITE_LOG_SYS, "read failed on %s, director=%s", fn, dp->name);
@@ -406,12 +432,11 @@ finish_forward(dp, addr, new, defer, fail)
 	    xfree(fn);
 	    return FAIL;
 	}
-
-	(void) fclose(fp);		/* don't need the open file anymore */
-
 	buf[statbuf.st_size] = '\0';	/* terminate the buffer */
 	DEBUG1(DBG_DRIVER_HI, "read %ld bytes\n", (long) statbuf.st_size);
     }
+    (void) fclose(fp);		/* don't need the open file anymore */
+    xfree(fn);
 
     /* matched a forward file */
     DEBUG2(DBG_DRIVER_LO, "director %s: matched %s",
@@ -461,8 +486,8 @@ finish_forward(dp, addr, new, defer, fail)
      */
     if (cur == NULL) {
 	DEBUG2(DBG_DRIVER_LO,
-	      "director %s: file for %s has no addresses, ignored\n",
-	      dp->name, addr->in_addr);
+	       "director %s: file for %s has no addresses, ignored\n",
+	       dp->name, addr->in_addr);
 	return FAIL;
     }
 
@@ -477,8 +502,8 @@ finish_forward(dp, addr, new, defer, fail)
 				remainder)) ||
         !(preparsed_owner = preparse_address(owner, &error)))
     {
+	DEBUG1(DBG_DRIVER_HI, "dtd_forwardfile: no owner set for %s.\n", addr->remainder);
         owner = NULL;
-
     } else {
 	/* Must verify that this owner is sensible (to avoid things like
 	 * xxxx-request-request).  To do this, use the pre-parsed address.
@@ -487,31 +512,31 @@ finish_forward(dp, addr, new, defer, fail)
 	 * non-standard local domain name for list handling.  So make a copy
 	 * beforehand.
 	 */
-	struct addr *new2 = alloc_addr();
-	struct addr *defer2 = alloc_addr();
-	struct addr *fail2_safe = alloc_addr();
-	struct addr *fail2;
-	struct addr *okay = NULL;
-
-	fail2 = fail2_safe;		/* verify_addr_list can clobber last arg */
 	owner = COPY_STRING(preparsed_owner); /* copy to use later */
 
-	new2->in_addr = preparsed_owner;
-	new2->work_addr = preparsed_owner;
+	if (strcmpic(preparsed_owner, "postmaster") != 0) { 
+	    struct addr *new2 = alloc_addr();
+	    struct addr *defer2 = NULL;
+	    struct addr *fail2 = NULL;
+	    struct addr *okay = NULL;
 
-	/* disable use of smartuser driver */
-	new2->flags = ADDR_SMARTUSER;
+	    DEBUG1(DBG_DRIVER_MID, "dtd_forwardfile: owner is not postmaster (is '%s'), will attempt to verify.\n", preparsed_owner);
 
-	resolve_addr_list(new2, &okay, &defer2, &fail2, FALSE);
+	    new2->in_addr = preparsed_owner;
+	    new2->work_addr = preparsed_owner;
 
-	if (!okay) {
-	    xfree(owner);
-	    owner = NULL;
+	    /* disable use of smartuser driver */
+	    new2->flags = ADDR_SMARTUSER | ADDR_VRFY_ONLY;
+
+	    resolve_addr_list(new2, &okay, &defer2, &fail2, FALSE);
+
+	    if (!okay) {
+		xfree(owner);
+		owner = NULL;
+	    }
+
+	    xfree((char *) new2);
 	}
-
-	xfree(new2);
-	xfree(defer2);
-	xfree(fail2_safe);
     }
 
     /* Set up the newly-created addresses */
@@ -531,10 +556,10 @@ finish_forward(dp, addr, new, defer, fail)
 	*new = cur;
     }
 
-    DEBUG3(DBG_DRIVER_LO, "director <%s> matched <%s> owner <%s>\n",
+    DEBUG3(DBG_DRIVER_LO, "director %s: matched %s, owner %s\n",
            dp->name, addr->remainder,
-           owner? owner : "(none)");
-    xfree(fn);
+           owner ? owner : "(none)");
+
     return SUCCEED;
 }
 
