@@ -1,4 +1,6 @@
-/* @(#) fwdfile.h,v 1.4 1995/03/01 13:47:31 nm4 Exp */
+/*
+#ident	"@(#)smail/src/directors:RELEASE-3_2_0_112:fwdfile.h,v 1.5 2000/09/03 16:26:33 woods Exp"
+ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -7,7 +9,7 @@
  * See the file COPYING, distributed with smail, for restriction
  * and warranty information.
  *
- * namei master id: @(#) fwdfile.h,v 1.4 1995/03/01 13:47:31 nm4 Exp
+ * namei master id: @(#) fwdfile.h,v 1.5 2000/09/03 16:26:33 woods Exp
  */
 
 /*
@@ -31,3 +33,12 @@ struct forwardfile_private {
     char *prefix;			/* prefix required for a match */
     char *suffix;			/* suffix required for a match */
 };
+
+extern struct addr *dtd_forwardfile __P((struct director *,
+					 struct addr *,
+					 struct addr **,
+					 struct addr **,
+					 struct addr **,
+					 struct addr **));
+extern char *dtb_forwardfile __P((struct director *, struct attribute *));
+extern void dtp_forwardfile __P((FILE *, struct director *));
