@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/directors:RELEASE-3_2_0_111:smartuser.c,v 1.11 1998/11/20 01:00:04 woods Exp"
+#ident	"@(#)smail/directors:RELEASE-3_2_0_112:smartuser.c,v 1.14 2000/09/03 17:00:21 woods Exp"
  */
 
 /*
@@ -36,6 +36,7 @@
  *	  driver using only a config file.
  */
 #include <stdio.h>
+#include <errno.h>
 #include <pwd.h>
 #include <grp.h>
 #include <sys/types.h>
@@ -43,14 +44,17 @@
 #include <ctype.h>
 #include "defs.h"
 #include "../smail.h"
-#include "../smailconf.h"
+#include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../field.h"
 #include "../log.h"
 #include "../direct.h"
+#include "../transport.h"
+#include "../alloc.h"
 #include "../dys.h"
 #include "../exitcodes.h"
+#include "../smailconf.h"
 #include "smartuser.h"
 #ifndef DEPEND
 # include "../extern.h"
@@ -59,8 +63,8 @@
 #endif
 
 /* functions local to this file */
-static char *make_well_formed();
-static char *make_passable();
+static char *make_well_formed __P((char *));
+static char *make_passable __P((char *));
 
 static struct attr_table smartuser_attributes[] = {
     { "new_user", t_string, NULL, NULL,
@@ -138,9 +142,20 @@ dtd_smartuser(dp, in, out, new, defer, fail)
 
 	cur->director = dp;		/* match anything, at this point */
 
+	/*
+	 * if we are only verifying deliverability then we don't want to
+	 * actually transform anything just yet so just push this one onto the
+	 * "resolved addresses" list and go on to the next...
+	 */
+	if (cur->flags & ADDR_VRFY_ONLY) {
+	    cur->succ = *out;
+	    *out = cur;
+	    continue;
+	}
+
 	if ((priv->new_user != NULL) || (smart_user != NULL)) {
-	    new_user = expand_string(priv->new_user? priv->new_user: smart_user,
-				     (struct addr *)NULL, (char *)NULL, remainder);
+	    new_user = expand_string(priv->new_user ? priv->new_user : smart_user,
+				     (struct addr *) NULL, (char *) NULL, remainder);
 	} else {
 	    new_user = remainder;
 	}
@@ -160,8 +175,7 @@ dtd_smartuser(dp, in, out, new, defer, fail)
 	     *      new_user attribute.
 	     */
 	    cur->error = note_error(ERR_CONFERR|ERR_120,
-				    xprintf(
-				      "director %s: new_user expansion failed",
+				    xprintf("director %s: new_user expansion failed",
 					    dp->name));
 	    cur->succ = *defer;
 	    *defer = cur;
@@ -299,7 +313,7 @@ make_well_formed(name)
     register char *name;		/* input string */
 {
     static char *result = NULL;		/* region for forming result */
-    static int result_size;		/* allocated region length */
+    static unsigned int result_size;		/* allocated region length */
     register char *p;			/* point to result */
     register int c;			/* char from name string */
 
@@ -307,7 +321,7 @@ make_well_formed(name)
     if (result == NULL) {
 	result = xmalloc(result_size = strlen(name) + 1);
     } else {
-	register int new_size = strlen(name) + 1;
+	register unsigned int new_size = strlen(name) + 1;
 
 	if (new_size > result_size) {
 	    result = xrealloc(result, new_size);
@@ -360,8 +374,8 @@ make_passable(name)
     int need_quotes = FALSE;		/* true if we should put in quotes */
     int esc_ct = 0;			/* count of \'s required */
     static char *result = NULL;		/* region for building result */
-    static int result_size;		/* size of result region */
-    int new_size;			/* minimum size required */
+    static unsigned int result_size;	/* size of result region */
+    unsigned int new_size;		/* minimum size required */
     register char *p;			/* temp for building result */
     register char *q;			/* temp for scanning name */
     register int c;			/* char from name */
