@@ -1,4 +1,6 @@
-/* @(#) smartuser.h,v 1.4 1995/05/30 16:16:29 nm4 Exp */
+/*
+#ident	"@(#)smail/src/directors:RELEASE-3_2_0_112:smartuser.h,v 1.5 2000/09/03 16:26:33 woods Exp"
+ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -22,3 +24,12 @@ struct smartuser_private {
     char *transport;			/* transport if routing to "smart" transport */
     struct transport *transport_ptr;	/* transport structure */
 };
+
+extern struct addr *dtd_smartuser __P((struct director *,
+				       struct addr *,
+				       struct addr **,
+				       struct addr **,
+				       struct addr **,
+				       struct addr **));
+extern char *dtb_smartuser __P((struct director *, struct attribute *));
+extern void dtp_smartuser __P((FILE *, struct director *));
