@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/directors:RELEASE-3_2_0_111:user.c,v 1.15 1998/11/20 01:00:04 woods Exp"
+#ident	"@(#)smail/src/directors:RELEASE-3_2_0_112:user.c,v 1.18 2000/09/03 17:00:22 woods Exp"
  */
 
 /*
@@ -25,17 +25,20 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
+#include <errno.h>
 #include <pwd.h>
 #include "defs.h"
 #include "../smail.h"
-#include "../smailconf.h"
+#include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../log.h"
 #include "../direct.h"
 #include "../transport.h"
 #include "../exitcodes.h"
+#include "../alloc.h"
 #include "../dys.h"
+#include "../smailconf.h"
 #include "user.h"
 #ifndef DEPEND
 # include "../extern.h"
@@ -92,8 +95,9 @@ dtd_user(dp, in, out, new, defer, fail)
     struct addr *next;			/* next value for cur */
     struct user_private *priv;
 
-    if (!(priv = (struct user_private *)dp->private))
+    if (!(priv = (struct user_private *)dp->private)) {
 	priv = &user_template;
+    }
 
     DEBUG(DBG_DRIVER_HI, "dtd_user called\n");
     for (cur = in; cur; cur = next) {
@@ -101,7 +105,7 @@ dtd_user(dp, in, out, new, defer, fail)
 
 	if (priv->prefix) {
 	    struct addr *new2;
-	    int len = strlen(priv->prefix);
+	    unsigned int len = strlen(priv->prefix);
 
 	    /*
 	     * if we are testing against a prefix, strip the prefix and
@@ -116,13 +120,15 @@ dtd_user(dp, in, out, new, defer, fail)
 	    new2 = alloc_addr();
 	    new2->remainder = COPY_STRING(cur->remainder + len);
 #ifdef HAVE_FGETPWENT
-	    if (priv->pwfile)
+	    if (priv->pwfile) {
 		direct_userinfo_pwfile(new2, priv->pwfile);
-	    else
+	    } else
 #endif
+	    {
 		director_user_info(new2);
+	    }
 
-	    if ((new2->flags&ADDR_NOTUSER) || (new2->error)) {
+	    if ((new2->flags & ADDR_NOTUSER) || (new2->error)) {
 		/* we did not match a user on the local host */
 	        cur->error = new2->error;
 		xfree(new2->remainder);
@@ -136,10 +142,13 @@ dtd_user(dp, in, out, new, defer, fail)
 		}
 		continue;
 	    }
-	    new2->in_addr = COPY_STRING(new2->remainder);
-	    new2->director = dp;
-	    new2->parent = cur;
-	    cur = new2;
+	    if (!(cur->flags & ADDR_VRFY_ONLY)) {
+		DEBUG2(DBG_DRIVER_HI, "dtd_user: re-writing addr %s to %s.\n", cur->in_addr, new2->remainder);
+		new2->in_addr = COPY_STRING(new2->remainder);
+		new2->director = dp;
+		new2->parent = cur;
+		cur = new2;
+	    }
 	} else {
 	    /* fill in any user information */
 #ifdef HAVE_FGETPWENT
@@ -152,10 +161,12 @@ dtd_user(dp, in, out, new, defer, fail)
 		}		    
 	    } else
 #endif
+	    {
 		director_user_info(cur);
+	    }
 
 	    /* don't match it if it is not a local user */
-	    if (cur->flags&ADDR_NOTUSER) {
+	    if (cur->flags & ADDR_NOTUSER) {
 		cur->succ = pass;
 		pass = cur;
 		continue;
@@ -184,13 +195,13 @@ dtd_user(dp, in, out, new, defer, fail)
 	     *      correct transport for local delivery.
 	     */
 	    cur->error = note_error(ERR_CONFERR|ERR_122,
-				  xprintf("director %s: transport not defined",
-					  dp->name));
+				    xprintf("director %s: transport not defined",
+					    dp->name));
 	    cur->succ = *defer;
 	    *defer = cur;
 	    continue;
 	}
-	cur->next_addr = COPY_STRING(cur->remainder);
+	cur->next_addr = COPY_STRING(cur->remainder); /* XXX avoid if ADDR_VRFY_ONLY? */
 	cur->succ = *out;
 	*out = cur;
     }
@@ -240,15 +251,15 @@ dtb_user(dp, attrs)
  */
 void
 dtp_user(f, dp)
-     FILE * f;
-     struct director *dp;
+    FILE * f;
+    struct director *dp;
 {
-    	(void) dump_standard_config(f,
-				    (dp->private) ? dp->private : (char *)&user_template,
-				    dp->name,
-				    dp->flags,
-				    user_attributes,
-				    end_user_attributes);
+    (void) dump_standard_config(f,
+				(dp->private) ? dp->private : (char *)&user_template,
+				dp->name,
+				dp->flags,
+				user_attributes,
+				end_user_attributes);
 }
 
 
