@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/directors:RELEASE-3_2_0_111:user.h,v 1.6 1996/05/29 18:49:01 woods Exp"
+#ident	"@(#)smail/src/directors:RELEASE-3_2_0_112:user.h,v 1.7 2000/09/03 16:29:03 woods Exp"
  */
 
 /*
@@ -21,3 +21,12 @@ struct user_private {
     char *prefix;			/* prefix, e.g., "real-" */
     char *pwfile;			/* name of password file */
 };
+
+extern struct addr *dtd_user __P((struct director *,
+				  struct addr *,
+				  struct addr **,
+				  struct addr **,
+				  struct addr **,
+				  struct addr **));
+extern char *dtb_user __P((struct director *, struct attribute *));
+extern void dtp_user __P((FILE *, struct director *));
