@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:dys.h,v 1.10 1999/11/26 03:38:47 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:dys.h,v 1.12 2000/09/03 15:57:14 woods Exp"
  */
 
 /*
@@ -51,7 +51,7 @@ struct str {
  * other macros in this package before those other macros are used
  */
 #define STR_INIT(sp)					\
-	(((sp)->a = STR_BUMP - sizeof(long)),		\
+	(((sp)->a = STR_BUMP),				\
 	 ((sp)->i = 0),					\
 	 ((sp)->p = xmalloc((sp)->a)))
 
@@ -119,4 +119,4 @@ struct str {
 /*
  * COPY_STRING - copy a C-style string to a new xmalloc'd region
  */
-#define COPY_STRING(sp)	(strcpy(xmalloc((unsigned) (strlen((sp)) + 1)), (sp)))
+#define COPY_STRING(sp)	(strcpy(xmalloc((unsigned int) (strlen((sp)) + 1)), (sp)))
