@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:expand.c,v 1.31 1999/11/26 03:38:48 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:expand.c,v 1.33 2000/09/11 01:19:55 woods Exp"
  */
 
 /*
@@ -19,6 +19,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
+#include <errno.h>
 #include <ctype.h>
 #include <pwd.h>
 #include "defs.h"
@@ -39,6 +40,8 @@
 #endif
 
 #include "smail.h"
+#include "main.h"
+#include "parse.h"
 #include "addr.h"
 #include "direct.h"
 #include "route.h"
@@ -47,29 +50,29 @@
 #include "log.h"
 #include "alloc.h"
 #include "dys.h"
+#include "spool.h"
 #include "exitcodes.h"
-#ifndef DEPEND
-# include "debug.h"
-# include "extern.h"
-#endif
+#include "version.h"
+#include "debug.h"
+#include "extern.h"
 
 /* functions local to this file */
-static int expand_string_to();
-static int if_clause();
-static int lookup_clause();
-static int test_cond();
-static char **build_argv();
-static char *substitute();
-static char *strip_fold();
-static char *uc_fold();
-static char *lc_fold();
-static char *quote_fold();
-static char *shell_quote_fold();
-static char *skip_ident();
-static int skip_nesting();
-static int clause_token();
+static int expand_string_to __P((struct str *, char *, struct addr *, char *));
+static int if_clause __P((struct addr *, char *, struct str *, char *, char *));
+static int lookup_clause __P((struct addr *, struct str *, char *, char *));
+static int test_cond __P((struct addr *, char *, char **));
+static char **build_argv __P((char *, int));
+static char *substitute __P((struct addr *, char *, char *, char *, unsigned int));
+static char *strip_fold __P((char *));
+static char *uc_fold __P((char *));
+static char *lc_fold __P((char *));
+static char *quote_fold __P((char *));
+static char *shell_quote_fold __P((char *));
+static char *skip_ident __P((char *, char *));
+static int skip_nesting __P((char **));
+static int clause_token __P((char **, char **, int*));
 #ifndef NODEBUG
-static void bad_subst();
+static void bad_subst __P((char *, unsigned int));
 #endif
 
 
@@ -158,7 +161,6 @@ expand_string_to(str, string, addr, found)
 	    /* ~user turns into home director for the given user */
 	    char *p = string + 1;
 	    struct passwd *pw;
-	    extern struct passwd *getpwbyname();
 
 	    while (*string && *string != '/') {
 		string++;
@@ -204,7 +206,7 @@ expand_string_to(str, string, addr, found)
 		if ((q - p) == 2 && strncmpic(p, "if", 2) == 0) {
 		    if (if_clause(addr, found, str, q, string) == FAIL) {
 #ifndef NODEBUG
-			bad_subst(p, string - p);
+			bad_subst(p, (unsigned int) (string - p));
 #endif
 			return FAIL;
 		    }
@@ -215,19 +217,19 @@ expand_string_to(str, string, addr, found)
 		else if ((q - p) == 6 && strncmpic(p, "lookup", 6) == 0) {
 		    if (lookup_clause(addr, str, q, string) == FAIL) {
 #ifndef NODEBUG
-			bad_subst(p, string - p);
+			bad_subst(p, (unsigned int) (string - p));
 #endif
 			return FAIL;
 		    }
 		} else {
 		    /* not conditional, just a variable */
-		    new = substitute(addr, (char *)NULL, found, p, string - p);
+		    new = substitute(addr, (char *) NULL, found, p, (unsigned int) (string - p));
 		    if (new) {
 			STR_CAT(str, new);
 		    } else {
 			/* unrecognized substitution */
 #ifndef NODEBUG
-			bad_subst(p, string - p);
+			bad_subst(p, (unsigned int) (string - p));
 #endif
 			return FAIL;
 		    }
@@ -241,13 +243,13 @@ expand_string_to(str, string, addr, found)
 		char *new;
 
 		string = skip_ident(string + 1, (char *) NULL);
-		new = substitute(addr, (char *) NULL, found, p, string - p);
+		new = substitute(addr, (char *) NULL, found, p, (unsigned int) (string - p));
 		if (new) {
 		    STR_CAT(str, new);
 		} else {
 		    /* unrecognized substitution */
 #ifndef NODEBUG
-		    bad_subst(p, string - p);
+		    bad_subst(p, (unsigned int) (string - p));
 #endif
 		    return FAIL;
 		}
@@ -391,7 +393,7 @@ test_cond(addr, found, sp)
     char *q = *sp;
     char *s, *s2;
     char *new;
-    int len, len2;
+    unsigned int len, len2;
     struct list *hdr;
     long tpflags;
     struct str str;
@@ -575,7 +577,8 @@ lookup_clause(addr, str, s, es)
     char *xkey = NULL;
     char *xproto = NULL;
     char *xfile = NULL;
-    int key_len, proto_len, file_len;
+    unsigned int key_len;
+    int proto_len, file_len;
     char *then_clause, *then_end, *else_clause, *else_end;
     struct str file_str;
     char *dbinfo;
@@ -908,7 +911,7 @@ build_cmd_line(cmd, addr, file, error)
 		cmd = skip_ident(cmd, cmd + strlen(cmd));
 		/* cmd now one beyond where it should be */
 	    }
-	    new = substitute(addr, file, (char *)NULL, mark, cmd - mark);
+	    new = substitute(addr, file, (char *)NULL, mark, (unsigned int) (cmd - mark));
 	    if (new == NULL) {
 		int c_save = mark[cmd-mark];
 
@@ -1017,7 +1020,7 @@ substitute(addr, file, found, var, len)
     char *file;				/* source for $file */
     char *found;			/* $value value */
     register char *var;			/* start of variable */
-    register int len;			/* length of variable */
+    register unsigned int len;		/* length of variable */
 {
     static char buf[50];		/* big enough for a %ld expansion */
 
@@ -1124,7 +1127,7 @@ substitute(addr, file, found, var, len)
 	return get_arpa_date(message_date());
     }
     if (MATCH("$") || MATCH("pid")) {
-	(void) sprintf(buf, "%d", getpid());
+	(void) sprintf(buf, "%ld", (long) getpid());
 	return buf;
     }
     if (MATCH("uucp_name")) {
@@ -1206,8 +1209,8 @@ static char *
 lc_fold(value)
     register char *value;
 {
-    static int lc_size;			/* keep size of allocated region */
-    int value_size;
+    static unsigned int lc_size;	/* keep size of allocated region */
+    unsigned int value_size;
     static char *lc = NULL;		/* retained malloc region */
     register char *p;			/* for scanning through lc */
 
@@ -1238,8 +1241,8 @@ static char *
 uc_fold(value)
     register char *value;
 {
-    static int uc_size;			/* keep size of allocated region */
-    int value_size;
+    static unsigned int uc_size;	/* keep size of allocated region */
+    unsigned int value_size;
     static char *uc = NULL;		/* retained malloc region */
     register char *p;			/* for scanning through lc */
 
@@ -1392,8 +1395,8 @@ static char *
 strip_fold(value)
     char *value;
 {
-    static int strip_size;		/* keep size of allocated region */
-    int value_size;
+    static unsigned int strip_size;	/* keep size of allocated region */
+    unsigned int value_size;
     static char *strip_buf = NULL;	/* retained malloc region */
     register char *p;			/* for scanning through strip_buf */
     register char *q;			/* also for scanning strip_buf */
@@ -1546,7 +1549,7 @@ skip_nesting(sp)
 static void
 bad_subst(var, len)
     char *var;
-    int len;
+    unsigned int len;
 {
     char *saved;
 
