@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:extern.h,v 1.75 2000/02/17 04:39:15 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:extern.h,v 1.78 2001/01/18 00:31:32 woods Exp"
  */
 
 /*
@@ -14,7 +14,9 @@
  * extern.h:
  *	externals used by the smail program
  */
+#ifdef NEED_ERRNO_DECL
 extern int errno;			/* previous system error value */
+#endif
 
 /*
  * External variables used in the smail program
@@ -33,8 +35,8 @@ extern char *hostnames;			/* list of other local host names */
 extern char *more_hostnames;		/* additional list of hostnames */
 extern long max_message_size;		/* max size of message body */
 extern char *grades;			/* mapping of precedence to grade */
-extern int nobody_uid;			/* user id for access permission */
-extern int nobody_gid;			/* group id for access permission */
+extern unsigned int nobody_uid;		/* user id for access permission */
+extern unsigned int nobody_gid;		/* group id for access permission */
 extern char *nobody;			/* name of nobody user */
 extern char *log_fn;			/* name of information log file */
 extern char *panic_fn;			/* name of panic log file */
@@ -73,7 +75,7 @@ extern unsigned message_bufsiz;		/* size of message buffers */
 extern int hit_table_len;		/* #entries in address hit table */
 extern int flock_mailbox;		/* TRUE to use lock_fd_wait() macro */
 extern int fnlock_retries;		/* retries for lock_file() creat */
-extern int fnlock_interval;		/* retry intervals for lock_file() */
+extern unsigned int fnlock_interval;	/* retry intervals for lock_file() */
 extern int fnlock_mode;			/* mode for lock_file() lockfiles */
 extern int lock_by_name;		/* TRUE to use spool lockfiles */
 extern int queue_only;			/* true to queue but not deliver */
@@ -87,7 +89,7 @@ extern char *smart_transport;		/* transport for smarthost driver */
 extern char *second_config_file;	/* secondary configuration file */
 extern char *copying_file;		/* pathname to COPYING file */
 extern int auto_mkdir;			/* TRUE to auto create directories */
-extern int auto_mkdir_mode;		/* the mode for auto directories */
+extern unsigned int auto_mkdir_mode;	/* the mode for auto directories */
 extern int require_configs;		/* TRUE to require config files */
 extern char *postmaster_address;	/* default addr of postmaster */
 extern char *smtp_banner;		/* smtp startup banner message */
@@ -100,22 +102,26 @@ extern char *smtp_remote_allow;		/* ip list that can send remote smtp */
 extern char *smtp_local_net;		/* the local network calculated from src. addr */
 extern int smtp_hello_verify;		/* verify HELO/EHLO host has A RR matching sockaddr */
 extern char *smtp_hello_broken_allow;	/* ip list that can avoid hello verification */
+extern int smtp_hello_reject_dns_paranoid; /* like TCP Wrappers PARANOID check */
+extern char *smtp_hello_reject_hosts;	/* like TCP Wrappers */
 extern int smtp_hello_verify_literal;	/* verify HELO/EHLO domain literal has PTR RR */
 extern int smtp_hello_verify_ptr;	/* verify HELO/EHLO matches PTR RR for sockaddr */
+extern char *smtp_rbl_domains;		/* list of RBL domains */
+extern char *smtp_rbl_except;		/* list of exceptions to RBL lookups */
 extern char *smtp_sender_no_verify;	/* ip list for not verifying MAIL FROM */
 extern int smtp_sender_verify_mx_only;	/* require MX RRs for MAIL FROM: hosts */
 extern int smtp_max_recipients;		/* maximum recipients per SMTP connect */
-extern int smtp_expn_delay;		/* seconds to delay after an EXPN command */
-extern int smtp_vrfy_delay;		/* seconds to delay after a VRFY command */
-extern int debug_pause_for_smtp_connections; /* seconds to pause after child fork for debugger connect */
+extern unsigned int smtp_expn_delay;	/* seconds to delay after an EXPN command */
+extern unsigned int smtp_vrfy_delay;	/* seconds to delay after a VRFY command */
+extern unsigned int debug_pause_for_smtp_connections; /* seconds to pause after child fork for debugger connect */
 extern char *sender_env_variable;	/* env variable naming user */
 extern int switch_percent_and_bang;	/* switch precedence of % and ! */
 extern int error_copy_postmaster;	/* copy postmaster on errors */
 extern long retry_interval;		/* default delivery retry interval */
 extern long retry_duration;		/* default delivery retry duration */
 extern long host_lock_timeout;		/* timeout for host exclusivity lock */
-extern long smtp_receive_command_timeout; /* timeout for receiving SMTP cmds */
-extern long smtp_receive_message_timeout; /* timeout for receiving SMTP msg */
+extern unsigned int smtp_receive_command_timeout; /* timeout for receiving SMTP cmds */
+extern unsigned int smtp_receive_message_timeout; /* timeout for receiving SMTP msg */
 extern char *auth_domains;		/* authoritative domain list */
 extern long rfc1413_query_timeout;	/* Timeout on RFC1413 queries - initially disabled */
 extern long resolve_timeout;		/* timeout on directors/routers */
@@ -180,14 +186,15 @@ extern struct addr *recipients;		/* list of recipient addresses */
 extern int num_recipients;		/* number of entries in 'recipients' */
 extern char *primary_name;		/* primary local name from hostnames */
 extern FILE *errfile;			/* file to write debug messages to */
-extern int real_uid;			/* saved real uid before ruid setup */
+extern unsigned int real_uid;		/* saved real uid before ruid setup */
+extern unsigned int real_gid;		/* saved real gid before rgid setup */
 extern char **save_argv;		/* saved pointer to arguments */
 extern int some_deferred_addrs;		/* don't unlink spool file */
 					/* as some addrs were deferred */
-extern int prog_euid;			/* effective uid of program */
-extern int prog_egid;			/* effective gid of program */
+extern unsigned int prog_euid;		/* effective uid of program */
+extern unsigned int prog_egid;		/* effective gid of program */
 extern int force_zero_exitvalue;	/* if TRUE always exit with status 0 */
-extern int call_defer_message;		/* if TRUE must call defer_message() */
+extern int call_freeze_message;		/* if TRUE must call freeze_message() */
 extern int sender_is_trusted;		/* TRUE if sender is a trusted user */
 extern char *sender_host;		/* name of sender's host */
 extern char *sender_host_addr;		/* inet address of sender's host */
@@ -243,9 +250,6 @@ extern char *bat;			/* the proper bat for this release */
 extern char *path_to_sender;		/* uucp-style route to sender */
 extern int cached_transports;		/* TRUE if cache_transports() called */
 
-/* external variables defined in silly.c */
-extern void silly();			/* a very silly function */
-
 /* external variables defined in smtprecv.c */
 extern char *sender_host_really;	/* result of PTR lookup */
 
@@ -253,290 +257,142 @@ extern char *sender_host_really;	/* result of PTR lookup */
 /*
  * External functions used in the smail program
  */
-/* external functions defined in addr.c */
-extern char *preparse_address();
-extern char *preparse_address_1();
-extern int parse_address();
-extern char *address_token();
-extern char *back_address_token();
-extern int mixed_address();
-extern char *build_uucp_route();
-extern char *build_partial_uucp_route();
-extern void split_addr_list();
-extern void strip_rfc822_comments();
-extern void strip_rfc822_whitespace();
-extern struct addr *alloc_addr();
-extern void insert_addr_list();
-extern struct addr *addr_sort();
-extern struct error *note_error();
 
-/* external functions defined in alloc.c */
-extern char *xmalloc();
-extern char *xrealloc();
-extern void xfree();
-extern char *bmalloc();
-extern char *brealloc();
-extern void bfree();
-extern struct block *malloc_block();
-extern void realloc_block();
-extern void free_block();
-
-#ifdef HAVE_BIND
-/* external functions defined in bindlib.c */
-extern int bind_check_if_canonical_host();
-extern int bind_check_if_local_mxs();
-extern int bind_addr();
-extern char * bind_compute_domain();
-#endif /* HAVE_BIND */
-
-/* external functions defined in direct.c */
-extern void direct_local_addrs();
-extern void director_user_info();
-extern struct director *find_director();
-extern struct direct_driver *find_direct_driver();
-extern void cache_directors();
-extern void finish_directors();
-extern char *read_director_file();
-extern void dump_director_config();
+/* external functions defined in silly.c */
+extern void silly __P((void));			/* a very silly function */
 
 /* external functions defined in expand.c */
-extern char *expand_string();
-extern char **build_cmd_line();
-
-/* external functions defined in field.c */
-extern char *tokenize();
-extern char *detokenize();
-extern char *process_field();
-extern void dump_tokens();
-
-/* external functions defined in hash.c */
-extern int add_to_hash();
-extern int lookup_in_hash();
-extern struct hash_table *new_hash_table();
-extern void write_hash_table();
-extern struct hash *delete_from_hash();
-extern struct hash *replace_in_hash();
-extern struct hash *store_in_hash();
-extern struct hash *walk_hash();
-extern void free_hash_element();
-extern void free_hash_table();
+extern char *expand_string __P((char *, struct addr *, char *, char *));
+extern char **build_cmd_line __P((char *, struct addr *, char *, char **));
 
 /* external functions defined in header.c */
-extern char *process_header();
-extern int read_header();
-extern int write_header();
-extern int parse_precedence();
-
-/* external functions defined in log.c */
-extern void open_system_logs();
-extern void close_system_logs();
-extern void open_msg_log();
-extern void close_msg_log();
-extern void unlink_msg_log();
-#ifdef ANSI_C
-extern void panic(int, char *, ...);
-extern void write_log(int, char *, ...);
-#else
-extern void panic();
-extern void write_log();
-#endif
-extern void send_log();
-extern char *scan_msg_log();
-
-/* external functions defined in lookup.c */
-extern int open_database();
-extern void close_database();
-extern int lookup_database();
-extern int match_ip();
-
-/* external functions defined in main.c */
-extern void initialize_state();
-extern void reset_hit_table();
-extern void process_args();
+extern char *process_header __P((struct addr **));
+extern int read_header __P((void));
+extern int write_header __P((FILE *, struct addr *));
+extern int parse_precedence __P((char *));
 
 /* external functions defined in modes.c */
-extern void build_host_strings();
-extern void compute_nobody();
-extern void input_signals();
-extern void processing_signals();
-extern void delivery_signals();
-extern void test_addresses();
-extern void perform_deliver_mail();
-extern void deliver_mail();
-extern void daemon_mode();
-extern void noop_mode();
-extern void verify_addresses();
-extern void print_version();
-extern void print_copying_file();
-extern void print_variables();
-extern void print_queue();
-extern void smtp_mode();
-extern int process_spool_file();
-extern int fork_wait();
-extern void run_queue();
+extern void build_host_strings __P((void));
+extern void compute_nobody __P((void));
+extern void input_signals __P((void));
+extern void processing_signals __P((void));
+extern void delivery_signals __P((void));
+extern void test_addresses __P((void));
+extern void perform_deliver_mail __P((void));
+extern void deliver_mail __P((void));
+extern void daemon_mode __P((void));
+extern void noop_mode __P((void));
+extern void verify_addresses __P((void));
+extern void print_version __P((void));
+extern void print_copying_file __P((void));
+extern void print_variables __P((void));
+extern void print_queue __P((void));
+extern void smtp_mode __P((FILE *, FILE *));
+extern int process_spool_file __P((char *));
+extern int fork_wait __P((void));
+extern void run_queue __P((int));
 
 /* external functions defined in notify.c */
-extern void fail_delivery();
-extern void defer_delivery();
-extern void succeed_delivery();
-extern void error_delivery();
-extern struct addr *process_msg_log();
-extern void notify();
-extern void hash_predelivered_addresses();
-
-/* external functions defined in parse.c */
-extern char *parse_entry();
-extern struct attribute *parse_config();
-extern struct attribute *parse_table();
-extern char *read_entry();
+extern void fail_delivery __P((struct addr *));
+extern void defer_delivery __P((struct addr *, struct defer_addr *));
+extern void succeed_delivery __P((struct addr *));
+extern void error_delivery __P((struct addr *));
+extern struct addr *process_msg_log __P((struct addr *, struct identify_addr **, struct defer_addr **));
+extern void notify __P((struct addr *, struct addr *, struct identify_addr *));
+extern void hash_predelivered_addresses __P((void));
 
 /* external functions defined in pathto.c */
-extern void pathto();
-extern void optto();
-extern void uupath();
+extern void pathto __P((int, char **));
+extern void optto __P((int, char **));
+extern void uupath __P((int, char **));
 
 /* external functions defined in pwcache.c */
-extern struct passwd *getpwbyname();
-extern struct group *getgrbyname();
-extern struct passwd *getpwbyuid();
-extern struct group *getgrbygid();
+extern struct passwd *getpwbyname __P((char *));
+extern struct group *getgrbyname __P((char *));
+extern struct passwd *getpwbyuid __P((unsigned int));
+extern struct group *getgrbygid __P((unsigned int));
 
 /* external functions defined in queue.c */
-extern int queue_message();
-extern char **read_message();
-extern int write_body();
-extern void check_grade();
-extern void log_incoming();
-extern char **scan_spool_dirs();
-extern int swallow_smtp();
+extern int queue_message __P((FILE *, enum dot_usage, struct addr *, char **));
+extern char **read_message __P((void));
+extern int write_body __P((FILE *, long));
+extern void check_grade __P((void));
+extern void log_incoming __P((void));
+extern char **scan_spool_dirs __P((void));
+extern int swallow_smtp __P((FILE *));
 
 /* external functions defined in resolve.c */
-extern void resolve_addr_list();
-extern int islocalhost();
+extern void resolve_addr_list __P((struct addr *, struct addr **, struct addr **, struct addr **, int));
+extern int islocalhost __P((char *));
 
 /* external functions defined in qualify.c */
-extern char *read_qualify_file();
-extern char *qualify_domain();
-extern void dump_qualify_config();
+extern char *read_qualify_file __P((void));
+extern char *qualify_domain __P((char *));
+extern void dump_qualify_config __P((FILE *));
 
 /* external functions defined in retry.c */
-extern char *read_retry_file();
-extern void dump_retry_config();
-extern struct addr *retry_addr_before();
-extern struct addr *retry_addr_after();
-extern void retry_addr_finished();
-extern int retry_host_lock();
-extern void retry_host_unlock();
-
-/* external functions defined in route.c */
-extern void route_remote_addrs();
-extern char *match_end_domain();
-extern char *match_end();
-extern char *is_suffix();
-extern void route_driver_finish();
-extern struct router *find_router();
-extern struct route_driver *find_route_driver();
-extern void cache_routers();
-extern void finish_routers();
-extern char *read_router_file();
-extern void dump_router_config();
-extern struct method *read_method_file();
-
-/* external functions defined in smailconf.c */
-extern char *read_config_file();
-extern char *quote_string_value();
-extern void print_config_variable();
-extern char *read_standard_file();
-extern void dump_standard_config();
-extern char *fill_attributes();
-extern struct attr_table *find_attribute();
-extern void add_config_stat();
-extern int is_newconf();
-extern char *make_lib_fn();
+extern char *read_retry_file __P((void));
+extern void dump_retry_config __P((FILE *));
+extern struct addr *retry_addr_before __P((struct addr *, struct addr **, struct addr **));
+extern struct addr *retry_addr_after __P((time_t, struct addr *, struct addr **));
+extern void retry_addr_finished __P((struct addr *));
+extern int retry_host_lock __P((struct transport *, char *, int *, struct error **));
+extern void retry_host_unlock __P((time_t, struct error *));
 
 /* external functions defined in smtprecv.c */
-extern char **receive_smtp();
-
-/* external functions defined in spool.c */
-extern int creat_spool();
-extern int lock_message();
-extern void unlock_message();
-extern int write_spool();
-extern int open_spool();
-extern void close_spool();
-extern void unlink_spool();
-extern int seek_spool();
-extern off_t tell_spool();
-extern int send_spool();
-extern int read_spool();
-extern void log_spool_errors();
-extern int new_grade();
-extern void defer_message();
-extern time_t message_date();
-extern long spool_max_free_space();
+extern char **receive_smtp __P((FILE *, FILE *));
 
 /* external functions defined in string.c */
-extern char *str2lower();
-extern int strcmpic();
-extern int strncmpic();
-extern int strip();
-extern char *strcolon();
-extern int is_string_in_list();
+extern char *str2lower __P((char *));
+extern int strcmpic __P((char *, char *));
+extern int strncmpic __P((char *, char *, unsigned int));
+extern int strip __P((char *));
+extern char *strcolon __P((char *));
+extern int is_string_in_list __P((char *, char *));
 #ifndef HAVE_STRERROR
-extern char *strerror();
+extern char *strerror __P((int));
 #endif
-extern char *strsysexit();
-extern void str_printf();
-extern char *xprintf();
-extern int dprintf();
-extern long c_atol();
-extern char *base62();
-extern char *read_line();
-extern void str_cat();
-extern void str_ncat();
+extern char *strsysexit __P((int));
+extern void str_printf __P((struct str *, char *, ...));
+extern char *xprintf __P((char *, ...));
+extern int dprintf __P((FILE *, char *, ...));
+extern long c_atol __P((char *, char **));
+extern char *base62 __P((unsigned long));
+extern char *read_line __P((FILE *));
+extern void str_cat __P((struct str *, char *));
+extern void str_ncat __P((struct str *, char *, unsigned int));
 #ifndef HAVE_VFPRINTF
-extern int vfprintf();
+extern int vfprintf __P((FILE *, char *, va_list));
 #endif
-extern long ivaltol();
-extern char *ltoival();
-extern char *copy();
-extern char *rcopy();
+extern long ivaltol __P((char *));
+extern char *ltoival __P((long));
+extern char *copy __P((char *));
+extern char *rcopy __P((char *, char *));
 
 /* external functions defined in sysdep.c */
-extern char *time_stamp();
-extern char *get_arpa_date();
-extern int get_local_year();
-extern char *unix_date();
-extern void compute_local_sender();
-extern void getfullname();
-extern FILE *fopen_as_user();
-extern int lock_file();
-extern void unlock_file();
-extern char *compute_hostname();
-extern char *compute_domain();
-extern int open_child();
-extern int close_child();
-extern void close_all();
-extern char *scan_dir();
-extern int touch();
-extern int fsetsize();
+extern char *time_stamp __P((void));
+extern char *get_arpa_date __P((time_t));
+extern int get_local_year __P((void));
+extern char *unix_date __P((void));
+extern void compute_local_sender __P((void));
+extern void getfullname __P((void));
+extern FILE *fopen_as_user __P((char *, char *, unsigned int, unsigned int, unsigned int));
+extern int lock_file __P((char *, FILE *));
+extern void unlock_file __P((char *, FILE *));
+#ifdef	USE_FCNTL_RD_LOCK
+extern int fcntl_rd_lock __P((int));
+#endif
+extern char *compute_hostname __P((void));
+extern char *compute_domain __P((char *));
+extern int open_child __P((char **, char **, FILE **, FILE **, int, int, unsigned int, unsigned int));
+extern int close_child __P((FILE *, FILE *, int));
+extern void close_all __P((void));
+extern char *scan_dir __P((char *));
+extern int touch __P((char *, time_t, time_t));
+extern int fsetsize __P((int, off_t, off_t));
 #ifdef NEED_HSTRERROR
-extern const char *hstrerror();
+extern const char *hstrerror __P((int));
 #endif
 
-/* external functions defined in transport.c */
-extern struct assign_transport *assign_transports();
-extern void call_transports();
-extern int write_message();
-extern char *get_sender_addr();
-extern char *remote_from_line();
-extern char *local_from_line();
-extern struct transport *find_transport();
-extern struct transport_driver *find_transport_driver();
-extern void cache_transports();
-extern void finish_transports();
-extern char *read_transport_file();
-extern void dump_transport_config();
-
-/* external functions defined in version.c */
-extern char *version();
+/* external functions defined in libcompat XXX should have compat.h */
+int dummy_reference __P((void));
