@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:field.c,v 1.14 2000/02/14 05:30:25 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:field.c,v 1.16 2000/09/29 22:15:05 woods Exp"
  */
 
 /*
@@ -29,28 +29,29 @@
  *	external functions:  process_field, tokenize, detokenize, dump_tokens
  */
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <stdio.h>
 #include <ctype.h>
 #include "defs.h"
 #include "smail.h"
-#include "field.h"
+#include "main.h"
 #include "addr.h"
+#include "field.h"
 #include "log.h"
+#include "alloc.h"
 #include "dys.h"
 #include "exitcodes.h"
-#ifndef DEPEND
-# include "debug.h"
-# include "extern.h"
-#endif
+#include "debug.h"
+#include "extern.h"
 
 /* functions local to this file */
-static char *finish_mod_clean();
-static void insert_comma();
-static int match_route_or_group();
-static int match_group_term();
-static int match_general();
-static char *queue_qualify_domain();
-static int enqueue_address();
+static char *finish_mod_clean __P((char *, unsigned int, struct token *, unsigned len));
+static void insert_comma __P((char *));
+static int match_route_or_group __P((struct token *, struct token **, struct addr **, int, unsigned int *, char *, char *, int, char **));
+static int match_group_term __P((struct token *, struct token **, struct addr **, int, char **));
+static int match_general __P((struct token *, struct token **, unsigned int *, struct addr **, char *, char *, int, char **));
+static char *queue_qualify_domain __P((struct token *, struct token *));
+static int enqueue_address __P((struct addr **, struct token *, struct token *, char **));
 
 /* macros local to this file */
 #define DUMP_TOKENS(d,t)  {if (d <= debug) dump_tokens(t);}
@@ -607,7 +608,7 @@ process_field(field, fp, domain, uucp_host, extract_q, flags, error)
     if (field) {
 	len = strlen(field) + 1;
     }
-    DEBUG(DBG_FIELD_LO, "process_field(): entry\n");
+    DEBUG(DBG_FIELD_HI, "process_field(): entry\n");
     /* tokenize the contents to make parsing easy */
     error_message = tokenize(fp, &tq_head, flags&F_ALIAS, field != NULL);
 
@@ -771,7 +772,7 @@ process_field(field, fp, domain, uucp_host, extract_q, flags, error)
 	field = finish_mod_clean(field, (unsigned)(fp-field), tq_head, len);
     }
 
-    DEBUG1(DBG_FIELD_LO, "process_field: return %s\n", field);
+    DEBUG1(DBG_FIELD_MID, "process_field: return %s.\n", field);
     return field;			/* all done, return the header */
 }
 
@@ -915,8 +916,8 @@ match_route_or_group(tq_anchor, tq_new, extract_q, group,
     struct token *tq_anchor;		/* anchor point from process_field */
     struct token **tq_new;		/* return last matched token here */
     struct addr **extract_q;		/* queue in which to put addresses */
-    unsigned *len;			/* len variable from process_field */
     int group;				/* TRUE if group would be recursive */
+    unsigned *len;			/* len variable from process_field */
     char *domain;			/* domain to add to local addresses */
     char *uucp_host;			/* uucp host to prepend to ! routes */
     int flags;				/* miscellaneous flags */
@@ -944,13 +945,13 @@ match_route_or_group(tq_anchor, tq_new, extract_q, group,
     	}
     	if (tq->text[0] == '<') {
     	    /* we have a route */
-    	    DEBUG(DBG_FIELD_HI, "We have a route\n");
+    	    DEBUG(DBG_FIELD_HI, "process_field: We have a route\n");
     	    *tq_new = tq;
 	    break;
     	}
     	if (tq->text[0] == ':' && tq->form == T_OPER) {
     	    /* we have a group */
-    	    DEBUG(DBG_FIELD_HI, "We have a group\n");
+    	    DEBUG(DBG_FIELD_HI, "process_field: We have a group\n");
     	    *tq_new = tq;
     	    if (group) {
     	    	/* catch recursive groups */
@@ -977,7 +978,7 @@ match_route_or_group(tq_anchor, tq_new, extract_q, group,
     {
 	if (tq->text[0] == '<') {
 	    recursion_level++;
-	    DEBUG(DBG_FIELD_HI, "bump up recursion level on route\n");
+	    DEBUG(DBG_FIELD_HI, "process_field: bump up recursion level on route\n");
 
 	    /* at more deeply nested address, forget what we had before */
 	    tq_start = tq->succ;
@@ -985,7 +986,7 @@ match_route_or_group(tq_anchor, tq_new, extract_q, group,
 	    seek_end = 1;
 	} else if (tq->text[0] == '>') {
 	    recursion_level--;
-	    DEBUG(DBG_FIELD_HI, "bump down recursion level on route\n");
+	    DEBUG(DBG_FIELD_HI, "process_field: bump down recursion level on route\n");
 	    seek_end = 0;		/* the end, if no more < tokens */
 
 	    if (recursion_level == 0) {
@@ -1262,7 +1263,7 @@ match_general(tq_anchor, tq_new, len, extract_q, domain,
 	return T_MUTANT_FORM;		/* mutant form allowed for route */
     }
 
-    DEBUG(DBG_FIELD_HI, "found a WORD op WORD op ... WORD sequence\n");
+    DEBUG(DBG_FIELD_HI, "process_field: found a WORD op WORD op ... WORD sequence\n");
     *tq_new = tq;		/* at end of sequence */
 
     /*
