@@ -1,4 +1,6 @@
-/* @(#) field.h,v 1.3 1992/07/11 11:49:14 tron Exp */
+/*
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:field.h,v 1.4 2000/09/03 16:26:31 woods Exp"
+ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -52,3 +54,9 @@ struct token {				/* elements of token lists */
 #define F_LOCAL		0x01		/* process as locally generated mail */
 #define F_STRICT	0x02		/* use strict RFC822 reformatting */
 #define F_ALIAS		0x04		/* process as with aliases file */
+
+/* external functions defined in field.c */
+extern char *tokenize __P((char *, struct token **, int, int));
+extern char *detokenize __P((int, char *, struct token *, struct token *));
+extern char *process_field __P((char *, char *, char *, char *, struct addr **, int, char **));
+extern void dump_tokens __P((struct token *));
