@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:hash.c,v 1.16 2000/02/17 03:54:44 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:hash.c,v 1.17 2000/09/03 17:00:13 woods Exp"
  */
 
 /*
@@ -31,6 +31,7 @@
  *		free_hash_table
  */
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <stdio.h>
 #include <ctype.h>
 #include "defs.h"
@@ -38,18 +39,22 @@
 # include <stdlib.h>
 #endif
 #include "smail.h"
+#include "main.h"
+#include "addr.h"
 #include "exitcodes.h"
 #include "hash.h"
 #include "alloc.h"
+#include "dys.h"
+#include "log.h"
 #ifndef DEPEND
 # include "extern.h"
 # include "debug.h"
 #endif
 
 /* static functions used in this file */
-static int hash_str();
-static int hash_stric();
-static struct hash *new_hash_element();
+static int hash_str __P((char *, int));
+static int hash_stric __P((char *, int));
+static struct hash *new_hash_element __P((char *, char *, int, struct hash_table *));
 
 #ifdef STANDALONE
 int debug = 0;	/* default debug level is NONE */
@@ -117,19 +122,19 @@ hash_str(str, mod)
 	/* hash each character using the L_ values */
 	for (val = 0; (c = (unsigned long) *str); ++str) {
 	    val = (val << HASH_UP_SHIFT) + c;
-	    val ^= ((excess = (val&L_HASH_MASK)) >> L_DOWN_SHIFT);
+	    val ^= ((excess = (val & (unsigned long) L_HASH_MASK)) >> L_DOWN_SHIFT);
 	    val ^= excess;
 	}
     } else {
 	/* hash each character using the H_ values */
 	for (val = 0; (c = (unsigned long) *str); ++str) {
 	    val = (val << HASH_UP_SHIFT) + c;
-	    val ^= ((excess = (val&H_HASH_MASK)) >> H_DOWN_SHIFT);
+	    val ^= ((excess = (val & (unsigned long) H_HASH_MASK)) >> H_DOWN_SHIFT);
 	    val ^= excess;
 	}
     }
     
-    return (int)(val%mod);    /* our hash value, mod the hash size */
+    return (int) (val % mod);    /* our hash value, mod the hash size */
 }
 
 /*
@@ -182,19 +187,19 @@ hash_stric(str, mod)
 	/* hash each character using the L_ values */
 	for (val = 0; (c = (unsigned long) lowercase((unsigned char) *str)); ++str) {
 	    val = (val << HASH_UP_SHIFT) + c;
-	    val ^= ((excess = (val&L_HASH_MASK)) >> L_DOWN_SHIFT);
+	    val ^= ((excess = (val & (unsigned long) L_HASH_MASK)) >> L_DOWN_SHIFT);
 	    val ^= excess;
 	}
     } else {
 	/* hash each character using the H_ values */
 	for (val = 0; (c = (unsigned long) lowercase((unsigned char) *str)); ++str) {
 	    val = (val << HASH_UP_SHIFT) + c;
-	    val ^= ((excess = (val&H_HASH_MASK)) >> H_DOWN_SHIFT);
+	    val ^= ((excess = (val & (unsigned long) H_HASH_MASK)) >> H_DOWN_SHIFT);
 	    val ^= excess;
 	}
     }
     
-    return (int)(val%mod);    /* our hash value, mod the hash size */
+    return (int) (val % mod);    /* our hash value, mod the hash size */
 }
 
 
@@ -335,13 +340,13 @@ new_hash_element(keystr, data, datalen, table)
 	panic(EX_SOFTWARE, "new_hash_element: NULL keystring or table");
     }
     lk = keystr_len(keystr);		/* compute padded key length */
-    if (lk >= (unsigned long)(1L<<BITS_PER_SHORT)) {
+    if (lk >= (unsigned long) (1L << BITS_PER_SHORT)) {
 	panic(EX_SOFTWARE, "new_hash_element: key too long");
     }
     keylen = (int)lk;		/* now we know it will fit in an int */
     /* firewall - check against bad data being passed to us */
     if (datalen < 0 || (datalen > 0 && data == NULL) || 
-      (unsigned long)datalen >= (unsigned long)(1L<<BITS_PER_SHORT))  {
+      (unsigned long) datalen >= (unsigned long) (1L << BITS_PER_SHORT))  {
 	panic(EX_SOFTWARE,
 	    "new_hash_element: bad data passed with: <%s>  datalen: <%d>",
 	    keystr, datalen);
@@ -350,7 +355,7 @@ new_hash_element(keystr, data, datalen, table)
     /*
      * malloc the storage
      */
-    new = (struct hash *)bmalloc( hashslot_size(keylen,datalen), table->life );
+    new = (struct hash *) bmalloc((unsigned int) hashslot_size(keylen, datalen), table->life);
     /* firewall */
     if (is_odd(new)) {
 	panic(EX_SOFTWARE,
@@ -420,7 +425,7 @@ free_hash_element(cur, search, table)
     if (!table->life) {
 	DEBUG(DBG_HASH_LO, "free_hash_element(): about to panic() with NULL block\n");
     }
-    bfree(cur, table->life);
+    bfree((char *) cur, table->life);
     return;
 }
 
@@ -506,7 +511,7 @@ free_hash_table(table)
     if (!table->life) {
 	DEBUG(DBG_HASH_LO, "free_hash_table(): about to panic() with NULL block\n");
     }
-    bfree(table, table->life);
+    bfree((char *) table, table->life);
 
     /* XXX why don't we free 'table' itself?  or free_block() it? */
 
