@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:hash.h,v 1.7 1996/02/28 14:26:30 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:hash.h,v 1.9 2000/09/11 19:11:36 woods Exp"
  */
 
 /*
@@ -16,28 +16,40 @@
  */
 
 /*
- * hash function values
+ *	hash function values
+ */
+
+/*
+ * shift the hash up HASH_UP_SHIFT bits before adding the next character
+ */
+#define HASH_UP_SHIFT	 (3)
+
+/*
+ * if hash mod < (1 << HASH_LEVEL_SHIFT), use L_ hash values, otherwise H_
  */
-/* shift the hash up HASH_UP_SHIFT bits before adding the next character */
-#define HASH_UP_SHIFT (3)
-/* if hash mod < (1<<HASH_LEVEL_SHIFT), use L_ hash values, otherwise H_ */
 #define HASH_LEVEL_SHIFT (17)
-#define HASH_LEVEL (1<<HASH_LEVEL_SHIFT)
-/* for hash sizes where the hash mod < HASH_LEVEL */
-#define L_MASK_SHIFT (HASH_LEVEL_SHIFT)
-#define L_HASH_MASK (~((1<<L_MASK_SHIFT)-1))     /* mask of excess bits */
-#define L_DOWN_SHIFT (L_MASK_SHIFT-1)		 /* shift down excess */
-/* for hash sizes where the hash mod >= HASH_LEVEL */
-#define H_MASK_SHIFT (BITS_PER_LONG-HASH_UP_SHIFT-2)
-#define H_HASH_MASK (~((1<<H_MASK_SHIFT)-1))	 /* mask of excess bits */
-#define H_DOWN_SHIFT (H_MASK_SHIFT-2)		 /* shift down excess */
+#define HASH_LEVEL	(1 << HASH_LEVEL_SHIFT)
+
+/*
+ * for hash sizes where the hash mod < HASH_LEVEL
+ */
+#define L_MASK_SHIFT	(HASH_LEVEL_SHIFT)
+#define L_HASH_MASK	(~((1 << L_MASK_SHIFT) - 1)) /* mask of excess bits */
+#define L_DOWN_SHIFT	(L_MASK_SHIFT - 1)	/* shift down excess */
+
+/*
+ * for hash sizes where the hash mod >= HASH_LEVEL
+ */
+#define H_MASK_SHIFT	(BITS_PER_LONG - HASH_UP_SHIFT - 2)
+#define H_HASH_MASK	(~((1 << H_MASK_SHIFT) - 1)) /* mask of excess bits */
+#define H_DOWN_SHIFT	(H_MASK_SHIFT - 2)	/* shift down excess */
 
 /*
  * what add_to_hash(), lookup_in_hash(), ... return
  */
-#define NOT_HASHED (-1)		/* the key was not hashed */
-#define JUST_HASHED (0)		/* the key was just hashed */
-#define ALREADY_HASHED (1)	/* the key has been already hashed */
+#define NOT_HASHED	(-1)		/* the key was not hashed */
+#define JUST_HASHED	(0)		/* the key was just hashed */
+#define ALREADY_HASHED	(1)		/* the key has been already hashed */
 
 /*
  * hash_table - hash slots which map integers to mixed chains of hash elements
@@ -58,13 +70,22 @@ struct hash_table {
     struct block *life;	/* the malloc block storage belongs to */
     struct hash *slot[1];	/* ``len'' consecutive slot chain pointers */
 };
-/* hash table entry size in bytes - given the number of slots */
+
+/*
+ * hash table entry size in bytes - given the number of slots
+ */
 #define table_size(len) \
-    (((len)*sizeof(struct hash *)) + OFFSET(hash_table, slot[0]))
-/* return TRUE if the hash table slot is empty, not-TRUE otherwise */
-#define empty_slot(slot) ((struct hash *)(slot) == NULL)
-/* return FALSE if the hash table slot is empty, not-FALSE otherwise */
-#define full_slot(slot) ((struct hash *)(slot) != NULL)
+    (((len) * sizeof(struct hash *)) + OFFSET(hash_table, slot[0]))
+
+/*
+ * return TRUE if the hash table slot is empty, not-TRUE otherwise
+ */
+#define empty_slot(slot)	((struct hash *) (slot) == NULL)
+
+/*
+ * return FALSE if the hash table slot is empty, not-FALSE otherwise
+ */
+#define full_slot(slot)		((struct hash *) (slot) != NULL)
 
 /*
  * hash flags
@@ -105,40 +126,55 @@ struct hash_table {
  * The length of the key string is ``element.keylen''.  Each key string must
  * be NULL byte terminated.
  *
- * One can may optionally associate data with the element.  The length of
- * this data is found in ``element.datalen''.  Extra bytes may be added to
- * pad the ``element'' to a BYTES_PER_ALIGN byte length.  The first byte of
- * the starting data is located at ``element.keystr[element.keylen]''.
+ * One may optionally associate data with the element.  The length of this data
+ * is found in ``element.datalen''.  Extra bytes may be added to pad the
+ * ``element'' to a BYTES_PER_ALIGN byte length.  The first byte of the
+ * starting data is located at ``element.keystr[element.keylen]'' (i.e. just
+ * after the end of the key string).
+ *
+ * We assume that OFFSET(hash, keystr[0]) will be aligned to BYTES_PER_ALIGN by
+ * the compiler and elements will be allocated to start on allignment boundaries.
  */
 struct hash {
     /* NOTE: succ must be the first element */
     struct hash *succ; 	/* pointer to next hash entry, or NULL */
-    short keylen;	/* length of key string + '\0' + word boundary pad */
+    short keylen;	/* strlen() of key string + '\0' + alignment pad */
     short datalen;	/* length in bytes of the data beyond the key string */
     char keystr[1];	/* padded key string, optionally followed by data */
 };
 
-/* hash_align aligns objects on optimal addresses */
-#define hash_align(val) (((int)(val)+(BYTES_PER_ALIGN-1))&~(BYTES_PER_ALIGN-1))
-/* correctly padded length of the key string - given the key string */
-#define keystr_len(keystring) \
-  hash_align(strlen((char *)(keystring))+1)
-/* hash slot size in bytes - given lengths of the padded key string and data */
-#define hashslot_size(keystrlen,datalen) \
- (hash_align(OFFSET(hash, keystr[0]) + \
-	     hash_align((int)(keystrlen)+1) + \
-	     hash_align((int)(datalen))))
-/* hash slot length in bytes - given a ``struct hash'' element pointer */
-#define hash_len(cur) \
- (OFFSET(hash, keystr[0]) + \
-  (int)(((struct hash *)(cur))->keylen) + \
-  hash_align((((struct hash *)(cur))->datalen)))
+/* 
+ * hash_align aligns objects on optimal addresses
+ */
+#define hash_align(val)		(((int) (val) + (BYTES_PER_ALIGN - 1)) & ~(BYTES_PER_ALIGN - 1))
 
-/* pointer to hash data - given a ``struct hash'' element pointer */
-#define hash_data(cur) \
- ((((struct hash *)(cur))->datalen > 0) ? \
-  ((char *)(((struct hash *)(cur))->keystr+((struct hash *)(cur))->keylen)) :\
-  ((char *)NULL))
+/*
+ * correctly padded length of the key string - given the key string
+ */
+#define keystr_len(keystring)	hash_align(strlen((char *) (keystring)) + 1)
+
+/* 
+ * hash slot size in bytes - given the lengths of the padded key string and
+ * unpadded data
+ */
+#define hashslot_size(klen, dlen)	(OFFSET(hash, keystr[0]) + klen + hash_align(dlen))
+
+/* 
+ * hash slot length in bytes - given a ``struct hash'' element pointer
+ */
+#define hash_len(cur)						\
+		(OFFSET(hash, keystr[0]) +			\
+		 (int) (((struct hash *) (cur))->keylen) +	\
+  		 hash_align((((struct hash *) (cur))->datalen)))
+
+/*
+ * pointer to hash data - given a ``struct hash'' element pointer
+ */
+#define hash_data(cur)						\
+ 		((((struct hash *) (cur))->datalen > 0) ?	\
+		 ((char *) (((struct hash *) (cur))->keystr +	\
+			    ((struct hash *) (cur))->keylen)) :	\
+		 ((char *) NULL))
 
 /*
  * stringcmp - compare two strings
@@ -265,3 +301,15 @@ struct hash {
  *	a pointer to the next element, or NULL if no next element
  */
 #define next_hash(cur, table) hash_addr(cur->succ, table)
+
+/* external functions defined in hash.c */
+extern int add_to_hash __P((char *, char *, int, struct hash_table *));
+extern int lookup_in_hash __P((char *, char **, int *, struct hash_table *));
+extern struct hash_table *new_hash_table __P((int, struct block *, int));
+extern void write_hash_table __P((struct hash_table *, FILE *));
+extern struct hash *delete_from_hash __P((char *, struct hash_table *));
+extern struct hash *replace_in_hash __P((char *, char *, int, struct hash_table *));
+extern struct hash *store_in_hash __P((char *, char *, int, struct hash_table *));
+extern struct hash *walk_hash __P((struct hash *, struct hash_table *));
+extern void free_hash_element __P((struct hash *, int, struct hash_table *));
+extern void free_hash_table __P((struct hash_table *));
