@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:header.c,v 1.33 1999/12/03 00:47:42 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:header.c,v 1.34 2000/09/03 17:00:13 woods Exp"
  */
 
 /*
@@ -18,23 +18,25 @@
  */
 
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <pwd.h>
 #include "defs.h"
 #include "smail.h"
-#include "field.h"
+#include "main.h"
+#include "parse.h"
 #include "addr.h"
+#include "field.h"
 #include "transport.h"
 #include "main.h"
 #include "spool.h"
 #include "log.h"
 #include "exitcodes.h"
+#include "alloc.h"
 #include "dys.h"
-#ifndef DEPEND
-# include "extern.h"
-# include "debug.h"
-#endif
+#include "extern.h"
+#include "debug.h"
 
 /* variables defined in this file */
 struct list *header;			/* list of header fields */
@@ -63,18 +65,18 @@ static struct list *remote_header;	/* simple remote-form header */
 static struct list *strict_header;	/* strict RFC822 header */
 
 /* functions local to this file */
-static void check_for_resent();
-static void scan_header();
-static void new_field();
-static char *build_to_field();
-static char *build_date_field();
-static char *build_message_id_field();
-static char *build_from_field();
-static int write_field();
-static char *build_received_field();
-static char *build_return_path();
-static void build_remote_header();
-static void build_strict_header();
+static void check_for_resent __P((void));
+static void scan_header __P((struct addr **, char **));
+static void new_field __P((char *));
+static char *build_to_field __P((void));
+static char *build_date_field __P((void));
+static char *build_message_id_field __P((void));
+static char *build_from_field __P((char *));
+static int write_field __P((char *, FILE *, int));
+static char *build_received_field __P((struct addr *));
+static char *build_return_path __P((struct addr *));
+static void build_remote_header __P((void));
+static void build_strict_header __P((void));
 
 
 /*
@@ -578,7 +580,7 @@ int
 parse_precedence(field)
     char *field;
 {
-    int len;				/* length of token */
+    unsigned int len;			/* length of token */
     struct token *tok;			/* list of tokens from tokenize */
     register char *p = grades;		/* temp for scanning grades */
     char *error;			/* error message from tokenize */
