@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:log.c,v 1.35 2000/02/14 05:29:34 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:log.c,v 1.42 2001/01/18 07:34:14 woods Exp"
  */
 
 /*
@@ -22,12 +22,12 @@
  *			     open_msg_log, close_msg_log, unlink_msg_log,
  *			     panic, write_log, send_log, scan_msg_log
  */
-#include "defs.h"
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
-#include <sys/types.h>
-#include <sys/stat.h>
+#include "defs.h"
 #ifdef ANSI_C
 # include <stdarg.h>
 #else
@@ -45,11 +45,11 @@
 #endif	/* not POSIX_OS */
 
 #include "smail.h"
+#include "main.h"
+#include "addr.h"
 #include "alloc.h"
 #include "dys.h"
 #include "log.h"
-#include "addr.h"
-#include "main.h"
 #include "spool.h"
 #include "exitcodes.h"
 #ifndef DEPEND
@@ -91,12 +91,9 @@ static FILE *logfile = NULL;		/* open stream to system log file */
 static char *msg_log_fn;		/* name of per-message log file */
 
 /* functions local to this file */
-static void build_msg_log_fn();
-static void try_mkdir();
-static void write_log_va();
-
-/* variables imported from the C library */
-extern int errno;			/* system error number */
+static void build_msg_log_fn __P((void));
+static void try_mkdir __P((char *));
+static void write_log_va __P((int, char *, va_list));
 
 
 /*
@@ -145,13 +142,14 @@ open_system_logs()
 	    fd = creat(panic_fn, log_mode);
 #endif
 	}
-	if (fd < 0) {
-	    panic(EX_OSFILE, "cannot open %s: %s", panic_fn, strerror(errno));
-	    /*NOTREACHED*/
-	}
     }
-    panicfile = fdopen(fd, "a");
-    (void) setbuf(panicfile, panicbuf);	/* associate stream and buffer */
+    if (fd < 0) {
+	fprintf(stderr, "%s: [%lu] cannot open %s: %s\n", program, (long unsigned int) getpid(), panic_fn, strerror(errno));
+	panicfile = stderr;
+    } else {
+	panicfile = fdopen(fd, "a");
+	(void) setbuf(panicfile, panicbuf);   /* associate stream and buffer */
+    }
 
     /*
      * next open the system log file after which we can start issuing
@@ -363,13 +361,14 @@ unlink_msg_log()
 
 
 /*
- * panic - panic and die, attempting to write to the panic log file.
+ * panic - panic and die!
  *
- * If we fail to write to the panic log file, write to the console.
- * In all cases try to write to stderr.
+ * Attempt to write to the panic and system log files. and no matter what we
+ * also try to write to stderr.
  *
- * If this is called after spooling a message, then the message will
- * stay around for further processing.
+ * If this is called after spooling a message, then the message will be put in
+ * the error queue so that the postmaster can reprocess it after correcting
+ * whatever error caused the panic in the first place.
  *
  */
 /*VARARGS2*/
@@ -384,7 +383,6 @@ panic(exitcode, fmt, va_alist)
     va_dcl                              /* arguments for printf */
 #endif
 {
-    static int panic_count = 0;		/* panic not yet called once */
     va_list ap;
 
     if (errfile) {
@@ -396,63 +394,27 @@ panic(exitcode, fmt, va_alist)
 	write_log_va(WRITE_LOG_TTY, fmt, ap);
 	va_end(ap);
     }
-    /*
-     * if panic has been called before, but the panic log is not open
-     * write to the console.
-     *
-     * NOTE:  This will happen when open_system_logs calls panic because
-     *	      it could not open the panic log file.
-     */
-    if ((panicfile == NULL && panic_count == 1) || panic_count == 2) {
-	FILE *consfile;
-	static char consbuf[BUFSIZ];
+#ifdef ANSI_C
+    va_start(ap, fmt);
+#else
+    va_start(ap);
+#endif
+    write_log_va(WRITE_LOG_PANIC, fmt, ap);
+    va_end(ap);
 
-	consfile = fopen(cons_fn, "a");
-	if (consfile == NULL) {
-	    /*
-	     * not many situations are more difficult to handle than
-	     * this one--punt.
-	     */
-	    if (force_zero_exitvalue) {
-		/* if this is set, always exit with 0 */
-		exit(0);
-	    }
-	    exit(exitcode);
-	}
-	(void)setbuf(consfile, consbuf); /* avoid use of malloc */
-	(void)fprintf(consfile, "%s %s: mailer error: ",
-		      program, time_stamp());
-	if (message_id) {
-	    (void)fprintf(consfile, "[%s] ", message_id);
-	} else {
-	    (void)fprintf(consfile, "[%d] ", getpid());
-	}
 #ifdef ANSI_C
-	va_start(ap, fmt);
+    va_start(ap, fmt);
 #else
-	va_start(ap);
+    va_start(ap);
 #endif
-	(void)vfprintf(consfile, fmt, ap);
-	va_end(ap);
-	/* log messages don't come with \n */
-	(void)fprintf(consfile, "\r\n");
-	(void)fclose(consfile);		/* flush */
-    } else {
-	panic_count++;			/* help stamp out recursion */
-#ifdef ANSI_C
-	va_start(ap, fmt);
-#else
-	va_start(ap);
-#endif
-	write_log_va(WRITE_LOG_PANIC, fmt, ap);
-	va_end(ap);
-    }
+    write_log_va(WRITE_LOG_SYS, fmt, ap);
+    va_end(ap);
 
     if (daemon_pid != 0 && daemon_pid == getpid()) {
-	write_log(WRITE_LOG_SYS, "pid: smail daemon exiting on panic");
+	write_log(WRITE_LOG_SYS, "smail daemon exiting on panic");
     }
     if (spool_fn) {
-	defer_message();		/* put message in error/ directory */
+	freeze_message();		/* put message in error/ directory */
     }
     if (force_zero_exitvalue) {
 	exit(0);
@@ -544,11 +506,11 @@ write_log_va(who, fmt, ap)
     char *fmt;				/* printf(3) format */
     va_list ap;                         /* arguments for printf */
 {
+    int wct = 0;
+
     switch (who) {
 #ifndef NODEBUG
-    case WRITE_LOG_DEBUG: {
-	int wct = 0;
-
+    case WRITE_LOG_DEBUG:
 	/* if we are debugging at all, print logging messages */
 	(void) fprintf(errfile, "write_log:");
 
@@ -558,94 +520,81 @@ write_log_va(who, fmt, ap)
 	    wct++;
 	}
 	if (debug > 1 && (who & WRITE_LOG_PANIC)) {
-	    (void) fprintf(errfile, "%cPANIC", wct++? '|': ' ');
+	    (void) fprintf(errfile, "%cPANIC", wct++ ? '|' : ' ');
 	}
 	if (debug > 1 && (who & WRITE_LOG_MLOG)) {
-	    (void) fprintf(errfile, "%cMLOG", wct++? '|': ' ');
+	    (void) fprintf(errfile, "%cMLOG", wct++ ? '|' : ' ');
 	}
 	if (debug > 1 && (who & WRITE_LOG_TTY)) {
-	    (void) fprintf(errfile, "%cTTY", wct++? '|': ' ');
+	    (void) fprintf(errfile, "%cTTY", wct++ ? '|' : ' ');
 	}
 	if (debug > 1) {
 	    (void) fprintf(errfile, " ");
 	}
-	(void)vfprintf(errfile, fmt, ap);
-	/* log messages don't come with \n */
-	(void)fprintf(errfile, "\n");
-	(void)fflush(errfile);
-    }
-    break;
+	(void) vfprintf(errfile, fmt, ap);
+	/* log messages don't come with their own \n */
+	(void) fprintf(errfile, "\n");
+	(void) fflush(errfile);
+	break;
 
 #endif	/* NODEBUG */
 
     case WRITE_LOG_TTY:
-    {
-	(void) fprintf(errfile, "%s: ", program);
+	(void) fprintf(errfile, "%s: [%lu] ", program, (long unsigned int) getpid());
 	if (message_id) {
-	    (void)fprintf(errfile, "[%s] ", message_id);
-	} else {
-	    (void)fprintf(errfile, "[%d] ", getpid());
+	    (void) fprintf(errfile, "[%s] ", message_id);
 	}
 	(void) vfprintf(errfile, fmt, ap);
-	/* log messages don't come with \n */
+	/* log messages don't come with their own \n */
 	(void) fprintf(errfile, "\n");
 	(void) fflush(errfile);
-    }
-    break;
+	break;
 
     case WRITE_LOG_SYS:
-    {
 	/* write out permanent log to the system log file */
 	if (logfile == NULL || panicfile == NULL) {
 	    open_system_logs();		/* if system log not open, open it */
 	}
 #ifdef LOG_EXTRA_NEWLINE
-	(void)fprintf(logfile, "\n%s: ", time_stamp());
+	(void) fprintf(logfile, "\n%s: [%lu] ", time_stamp(), (long unsigned int) getpid());
 #else
-	(void)fprintf(logfile, "%s: ", time_stamp());
+	(void) fprintf(logfile, "%s: [%lu] ", time_stamp(), (long unsigned int) getpid());
 #endif
 	if (message_id) {
-	    (void)fprintf(logfile, "[%s] ", message_id);
-	} else {
-	    (void)fprintf(logfile, "[%d] ", getpid());
+	    (void) fprintf(logfile, "[%s] ", message_id);
 	}
-	(void)vfprintf(logfile, fmt, ap);
-	/* log messages don't come with \n */
-	(void)fprintf(logfile, "\n");
-	(void)fflush(logfile);
+	(void) vfprintf(logfile, fmt, ap);
+	/* log messages don't come with their own \n */
+	(void) fprintf(logfile, "\n");
+	(void) fflush(logfile);
 	if (ferror(logfile)) {
 	    panic(EX_IOERR, "error writing %s: %s", log_fn, strerror(errno));
 	    /*NOTREACHED*/
 	}
-    }
-    break;
+	break;
 
     case WRITE_LOG_PANIC:
-    {
 	if (panicfile == NULL) {
 	    open_system_logs();		/* if system log not open, open it */
 	}
 #ifdef LOG_EXTRA_NEWLINE
-	(void)fprintf(panicfile, "\n%s: ", time_stamp());
+	(void) fprintf(panicfile, "\n%s: [%lu] ", time_stamp(), (long unsigned int) getpid());
 #else
-	(void)fprintf(panicfile, "%s: ", time_stamp());
+	(void) fprintf(panicfile, "%s: [%lu] ", time_stamp(), (long unsigned int) getpid());
 #endif
 	if (message_id) {
-	    (void)fprintf(panicfile, "[%s] ", message_id);
-	} else {
-	    (void)fprintf(panicfile, "[%d] ", getpid());
+	    (void) fprintf(panicfile, "[%s] ", message_id);
 	}
-	(void)vfprintf(panicfile, fmt, ap);
-	/* log messages don't come with \n */
-	(void)fprintf(panicfile, "\n");
-	(void)fflush(panicfile);
+	(void) vfprintf(panicfile, fmt, ap);
+	/* log messages don't come with their own \n */
+	(void) fprintf(panicfile, "\n");
+	(void) fflush(panicfile);
 	if (ferror(panicfile)) {
 	    panicfile = NULL;
 	    panic(EX_IOERR, "error writing %s: %s", log_fn, strerror(errno));
 	    /*NOTREACHED*/
 	}
-    }
-    break;
+	break;
 
     /*
      * NOTE:  if there is no spool_dir set, then a per-message logfile
@@ -659,10 +608,10 @@ write_log_va(who, fmt, ap)
 	    }
 
 	    /* write out the message to the per-message log file */
-	    (void)vfprintf(msg_logfile, fmt, ap);
-	    /* log messages don't come with \n */
-	    (void)fprintf(msg_logfile, "\n");
-	    (void)fflush(msg_logfile);
+	    (void) vfprintf(msg_logfile, fmt, ap);
+	    /* log messages don't come with their own \n */
+	    (void) fprintf(msg_logfile, "\n");
+	    (void) fflush(msg_logfile);
 	    if (ferror(msg_logfile)) {
 		panic(EX_IOERR, "error writing %s/%s: %s",
 		      spool_dir, msg_log_fn, strerror(errno));
@@ -751,7 +700,7 @@ scan_msg_log(first)
 {
     static FILE *mlog = NULL;		/* opened message log */
     static struct str line;		/* line marked with X */
-    static inited_line = FALSE;		/* TRUE if STR_INIT called for line */
+    static int inited_line = FALSE;	/* TRUE if STR_INIT called for line */
     register int c, last_c;
 
     build_msg_log_fn();
@@ -790,13 +739,25 @@ scan_msg_log(first)
 	STR_CLEAR(&line);
     }
     STR_NEXT(&line, 'X');
-    while ((c = getc(mlog)) != EOF && c != '\n') {
+    last_c = 'X';
+    while ((c = getc(mlog)) != EOF) {
+	if (last_c == '\n' && c == 'X') {
+	    break;
+	}
 	STR_NEXT(&line, c);
+	last_c = c;
+    }
+    if (c == 'X') {
+	ungetc(c, mlog);		/* back for next time */
+    }
+    if (last_c == '\n') {
+	STR_PREV(&line);		/* trim off the last newline */
     }
     STR_NEXT(&line, '\0');
 
     /* return that line */
-    DEBUG1(DBG_LOG_HI, "scan_msg_log returns: %s\n", STR(&line));
+    DEBUG1(DBG_LOG_HI, "scan_msg_log() returns:\n'%s'\n", STR(&line));
+
     return STR(&line);
 }
 
@@ -824,7 +785,7 @@ char *sender_name = "MasterControlProgram";
 char *local_sender;
 int force_zero_exitvalue = FALSE;
 FILE *errfile = stderr;
-int daemon_pid = 0xdead;
+long unsigned int daemon_pid = 4000000000;	/* 0xEE6B2800 */
 int real_uid = 0;
 
 char *
