@@ -1,4 +1,6 @@
-/* @(#) log.h,v 1.5 1998/04/11 23:30:29 woods Exp */
+/*
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:log.h,v 1.6 2000/09/03 16:26:31 woods Exp"
+ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -22,3 +24,14 @@
 #define WRITE_LOG_TTY	0x0010		/* write_log sends to stderr */
 
 #define WRITE_LOG_LAST	WRITE_LOG_TTY	/* the one with the highest bit set */
+
+/* external functions defined in log.c */
+extern void open_system_logs __P((void));
+extern void close_system_logs __P((void));
+extern void open_msg_log __P((void));
+extern void close_msg_log __P((void));
+extern void unlink_msg_log __P((void));
+extern void panic __P((int, char *, ...));
+extern void write_log __P((int, char *, ...));
+extern void send_log __P((FILE *, int, char *));
+extern char *scan_msg_log __P((int));
