@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:lookup.c,v 1.49 2000/02/05 20:59:41 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:lookup.c,v 1.53 2000/09/29 21:20:07 woods Exp"
  */
 
 /*
@@ -21,12 +21,13 @@
  * NeXT NetInfo aliases support added by Dan Danz <dan@az.stratus.com>
  */
 
-#include <stdio.h>
-#include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <stdio.h>
+#include <ctype.h>
 #include <signal.h>
 #include <errno.h>
+#define NEED_SOCKETS				/* for match_ip(), etc. */
 #include "defs.h"
 #ifdef	UNIX_BSD
 # include <sys/file.h>
@@ -50,15 +51,23 @@
 #endif	/* not POSIX_OS */
 
 #include "smail.h"
+#include "main.h"
+#include "parse.h"
+#include "addr.h"
 #include "alloc.h"
 #include "dys.h"
 #include "log.h"
 #include "lookup.h"			/* XXX only really needed with LIBWHOSON for now */
 #include "exitcodes.h"
+#include "smailconf.h"
 
 #ifdef HAVE_NIALIAS
 # include <aliasdb.h>
 #endif
+/*
+ * XXX WARNING: on SunOS-5.6 you'll get a T_UNSPEC redefined warning in
+ * <sys/tpicommon.h> as it is also defined in <arpa/nameser.h>....
+ */
 #ifdef  HAVE_NISPLUS
 # include <rpcsvc/nis.h>
 #endif
@@ -109,67 +118,67 @@ static int	regerr = 0;		/* error code from RE compiler */
 # include <whoson.h>
 #endif
 
-#ifdef ANSI_C
-# define VOLATILE	static volatile
-#else
-# define VOLATILE	/* most compilers won't optimize global variables */
-#endif
-
 #ifdef STANDALONE
 # define xmalloc malloc
 # define xrealloc realloc
 # define xfree free
 #endif	/* STANDALONE */
 
-/* variables imported from libc */
-extern int errno;
-
-/* functions local to this file */
-static int bsearch_open();
-static void bsearch_close();
-static int bsearch_lookup();
-
-static int lsearch_open();
-static void lsearch_close();
-static int lsearch_lookup();
-
-static int dbmbase_open();
-static void dbmbase_close();
-static int dbmbase_lookup();
-
-#ifdef	HAVE_YP
-static int yp_open();
-static void yp_close();
-static int yp_lookup();
-
-static int aliasyp_open();
-static void aliasyp_close();
-static int aliasyp_lookup();
-#endif
-
-#ifdef HAVE_NIALIAS
-static int aliasni_open();
-static void aliasni_close();
-static int aliasni_lookup();
-#endif
-
-#ifdef  HAVE_NISPLUS
-static int nisplus_open();
-static void nisplus_close();
-static int nisplus_lookup();
-#endif
-
+/* generic form of structure returned by database open calls */
+struct generic_db {
+    struct proto *proto;		/* access method */
+};
 
 /*
  * the following structure defines the available access methods.
  * Methods are identified by name when open_database is called.
  */
-static struct proto {
-    char *proto;			/* name of the access method */
-    int (*open)();			/* open database function */
-    void (*close)();			/* close database function */
-    int (*lookup)();			/* lookup in database function */
-} protos[] = {
+ struct proto {
+    char *proto;				/* name of the access method */
+    int (*open) __P((char *, struct proto *, int, unsigned int, struct stat *, char **, char **)); /* open database function */
+    void (*close) __P((struct generic_db *));	/* close database function */
+    int (*lookup) __P((struct generic_db *, char *, char **, char **)); /* lookup in database function */
+};
+
+/* functions local to this file */
+static int bsearch_open __P((char *, struct proto *, int, unsigned int, struct stat *, char **, char **));
+static void bsearch_close __P((struct generic_db *));
+static int bsearch_lookup __P((struct generic_db *, char *, char **, char **));
+
+static int lsearch_open __P((char *, struct proto *, int, unsigned int, struct stat *, char **, char **));
+static void lsearch_close __P((struct generic_db *));
+static int lsearch_lookup __P((struct generic_db *, char *, char **, char **));
+#ifdef USE_LSEARCH_REGEXCMP
+static int lsearch_regexcmp __P((char *, char *, int *));
+#endif
+
+static int dbmbase_open __P((char *, struct proto *, int, unsigned int, struct stat *, char **, char **));
+static void dbmbase_close __P((struct generic_db *));
+static int dbmbase_lookup __P((struct generic_db *, char *, char **, char **));
+
+#ifdef	HAVE_YP
+static int yp_open __P((char *, struct proto *, int, unsigned int, struct stat *, char **, char **));
+static void yp_close __P((struct generic_db *));
+static int yp_lookup __P((struct generic_db *, char *, char **, char **));
+
+static int aliasyp_open __P((char *, struct proto *, int, unsigned int, struct stat *, char **, char **));
+static void aliasyp_close __P((struct generic_db *));
+static int aliasyp_lookup __P((struct generic_db *, char *, char **, char **));
+#endif
+
+#ifdef HAVE_NIALIAS
+static int aliasni_open __P((char *, struct proto *, int, unsigned int, struct stat *, char **, char **));
+static void aliasni_close __P((struct generic_db *));
+static int aliasni_lookup __P((struct generic_db *, char *, char **, char **));
+#endif
+
+#ifdef  HAVE_NISPLUS
+static int nisplus_open __P((char *, struct proto *, int, unsigned int, struct stat *, char **, char **));
+static void nisplus_close __P((struct generic_db *));
+static int nisplus_lookup __P((struct generic_db *, char *, char **, char **));
+#endif
+
+static struct proto protos[] = {
     { "bsearch",			/* binary search */
       bsearch_open, bsearch_close, bsearch_lookup },
     { "lsearch",
@@ -194,16 +203,10 @@ static struct proto {
     { NULL, NULL, NULL, NULL }
 #endif
 };
-
 /* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
 /* point to end of protos table */
 struct proto *end_protos = protos + sizeof(protos)/sizeof(protos[0]);
 
-/* generic form of structure returned by database open calls */
-struct generic_db {
-    struct proto *proto;		/* access method */
-};
-
 /*
  * open_database - open a database of the specified type
  *
@@ -227,7 +230,7 @@ open_database(name, proto, retries, interval, statp, db, error)
     char *name;				/* name of database */
     char *proto;			/* access method name */
     int retries;			/* retry count */
-    int interval;			/* retry interval */
+    unsigned int interval;		/* retry interval */
     struct stat *statp;			/* return a stat buffer */
     char **db;				/* store open database info here */
     char **error;			/* store error message here */
@@ -329,7 +332,7 @@ bsearch_open(name, proto, retries, interval, statp, db, error)
     char *name;				/* name of file */
     struct proto *proto;		/* access method */
     int retries;			/* retry count */
-    int interval;			/* retry interval */
+    unsigned int interval;		/* retry interval */
     struct stat *statp;			/* save stat results here */
     char **db;				/* store open database info here */
     char **error;			/* store error message here */
@@ -388,9 +391,11 @@ bsearch_open(name, proto, retries, interval, statp, db, error)
 
 /* bsearch_close - close the file */
 static void
-bsearch_close(db)
-    struct bsearch_db *db;
+bsearch_close(gdb)
+    struct generic_db *gdb;
 {
+    register struct bsearch_db *db = (struct bsearch_db *) gdb;
+
     DEBUG1(DBG_DRIVER_HI, "bsearch_close: called to close %s.\n", db->name);
 
     (void) fclose(db->f);
@@ -407,8 +412,8 @@ bsearch_close(db)
  */
 /*ARGSUSED*/
 static int
-bsearch_lookup(db, key, value, error)
-    register struct bsearch_db *db;
+bsearch_lookup(gdb, key, value, error)
+    register struct generic_db *gdb;
     char *key;
     char **value;
     char **error;
@@ -419,6 +424,7 @@ bsearch_lookup(db, key, value, error)
     static struct str str;		/* string in which to store data */
     static int str_inited = FALSE;	/* TRUE if str has been STR_INIT'd */
     int i;				/* temp */
+    register struct bsearch_db *db = (struct bsearch_db *) gdb;
 
     DEBUG2(DBG_DRIVER_HI, "bsearch_lookup: looking for <%s> in %s.\n", key, db->name);
 
@@ -527,7 +533,7 @@ lsearch_open(name, proto, retries, interval, statp, db, error)
     char *name;				/* name of file */
     struct proto *proto;		/* access method */
     int retries;			/* retry count */
-    int interval;			/* retry interval */
+    unsigned int interval;		/* retry interval */
     struct stat *statp;			/* save stat results here */
     char **db;				/* store open database info here */
     char **error;			/* store error message here */
@@ -582,9 +588,11 @@ lsearch_open(name, proto, retries, interval, statp, db, error)
 
 /* lsearch_close - close the file */
 static void
-lsearch_close(db)
-    struct lsearch_db *db;
+lsearch_close(gdb)
+    struct generic_db *gdb;
 {
+    register struct lsearch_db *db = (struct lsearch_db *) gdb;
+
     DEBUG1(DBG_DRIVER_HI, "lsearch_close: called to close %s.\n", db->name);
 
     (void) fclose(db->f);
@@ -692,15 +700,16 @@ lsearch_regexcmp(entry, key, len)
  */
 /*ARGSUSED*/
 static int
-lsearch_lookup(db, key, value, error)
-    register struct lsearch_db *db;
+lsearch_lookup(gdb, key, value, error)
+    register struct generic_db *gdb;
     char *key;
     char **value;
     char **error;
 {
-    int len;				/* length of comparison */
+    unsigned int len;			/* length of comparison */
     register char *entry;		/* entry from read_entry() */
     register char *p;			/* temp */
+    register struct lsearch_db *db = (struct lsearch_db *) gdb;
 
     DEBUG2(DBG_DRIVER_HI, "lsearch_lookup: looking for <%s> in %s.\n", key, db->name);
 
@@ -808,7 +817,7 @@ dbmbase_open(name, proto, retries, interval, statp, db, error)
     char *name;				/* name of database */
     struct proto *proto;		/* access method */
     int retries;			/* retry count */
-    int interval;			/* retry interval */
+    unsigned int interval;		/* retry interval */
     struct stat *statp;			/* return a stat structure */
     char **db;				/* store open database info here */
     char **error;			/* store error message here */
@@ -900,9 +909,11 @@ dbmbase_open(name, proto, retries, interval, statp, db, error)
 
 /* dbmbase_close - close the database */
 static void
-dbmbase_close(db)
-    struct dbmbase_db *db;
+dbmbase_close(gdb)
+    struct generic_db *gdb;
 {
+    register struct dbmbase_db *db = (struct dbmbase_db *) gdb;
+
     DEBUG1(DBG_DRIVER_HI, "dbmbase_close: called to close %s.\n", db->name);
 
 #ifdef HAVE_NDBM
@@ -920,17 +931,18 @@ dbmbase_close(db)
 
 /* dbmbase_lookup - call on the DBM routines to find that data */
 static int
-dbmbase_lookup(db, key, value, error)
-    register struct dbmbase_db *db;
+dbmbase_lookup(gdb, key, value, error)
+    register struct generic_db *gdb;
     char *key;
     char **value;
     char **error;
 {
     datum the_key;
     datum the_value;
-    static int temp_size;		/* size of temp_key area */
+    static unsigned int temp_size;	/* size of temp_key area */
     static char *temp_data = NULL;	/* growable temp area */
     int len;				/* length of key */
+    register struct dbmbase_db *db = (struct dbmbase_db *) gdb;
 
 #ifndef HAVE_NDBM
     if (opendb != db) {
@@ -1023,16 +1035,16 @@ struct yp_db {
 };
 
 static char *default_yp_domain = NULL;	/* from yp_get_default_domain(3N) */
-static int common_yp_lookup();
+static int common_yp_lookup __P((register struct yp_db *, char *, char **, char **, int));
 
 #ifdef notyet
 
-VOLATILE int save_time;			/* saved value from alarm() */
-VOLATILE JUMPSIG save_sigalrm;		/* previous SIGALRM handler */
+static volatile int save_time;		/* saved value from alarm() */
+static volatile JUMPSIG save_sigalrm;	/* previous SIGALRM handler */
 
 static JUMP_ENVBUF alarm_jmp;		/* jump here on SIGALRM */
 
-static void yp_sigalrm();		/* catch SIGALRM for YP timeouts */
+static void yp_sigalrm __P((void));	/* catch SIGALRM for YP timeouts */
 
 #define YP_TIMEOUT	30		/* 30 second timeout for YP */
 #endif
@@ -1047,7 +1059,7 @@ aliasyp_open(name, proto, retries, interval, statp, db, error)
     char *name;				/* name of database */
     struct proto *proto;		/* access method */
     int retries;			/* retry count */
-    int interval;			/* retry interval */
+    unsigned int interval;		/* retry interval */
     struct stat *statp;			/* return a stat structure */
     char **db;				/* store open database info here */
     char **error;			/* store error message here */
@@ -1063,7 +1075,7 @@ yp_open(name, proto, retries, interval, statp, db, error)
     char *name;				/* name of database */
     struct proto *proto;		/* access method */
     int retries;			/* retry count */
-    int interval;			/* retry interval */
+    unsigned int interval;		/* retry interval */
     struct stat *statp;			/* return a stat structure */
     char **db;				/* store open database info here */
     char **error;			/* store error message here */
@@ -1071,8 +1083,11 @@ yp_open(name, proto, retries, interval, statp, db, error)
     register struct yp_db *priv;
     register char *p, *pname;
     int err;				/* error from yp functions */
-    /* XXX: third arg to yp_order() is (ulong *) on Solaris-2 */
+#if defined(__svr4__) && defined(__sun__)	/* Solaris-2.x */
+    unsigned long order;		/* output from yp_order */
+#else  /* *BSD and the rest */
     int order;				/* output from yp_order */
+#endif
     char *domain;
 
     priv = (struct yp_db *)xmalloc(sizeof(*priv));
@@ -1170,48 +1185,56 @@ yp_open(name, proto, retries, interval, statp, db, error)
 
 /* yp_close, aliasyp_close - free up data allocated for to a YP database */
 static void
-aliasyp_close(db)
-    struct yp_db *db;
+aliasyp_close(gdb)
+    struct generic_db *gdb;
 {
+    register struct yp_db *db = (struct yp_db *) gdb;
+
     DEBUG1(DBG_DRIVER_HI, "aliasyp_close: called to close %s.\n", db->map);
 
-    yp_close(db);
+    yp_close(gdb);
 }
 
 /*ARGSUSED*/
 static void
-yp_close(db)
-    struct yp_db *db;
+yp_close(gdb)
+    struct generic_db *gdb;
 {
+    register struct yp_db *db = (struct yp_db *) gdb;
+
     if (db->domain) {
 	xfree(db->domain);
     } else {
 	xfree(db->map);
     }
-    xfree((char *)db);
+    xfree((char *) db);
     return;
 }
 
 /* yp_lookup, aliasyp_lookup - call on the YP routines to find that data */
 static int
-yp_lookup(db, key, value, error)
-    register struct yp_db *db;		/* open YP database */
+yp_lookup(gdb, key, value, error)
+    register struct generic_db *gdb;	/* open YP database */
     char *key;				/* search key */
     char **value;			/* return value here */
     char **error;			/* return error message here */
 {
+    register struct yp_db *db = (struct yp_db *) gdb;
+
     DEBUG2(DBG_DRIVER_HI, "yp_lookup: looking for <%s> in %s.\n", key, db->map);
 
     return common_yp_lookup(db, key, value, error, FALSE);
 }
 
 static int
-aliasyp_lookup(db, key, value, error)
-    register struct yp_db *db;		/* open YP database */
+aliasyp_lookup(gdb, key, value, error)
+    register struct generic_db *gdb;	/* open YP database */
     char *key;				/* search key */
     char **value;			/* return value here */
     char **error;			/* return error message here */
 {
+    register struct yp_db *db = (struct yp_db *) gdb;
+
     DEBUG2(DBG_DRIVER_HI, "aliasyp_lookup: looking for <%s> in %s.\n", key, db->map);
 
     return common_yp_lookup(db, key, value, error, TRUE);
@@ -1228,7 +1251,7 @@ common_yp_lookup(db, key, value, error, aliasflag)
 {
     int keylen;				/* length of key */
     static char *temp_key = NULL;	/* area for lower-case conversion */
-    static int temp_size;		/* size of temp area */
+    static unsigned int temp_size = 0;	/* size allocated for temp_key */
     register char *p;
     char *matchval;			/* matched data */
     int matchlen;			/* length of matched data */
@@ -1249,7 +1272,7 @@ common_yp_lookup(db, key, value, error, aliasflag)
 	temp_key = xrealloc(temp_key, temp_size);
     }
     for (p = temp_key; *key; p++, key++) {
-	*p = lowercase(*key);
+	*p = lowercase((int) *key);
     }
     *p = '\0';
 
@@ -1355,7 +1378,7 @@ aliasni_open(name, proto, retries, interval, statp, db, error)
     char *name;			/* name of file */
     struct proto *proto;	/* access method */
     int retries;		/* retry count */
-    int interval;		/* retry interval */
+    unsigned int interval;	/* retry interval */
     struct stat *statp;		/* save stat results here */
     char **db;			/* store open database info here */
     char **error;		/* store error message here */
@@ -1385,9 +1408,11 @@ aliasni_open(name, proto, retries, interval, statp, db, error)
 
 /* aliasni_close - close the NetInfo alias database */
 static void
-aliasni_close(db)
-    struct aliasni_db *db;
+aliasni_close(gdb)
+    struct generic_db *gdb;
 {
+    register struct aliasni_db *db = (struct aliasni_db *) gdb;
+
     DEBUG1(DBG_DRIVER_HI, "aliasni_close: called to close %s.\n", db->name);
 
     alias_endent();
@@ -1399,8 +1424,8 @@ aliasni_close(db)
 
 /*ARGSUSED*/
 static int
-aliasni_lookup(db, key, value, error)
-    register struct aliasni_db *db;
+aliasni_lookup(gdb, key, value, error)
+    register struct generic_db *gdb;
     char *key;
     char **value;
     char **error;
@@ -1408,6 +1433,7 @@ aliasni_lookup(db, key, value, error)
     aliasent    *a;
     int i,n;
     char        *p;
+    register struct aliasni_db *db = (struct aliasni_db *) gdb;
 
     DEBUG1(DBG_DRIVER_HI, "aliasni_lookup: looking for <%s> in %s.\n", key, db->name);
 
@@ -1481,8 +1507,8 @@ struct nisplus_db {
     char		*name;
     char		*table;
     int			fieldnum;
-    int			retries,	/* saved retry/interval for lookup */
-			interval;
+    int			retries;	/* saved retry for lookup */
+    unsigned int	interval;	/* saved interval for lookup */
 };
 
 /* open a NIS+ database */
@@ -1491,14 +1517,14 @@ nisplus_open(name, proto, retries, interval, statp, db, error)
     char *name;				/* name of file */
     struct proto *proto;		/* access method */
     int retries;			/* retry count */
-    int interval;			/* retry interval */
+    unsigned int interval;		/* retry interval */
     struct stat *statp;			/* save stat results here */
     char **db;				/* store open database info here */
     char **error;			/* store error message here */
 {
     register struct nisplus_db	*priv;
     nis_result			*result;
-    int				flags;
+    unsigned int		flags;
     char			*domain;
 
     DEBUG1(DBG_DRIVER_HI, "nisplus_open: called to open %s.\n", name);
@@ -1522,7 +1548,7 @@ nisplus_open(name, proto, retries, interval, statp, db, error)
         } else {
 	    /* Yech!  No commas!  This is a failure condition */
 	    *error = "syntax error in name given to nisplus proto";
-	    xfree(priv);
+	    xfree((char *) priv);
 	    return FILE_FAIL;
 	}
     } else {
@@ -1558,14 +1584,14 @@ nisplus_open(name, proto, retries, interval, statp, db, error)
 		    *error = "multiple results - NIS+ table name is ambiguous";
 		    nis_freeresult(result);
 		    DEBUG(DBG_DRIVER_HI, "nisplus_open: more than 1 value\n");
-	            xfree(priv);
+	            xfree((char *) priv);
 		    return FILE_FAIL;
 		}
 		if ( result->objects.objects_val[0].zo_data.zo_type != TABLE_OBJ ) {
 		    *error = "table open attempted on non-table NIS+ object";
 		    nis_freeresult(result);
 		    DEBUG(DBG_DRIVER_HI, "nisplus_open: not a table\n");
-	            xfree(priv);
+	            xfree((char *) priv);
 		    return FILE_FAIL;
 		}
 
@@ -1594,7 +1620,7 @@ nisplus_open(name, proto, retries, interval, statp, db, error)
 		*error = nis_sperror(result->status, "nisplus_open");
 		nis_freeresult(result);
 	        DEBUG1(DBG_DRIVER_HI, "nisplus_open: <%s> not found\n", domain);
-	        xfree(priv);
+	        xfree((char *) priv);
 		return FILE_NOMATCH;
 
 	    /* failure - try again later - FILE_AGAIN */
@@ -1605,7 +1631,7 @@ nisplus_open(name, proto, retries, interval, statp, db, error)
 		*error = nis_sperror(result->status, "nisplus_open");
 		nis_freeresult(result);
 	        DEBUG1(DBG_DRIVER_HI, "nisplus_open: try again <%s>\n", domain);
-	        xfree(priv);
+	        xfree((char *) priv);
 		return FILE_AGAIN;
 
 	    /* failure - permanent failure on this object - FILE_FAIL */
@@ -1615,7 +1641,7 @@ nisplus_open(name, proto, retries, interval, statp, db, error)
 		*error = nis_sperror(result->status, "nisplus_open");
 		nis_freeresult(result);
 	        DEBUG1(DBG_DRIVER_HI, "nisplus_open: <%s> not found\n", domain);
-	        xfree(priv);
+	        xfree((char *) priv);
 		return FILE_FAIL;
 	}
 
@@ -1632,41 +1658,43 @@ nisplus_open(name, proto, retries, interval, statp, db, error)
     *error = nis_sperror(result->status, "nisplus_open");
     nis_freeresult(result);
     DEBUG1(DBG_DRIVER_HI, "nisplus_open: try again <%s>\n", domain);
-    xfree(priv);
+    xfree((char *) priv);
     return FILE_AGAIN;
 }
 
 /* close a NIS+ database */
 static void
-nisplus_close(db)
-    struct nisplus_db *db;
+nisplus_close(gdb)
+    struct generic_db *gdb;
 {
+    register struct nisplus_db *db = (struct nisplus_db *) gdb;
+
     DEBUG1(DBG_DRIVER_HI, "nisplus_close: called to close %s.\n", db->name);
 
     xfree(db->name);
-    xfree((char *)db);
+    xfree((char *) db);
     return;
 }
 
 /* look up an element of a NIS+ database */
 /*ARGSUSED*/
 static int
-nisplus_lookup(db, key, value, error)
-    register struct nisplus_db *db;
+nisplus_lookup(gdb, key, value, error)
+    register struct generic_db *gdb;
     char *key;
     char **value;
     char **error;
 {
     nis_result			*result;
     struct nis_object		*objbuf;
-    int				flags,
-				col,
-				retries,
-				valsize;
-    char			*name,
-				*valbuf;
+    unsigned int		flags;
+    int				retries;
+    int				valsize;
+    char			*name;
+    char			*valbuf;
+    register struct nisplus_db *db = (struct nisplus_db *) gdb;
 
-    valsize=0;
+    valsize = 0;
     retries = db->retries;
 
     /* make the name to look up */
@@ -1820,26 +1848,40 @@ nisplus_lookup(db, key, value, error)
 
 
 
+#ifdef HAVE_BSD_NETWORKING
+
 /*
  * match_ip - match an ip against an allowed list
  *
- * Takes a list of IP:IP:IP and compares against an IP string.  Allows
- * "*" star type wildcards at the end of IPs.
+ * Takes a list of IP:IP:IP and compares against an IP string.
+ *
+ * 'IP' is an IP number or IP/bits CIDR network spec.  IP may optionally be
+ * followed by ';message text', which of course must not contain a ':'
+ *
+ * The field separator show above as ':' is specified by the caller.  The
+ * message sub-field separator show above as ';' is specified by the caller.
+ *
+ * TODO:
+ *	- add support for negation, must use last-match-wins!
  *
  * WARNING: cannot diddle with pattern -- may be read-only string.
  *
  * Returns 1 if matched, else 0.
  */
 int
-match_ip(ip, patterns)
+match_ip(ip, patterns, fldsep, msgsep, reasonp)
     char *ip;			/* ip address we are matching */
     char *patterns;		/* list of patterns we are using to match it */
+    int fldsep;			/* field separator character */
+    int msgsep;			/* optional message string separator */
+    char **reasonp;		/* optional pointer for returned message string */
 {
-    char	*pat_p, *end_p, wild = 0;
-    int		len;
+    char *pat_p;
+    char *end_p;
+    unsigned int len;
     
     /* run through the pattern list */
-    for (pat_p = patterns; /* NOTEST */ ; pat_p = end_p + 1, wild = 0) {
+    for (pat_p = patterns; /* NOTEST */ ; pat_p = end_p + 1) {
 
 	/* skip any spaces at front */
 	while (*pat_p && isspace(*pat_p)) {
@@ -1847,41 +1889,54 @@ match_ip(ip, patterns)
 	}
 	/* find the end of the next pattern */
 	end_p = pat_p;
-	while (*end_p && *end_p != ':' && !isspace(*pat_p)) {
+	while (*end_p && *end_p != fldsep && !isspace(*pat_p)) {
 	       end_p++;
 	}
 	/* skip empty patterns */
 	len = end_p - pat_p;
 	if (len > 0) {
-	    /* do we have a wild-card at end of pattern? */
-	    if (*(end_p - 1) == '*') {
-		len--;			/* may be set to zero if pattern is "*" */
-		wild = 1;
-	    }
-	    if (smtp_local_net && len > 0 && strncmpic("localnet", pat_p, len) == 0) {
-		DEBUG1(DBG_ADDR_HI,
-		       "(checking against 'localnet' pattern of '%s')\n",
-			smtp_local_net);
-		/* our pattern will be the string in smtp_local_net */
-		len = strlen(smtp_local_net);
-		if (smtp_local_net[len - 1] == '*') {
-		    len--;
-		    wild = 1;
+	    char *curpat;
+	    char *p;
+
+	    curpat = xmalloc(len + 1);
+	    strncpy(curpat, pat_p, len);
+	    curpat[len] = '\0';
+
+	    if (msgsep) {
+		if ((p = strchr(curpat, msgsep))) {
+		    *p = '\0';
+		    if (reasonp) {
+			*reasonp = COPY_STRING(p + 1);
+		    }
+		} else if (reasonp) {
+		    *reasonp = NULL;
 		}
-		/* have match if pat was wild or we checked the whole IP */
-		if (len && strncmp(ip, smtp_local_net, len) == 0 && (wild || ip[len] == '\0')) {
-		    DEBUG3(DBG_ADDR_MID,
-			   "ip '%s' verified by localnet '%s', up to len %d\n",
-			   ip, smtp_local_net, len);
+	    }
+	    /* warn if we have a wild-card at end of the current pattern? */
+	    if (*(end_p - 1) == '*') {
+		write_log(WRITE_LOG_PANIC, "match_ip(): an unsupported wildcard pattern [%s] was found!", curpat);
+	    }
+	    if (smtp_local_net && strcmpic("localnet", curpat) == 0) {
+		DEBUG2(DBG_ADDR_HI,
+		       "match_ip(): found 'localnet' in list...\n",
+		       ip,
+		       smtp_local_net);
+		/* our pattern will be the string in smtp_local_net */
+		if (match_ip_net(ip, smtp_local_net)) {
+		    DEBUG2(DBG_ADDR_MID,
+			   "ip [%s] matches localnet [%s]\n",
+			   ip, smtp_local_net);
+		    xfree(curpat);
 		    return 1;
 		}
+		/* continue with next pattern */
 #ifdef HAVE_LIBWHOSON
-	    } else if (len > 0 && strncmpic("whoson", pat_p, len) == 0) {
+	    } else if (strcmpic("whoson", curpat) == 0) {
 		int wso;
 		char retbuf[BUFSIZ];
 
 		DEBUG1(DBG_ADDR_HI,
-		       "(checking if '%s' is listed in the 'whoson' database)\n", ip);
+		       "(checking if [%s] is listed in the 'whoson' database)\n", ip);
 		wso = wso_query(ip, retbuf, sizeof(retbuf));
 		retbuf[sizeof(retbuf)-1] = '\0'; /* just in case... */
 		if (wso < 0) {
@@ -1889,41 +1944,97 @@ match_ip(ip, patterns)
 		    write_log(WRITE_LOG_PANIC, "wso_query(%s): whoson query failed: %s", ip, retbuf);
 		    /* continue with next pattern */
 		} else if (wso == 0) {
-		    DEBUG2(DBG_ADDR_MID, "ip '%s' verified by whoson as '%s'\n", ip, retbuf);
+		    DEBUG2(DBG_ADDR_MID, "ip [%s] found by whoson: %s.\n", ip, retbuf);
+		    xfree(curpat);
 		    return 1;
 		} else {
-		    DEBUG1(DBG_ADDR_MID, "ip '%s' NOT verified by whoson\n", ip);
+		    DEBUG1(DBG_ADDR_MID, "ip [%s] NOT found by whoson\n", ip);
 		    /* continue with next pattern */
 		}
 #endif
 	    } else {
-		DEBUG3(DBG_ADDR_HI,
-		       "(checking if '%s' matches pattern '%s' to len '%d')\n",
-		       ip, pat_p, len);
-		/* We have a match if pat was wild or we checked the whole IP.
-		 * NOTE:  len may be zero for pattern of just "*" which will
-		 * mean the first term of the expression will always be true,
-		 * which is what we want in this case.
-		 */
-		if (strncmp(ip, pat_p, len) == 0 && (wild || ip[len] == '\0')) {
-		    DEBUG3(DBG_ADDR_MID,
-			   "ip '%s' verified by sub-pattern of '%s', up to len %d\n",
-			   ip, pat_p, len);
+		if (match_ip_net(ip, curpat)) {
+		    DEBUG2(DBG_ADDR_MID,
+			   "ip [%s] matches IP/network of [%s]\n",
+			   ip, curpat);
+		    xfree(curpat);
 		    return 1;
 		}
+		/* continue with next pattern */
 	    }
+	    xfree(curpat);
 	}
 	if (*end_p == '\0') {
 	    break;
 	}
     }
 
-    DEBUG2(DBG_ADDR_HI, "ip '%s' not verified by any pattern in '%s'\n",
+    DEBUG2(DBG_ADDR_HI, "ip [%s] does not match any IP/network in '%s'\n",
 	   ip, patterns);
     
     return 0;
 }
 
+int
+match_ip_net(ipaddr, netspec)
+    char *ipaddr;
+    char *netspec;
+{
+    unsigned long dest;
+    unsigned long net;
+    unsigned long mask;
+    int bits;
+
+    DEBUG2(DBG_ADDR_HI, "match_ip_net([%s], [%s]) called...\n", ipaddr, netspec);
+
+    if ((bits = inet_net_pton(AF_INET, netspec, (void *) &net, sizeof(net))) == -1) {
+	DEBUG2(DBG_ADDR_LO, "match_ip_net(): [%s] is invalid: %s\n", netspec, strerror(errno));
+	return 0;
+    }
+    net = ntohl(net);
+    mask = 0xffffffffL << (32 - bits);
+    switch (inet_pton(AF_INET, ipaddr, &dest)) {
+    case 0:
+	DEBUG1(DBG_ADDR_LO, "match_ip_net(): [%s] is not parsable.\n", ipaddr);
+	return 0;
+    case -1:
+	DEBUG2(DBG_ADDR_LO, "match_ip_net(): [%s] is invalid: %s\n", ipaddr, strerror(errno));
+	return 0;
+    }
+    dest = ntohl(dest);
+#if 0 /* for heavy duty debugging only! */
+    DEBUG4(DBG_ADDR_HI, "match_ip_net(): dest=0x%x, net=0x%x, bits=%d, mask=0x%x\n", dest, net, bits, mask);
+#endif
+
+    return ((dest & mask) == (net & mask));
+}
+
+#define CHOP_B(x)	((int) ((x) & 0xff))
+
+char *
+flip_inet_addr(inaddr)
+    char *inaddr;
+{
+    unsigned long addr;		/* u_int32_t */
+    char revaddr[(4*4)];
+
+    addr = inet_addr(inaddr);
+
+    addr = ntohl(addr);
+
+    (void) sprintf(revaddr, "%u.%u.%u.%u",
+		   CHOP_B(addr),
+		   CHOP_B(addr >> 8),
+		   CHOP_B(addr >> 16),
+		   CHOP_B(addr >> 24));
+
+    return COPY_STRING(revaddr);
+}
+
+#undef CHOP_B
+
+#endif /* HAVE_BSD_NETWORKING */
+
 #ifdef STANDALONE
 
 int debug = 0;
@@ -1939,7 +2050,7 @@ main(argc, argv)
     char *name;
     char *db;
     char *error = NULL;
-    int interval = 3;
+    unsigned int interval = 3;
     int retries = 0;
     int success;
 
