@@ -1,4 +1,6 @@
-/* @(#) lookup.h,v 1.3 1992/07/11 11:49:34 tron Exp */
+/*
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:lookup.h,v 1.4 2000/09/03 16:26:31 woods Exp"
+ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -22,3 +24,11 @@
 #define FILE_FAIL	(-4)		/* unrecoverable database failure */
 #define FILE_AGAIN	(-5)		/* try using database later */
 #define FILE_NOMATCH	(-6)		/* no such file */
+
+/* external functions defined in lookup.c */
+extern int open_database __P((char *, char *, int, unsigned int, struct stat *, char **, char **));
+extern void close_database __P((char *));
+extern int lookup_database __P((char *, char *, char **, char **));
+extern int match_ip __P((char *, char *, int, int, char **));
+extern int match_ip_net __P((char *, char *));
+extern char *flip_inet_addr __P((char *));
