@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:main.c,v 1.57 2000/02/17 04:39:15 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:main.c,v 1.68 2001/01/18 00:31:32 woods Exp"
  */
 
 /*
@@ -19,11 +19,13 @@
  */
 #include <stdio.h>
 #include <ctype.h>
+#include <limits.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <errno.h>
 #include <pwd.h>
 #include <signal.h>
+#define NEED_SOCKETS			/* Required for _PATH_RESCONF from <resolv.h> */
 #include "defs.h"
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
@@ -49,19 +51,31 @@
 #include "config.h"
 #include "smail.h"
 #include "dys.h"
+#include "parse.h"
 #include "addr.h"
 #include "hash.h"
 #include "main.h"
 #include "log.h"
+#include "direct.h"
+#include "route.h"
 #include "transport.h"
 #include "child.h"
 #include "exitcodes.h"
 #include "smailconf.h"
 #include "alloc.h"
-#ifndef DEPEND
-# include "extern.h"
-# include "debug.h"
-# include "error.h"
+#include "extern.h"
+#include "debug.h"
+#include "error.h"
+
+#ifdef HAVE_BSD_NETWORKING
+# ifndef OBSOLETE_RESOLVER
+extern int h_errno;			/* import h_errno; many systems don't define it in <netdb.h> */
+# endif
+# ifdef HAVE_BIND
+#  ifndef _PATH_RESCONF
+#   define _PATH_RESCONF	"/etc/resolv.conf"
+#  endif
+# endif
 #endif
 
 /* exported variables */
@@ -89,16 +103,16 @@ struct addr *recipients;		/* list of recipient addresses */
 int num_recipients = 0;			/* number of entries in 'recipients' */
 char *primary_name;			/* primary local name from hostnames */
 FILE *errfile = NULL;			/* file to write debug messages to */
-int real_uid;				/* saved real uid before ruid setup */
-int real_gid;				/* saved real gid before rgid setup */
+unsigned int real_uid;			/* saved real uid before ruid setup */
+unsigned int real_gid;			/* saved real gid before rgid setup */
 enum prog_type prog_type;		/* type of program we are running as */
 char **save_argv;			/* saved pointer to arguments */
 int some_deferred_addrs;		/* don't unlink spool file */
 					/* as some addrs were deferred */
-int prog_euid;				/* effective uid of program */
-int prog_egid;				/* effective gid of program */
+unsigned int prog_euid;			/* effective uid of program */
+unsigned int prog_egid;			/* effective gid of program */
 int force_zero_exitvalue = FALSE;	/* if TRUE always exit with status 0 */
-int call_defer_message;			/* if TRUE must call defer_message() */
+int call_freeze_message;		/* if TRUE must call freeze_message() */
 int sender_is_trusted;			/* TRUE if sender is a trusted user */
 char *sender_host = NULL;		/* name of sender's host */
 char *sender_host_addr = NULL;		/* inet address of sender's host */
@@ -107,10 +121,11 @@ char *sender_program = NULL;		/* name of program that spooled msg */
 char *smtp_service_name = NULL;		/* smtp service name from -oX */
 
 /* functions local to this file */
-static void panic_if_null();
-static void rmail_panic();
-static char *escape_newline();
-static void parse_grade_range();
+static void panic_if_null __P((char *, char *));
+static void rmail_panic __P((void));
+static unsigned int xivaltou __P((char *));
+static char *escape_newline __P((char *));
+static void parse_grade_range __P((char *, int *, int *));
 
 /* variables local to this file */
 static int report_memory_usage = FALSE;	/* if TRUE, report sbrk(0) when done */
@@ -125,20 +140,12 @@ static char *arg_smail_util_dir = NULL;	/* smail util dir set by args */
 static char *arg_alias_file = NULL;	/* alias file set by -oA */
 static char *arg_debug_file = NULL;	/* debug file, defaulting to stderr */
 static char *arg_runq_grades = NULL;	/* which grades are processed by runq */
-
-/* variables imported from libc */
-extern int errno;
-
-/* functions imported from libc */
-char *getenv();
-
 
 /*
  * main - what to do after being exec'd
  *
  * main decodes the argument list and then performs specified action
  */
-/*ARGSUSED*/
 int
 main(argc, argv)
     int argc;				/* count of arguments passed */
@@ -169,18 +176,15 @@ main(argc, argv)
     /* close file descriptors left open by others */
     close_all();
 
-    /*
-     * get the basename for the program
-     */
-    program = rindex(*argv, '/');
-    if (program == NULL) {
-	program = *argv;
-    } else {
-	program++;
-    }
-    argv++;
+    /* get the basename for the program */
+    program = (program = strrchr(argv[0], '/')) ? program + 1 : argv[0];
+
+    /* default the sender program (-oMP) to argv[0] */
     sender_program = program;
 
+    /* skip on to the first parameter */
+    argv++;
+
     /* initialize per-message state information */
     initialize_state();
 
@@ -212,17 +216,22 @@ main(argc, argv)
 #endif	/* HAVE_SETGROUPS */
 
     /* get rid of any limits that might affect operation */
-#if defined(HAVE_ULIMIT) && !defined(HAVE_RLIMIT)
+#if defined(HAVE_ULIMIT) && !defined(HAVE_RLIMIT) /* XXX HAVE_SETRLIMIT */
     /* kill limits on file size */
-    (void) ulimit(2, ((long)1 << (BITS_PER_LONG - 2))/512);
-#endif	/* HAVE_ULIMIT && !defined(HAVE_RLIMIT) */
+    (void) ulimit(2, ((long) 1 << (BITS_PER_LONG - 2)) / 512);
+#endif	/* HAVE_ULIMIT && !HAVE_RLIMIT */
 
-    /* always get a write error for a SIGPIPE, rather than dying */
-    (void) signal(SIGPIPE, SIG_IGN);
-
-#if defined(HAVE_RLIMIT)
-    /* kill limits on file size, cpu, data segment, and unreasonable
-     * limits on stack size */
+#if defined(HAVE_RLIMIT) /* XXX should be HAVE_SETRLIMIT for autoconf */
+    /*
+     * Kill any limits on file size, and cpu time, and set reasonable limits on
+     * data segment, and on stack size to prevent Denial of Service attacks
+     * from the network.
+     *
+     * XXX for the latter two we should use getrusage() and choose some
+     * meaningful multiplier instead of having the installer choose a limit.
+     * It also might make more sense to hold off until we've loaded config
+     * files too.
+     */
     {
 	struct rlimit rl;
 
@@ -230,17 +239,28 @@ main(argc, argv)
 	rl.rlim_max = RLIM_INFINITY;
 	(void) setrlimit(RLIMIT_CPU, &rl);
 	(void) setrlimit(RLIMIT_FSIZE, &rl);
-#if defined(DATA_RLIMIT)
+#if defined(DATA_RLIMIT)			/* usually in conf/os/OS_TYPE */
 	rl.rlim_cur = DATA_RLIMIT;
+#elif defined (SMALL_MEMORY)
+	rl.rlim_cur = (4 * 1024 * 1024);
+#else
+	rl.rlim_cur = (64 * 1024 * 1024);	/* generous, aren't we! */
 #endif	/* DATA_RLIMIT */
 	(void) setrlimit(RLIMIT_DATA, &rl);
-#if defined(STACK_RLIMIT)
+#if defined(STACK_RLIMIT)			/* usually in conf/os/OS_TYPE */
 	rl.rlim_cur = STACK_RLIMIT;
+#elif defined (SMALL_MEMORY)
+	rl.rlim_cur = (1024 * 1024);
+#else
+	rl.rlim_cur = (8 * 1024 * 1024);	/* generous, aren't we! */
 #endif	/* STACK_RLIMIT */
 	(void) setrlimit(RLIMIT_STACK, &rl);
     }
 #endif	/* HAVE_RLIMIT */
 
+    /* always get a write error for a SIGPIPE, rather than the signal */
+    (void) signal(SIGPIPE, SIG_IGN);
+
 #if	defined(UNIX_AIX) && !defined(NO_AIX_CORE_DUMP)
     /* On a segmentation fault or bus error, we need a full core dump.  */
     {
@@ -314,22 +334,34 @@ main(argc, argv)
 	operation_mode = BATCH_SMTP_MODE;
     } else if (prog_type == PROG_SMTPD) {
 	operation_mode = SMTP_MODE;
+    } else if (prog_type == PROG_ROGUE) {
+	operation_mode = ROGUE_MODE;
     } else if (prog_type == PROG_PATHTO) {
-	pathto(argv);
+	operation_mode = PATHTO_MODE;
     } else if (prog_type == PROG_OPTTO) {
-	optto(argv);
+	operation_mode = OPTTO_MODE;
     } else if (prog_type == PROG_UUPATH) {
-	uupath(argv);
+	operation_mode = UUPATH_MODE;
     } else if (prog_type == PROG_RMAIL) {
 	dot_usage = NO_DOT_PROTOCOL;
     }
 
-    if (getenv("SMAIL_CONFIG")) {
+    if (getenv("SMAIL_CONFIG")) {	/* XXX is this sane!?!? */
 	config_file = getenv("SMAIL_CONFIG");
     }
 
-    /* process the args given by the user */
-    process_args(argv, TRUE);
+    switch (operation_mode) {
+    case PATHTO_MODE:
+    case UUPATH_MODE:
+    case OPTTO_MODE:
+	/* these do their own argument handling */
+	break;
+
+    default:
+	/* process the args given by the user */
+	process_args(argv, TRUE);
+	break;
+    }
 
     if (prog_type == PROG_RUNQUEUE) {
 	if (operation_mode == MODE_DEFAULT ||
@@ -352,10 +384,6 @@ main(argc, argv)
 	}
     }
 
-    if (prog_type == PROG_ROGUE) {
-	operation_mode = ROGUE_MODE;
-    }
-
     if (config_file != save_config_file || arg_second_config_file ||
 	arg_director_file || arg_router_file || arg_transport_file ||
 	arg_qualify_file || arg_retry_file || arg_smail_lib_dir ||
@@ -378,26 +406,24 @@ main(argc, argv)
 	smail_util_dir = arg_smail_util_dir;
     }
     error = read_config_file((config_file = make_lib_fn(config_file)));
-    /* we need to set this again, in case it was changed in the config files */
+    /* we need to set these again, in case they were changed in the main config file */
     if (arg_smail_lib_dir) {
 	smail_lib_dir = arg_smail_lib_dir;
     }
-    /* we need to set this again, in case it was changed in the config files */
-    if (arg_smail_util_dir) {
-	smail_util_dir = arg_smail_util_dir;
-    }
     if (arg_second_config_file) {
 	second_config_file = arg_second_config_file;
     }
+
     second_config_file = make_lib_fn(second_config_file);
     if (error == NULL && second_config_file) {
 	error = read_config_file(second_config_file);
     }
-    /* we need to set this again, in case it was changed in the config files */
+    /* re-compute nobody's IDs in case "nobody" was set in a config file */
+    compute_nobody();
+    /* we need to set these yet again in case they were changed in a config file */
     if (arg_smail_lib_dir) {
 	smail_lib_dir = arg_smail_lib_dir;
     }
-    /* we need to set this again, in case it was changed in the config files */
     if (arg_smail_util_dir) {
 	smail_util_dir = arg_smail_util_dir;
     }
@@ -419,8 +445,12 @@ main(argc, argv)
     if (arg_runq_grades) {
         runq_grades = arg_runq_grades;
     }
+
+    /* all configs should have been loaded now -- we can safely do this: */
+    build_host_strings();
     parse_grade_range(runq_grades, &min_runq_grade, &max_runq_grade);
     parse_grade_range(delivery_grades, &min_delivery_grade, &max_delivery_grade);
+
     /* get the config file names within the lib directory */
     director_file = make_lib_fn(director_file);
     router_file = make_lib_fn(router_file);
@@ -463,6 +493,9 @@ main(argc, argv)
 	case DELIVER_MAIL:
 	case PRINT_VARS_MODE:
 	case REBUILD_ALIASES:
+	case PATHTO_MODE:
+	case UUPATH_MODE:
+	case OPTTO_MODE:
 	    write_log(WRITE_LOG_TTY|WRITE_LOG_PANIC, "%s", error);
 	    break;
 
@@ -494,6 +527,15 @@ main(argc, argv)
 	} else {
 	    add_config_stat(smail, &statbuf);
 	}
+#if defined(HAVE_BSD_NETWORKING) && defined(HAVE_BIND)
+	/*
+	 * this is the easiest thing to do to avoid any resolver library
+	 * caching the nameserver when it first fires up....
+	 */
+	if (stat(_PATH_RESCONF, &statbuf) >= 0) {
+	    add_config_stat(_PATH_RESCONF, &statbuf);
+	}
+#endif
 	/* FALLTHRU */
 
     case PRINT_VARS_MODE:			/* This is to allow dumping of configs */
@@ -501,6 +543,9 @@ main(argc, argv)
     case VERIFY_ADDRS:
     case BATCH_SMTP_MODE:
     case SMTP_MODE:
+    case PATHTO_MODE:
+    case UUPATH_MODE:
+    case OPTTO_MODE:
 	/* note that retry_file depends on stuff in config_file */
 	if ((error = read_transport_file()) ||
 	    (error = read_router_file()) ||
@@ -664,7 +709,6 @@ main(argc, argv)
 	break;
 
     case NOOP_MODE:			/* generally, this means run queue */
-	dont_deliver = FALSE;		/* it is too dangerous to allow this */
 	noop_mode();
 	break;
 
@@ -697,7 +741,7 @@ main(argc, argv)
 	break;
 
     case DAEMON_MODE:			/* be a daemon waiting for requests */
-	dont_deliver = FALSE;		/* it is too dangerous to allow this */
+	dont_deliver = FALSE;		/* it is far too dangerous to allow this */
 	daemon_mode();
 	break;
 
@@ -723,6 +767,18 @@ main(argc, argv)
 	perform_deliver_mail();
 	break;
 
+    case PATHTO_MODE:
+	pathto(argc, argv);
+	break;
+
+    case UUPATH_MODE:
+	uupath(argc, argv);
+	break;
+
+    case OPTTO_MODE:
+	optto(argc, argv);
+	break;
+
     case ROGUE_MODE:			/* print a rogue tombstone */
 	silly();
 	break;
@@ -762,10 +818,10 @@ main(argc, argv)
 	if (arg_alias_file) {
 	    utilargs[utilargs[2] ? 3 : 2] = arg_alias_file;
 	}
-	DEBUG4(DBG_MAIN_LO, "main: about to run %s %s %s %s\n",
+	DEBUG4(DBG_MAIN_MID, "main: about to run %s %s %s %s\n",
 	       utilargs[0], utilargs[1], utilargs[2] ? utilargs[2] : "", utilargs[3] ? utilargs[3] : "");
 	child = open_child(utilargs, (char **) NULL, (FILE **) NULL, (FILE **) NULL, fileno(stderr),
-			   CHILD_MINENV, (int) getuid(), (int) getgid());
+			   CHILD_MINENV, (unsigned int) getuid(), (unsigned int) getgid());
 	if (child == -1) {
 	    if (errfile) {
 		fprintf(errfile, "%s: Cannot start %s: %s\n",
@@ -846,11 +902,14 @@ initialize_state()
     recipients = NULL;
     num_recipients = 0;
     some_deferred_addrs = FALSE;
-    call_defer_message = FALSE;
+    call_freeze_message = FALSE;
     sender_is_trusted = TRUE;
     dont_deliver = FALSE;
 
     reset_hit_table();
+
+    /* figure out the initial nobody uid/gid off the top */
+    compute_nobody();
 }
 
 /*
@@ -898,7 +957,7 @@ process_args(args, restrict)
 
 	    /* switch on each letter */
 	    arg++;
-	    while (*arg) switch(*arg++) {
+	    while (*arg) switch (*arg++) {
 
 	    case 'd':			/* set debug level */
 	    case 'v':			/* verbose, same as debug */
@@ -1041,25 +1100,7 @@ process_args(args, restrict)
 		}
 		process_queue = TRUE;
 		if (arg[0]) {
-		    long l = ivaltol(arg);
-
-		    if (l < 0) {
-			if (errfile) {
-			    (void) fprintf(errfile,
-					   "%s: %s: malformed interval\n",
-					   program, arg);
-			}
-			exit(EX_USAGE);
-		    }
-		    queue_interval = (unsigned) l;
-		    if (l != queue_interval) {
-			if (errfile) {
-			    (void) fprintf(errfile,
-					   "%s: %s: interval too large\n",
-					   program, arg);
-			}
-			exit(EX_USAGE);
-		    }
+		    queue_interval =  xivaltou(arg);
 		    arg = end_arg;	/* uses rest of argument */
 		}
 		break;
@@ -1120,8 +1161,7 @@ process_args(args, restrict)
 		    arg = *args++;
 		    panic_if_null(arg, "e");
 		}
-		switch(*arg) {
-
+		switch (*arg) {
 		case 'e':		/* we don't support berkenet */
 		case 'm':		/* mail back errors */
 		    error_processing = MAIL_BACK;
@@ -1135,6 +1175,12 @@ process_args(args, restrict)
 		case 'q':		/* be quiet about errors */
 		    error_processing = DEV_NULL;
 		    break;
+		default:
+		    if (errfile) {
+			(void) fprintf(errfile, "%s: %s: invalid '-e' suboption\n", program, arg);
+		    }
+		    exit(EX_USAGE);
+		    /* NOTREACHED */
 		}
 		arg = end_arg;		/* swallows complete argument */
 		break;
@@ -1145,7 +1191,6 @@ process_args(args, restrict)
 		    panic_if_null(arg, "o");
 		}
 		switch (*arg++) {
-
 		case 'A':               /* name of aliases file */
 		    if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
 			rmail_panic();
@@ -1170,6 +1215,7 @@ process_args(args, restrict)
 
 		case 'D':		/* name of director file */
 		    if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
+
 			rmail_panic();
 		    }
 		    if (*arg == '\0') {
@@ -1269,6 +1315,12 @@ process_args(args, restrict)
 			}
 			sender_program = arg;
 			break;
+		    default:
+			if (errfile) {
+			    (void) fprintf(errfile, "%s: %s: invalid '-oM' suboption\n", program, --arg);
+			}
+			exit(EX_USAGE);
+			/* NOTREACHED */
 		    }
 		    break;
 
@@ -1355,17 +1407,26 @@ process_args(args, restrict)
 		    }
 		    switch (*arg) {
 
-		    case 'f':
-			deliver_mode = FOREGROUND;
-			break;
-
 		    case 'b':
 			deliver_mode = BACKGROUND;
 			break;
 
+		    case 'f':
+		    case 'i':		/* sendmail "interactive delivery" */
+			deliver_mode = FOREGROUND;
+			break;
+
+		    case 'd':		/* sendmail "defer map lookups as well as queue" */
 		    case 'q':
 			deliver_mode = QUEUE_MESSAGE;
 			break;
+
+		    default:
+			if (errfile) {
+			    (void) fprintf(errfile, "%s: %s: invalid '-od' suboption\n", program, arg);
+			}
+			exit(EX_USAGE);
+			/* NOTREACHED */
 		    }
 		    break;
 
@@ -1388,6 +1449,12 @@ process_args(args, restrict)
 		    case 'q':
 			error_processing = DEV_NULL;
 			break;
+		    default:
+			if (errfile) {
+			    (void) fprintf(errfile, "%s: %s: invalid '-oe' suboption\n", program, arg);
+			}
+			exit(EX_USAGE);
+			/* NOTREACHED */
 		    }
 		    break;
 
@@ -1398,6 +1465,25 @@ process_args(args, restrict)
 		case 'm':		/* same as -m */
 		    me_too = TRUE;
 		    break;
+
+		case 'r':		/* sendmail "read timeout" */
+		    if (*arg == '\0') {
+			arg = *args++;
+			panic_if_null(arg, "or");
+		    }
+#ifdef notyet
+		    read_message_timeout = xivaltou(arg);
+#endif
+		    break;
+
+		default:
+		    if (errfile) {
+			(void) fprintf(errfile, "%s: %s: invalid '-o' suboption\n", program, --arg);
+		    }
+#ifdef NO_SENDMAIL_COMPATABILITY
+		    exit(EX_USAGE);
+		    /* NOTREACHED */
+#endif
 		}
 		arg = end_arg;
 		break;
@@ -1478,20 +1564,60 @@ process_args(args, restrict)
 		    }
 		    operation_mode = PRINT_VARS_MODE;
 		    break;
+		default:
+		    if (errfile) {
+			(void) fprintf(errfile, "%s: %s: invalid '-b' suboption\n", program, arg);
+		    }
+		    exit(EX_USAGE);
+		    /* NOTREACHED */
 		}
 		arg = end_arg;
 		break;
+
+	    case 'x':			/* special internal hacks */
+		if ((prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) && restrict) {
+		    rmail_panic();
+		}
+		if (arg[0] == '\0') {
+		    arg = *args++;
+		    panic_if_null(arg, "x");
+		}
+		switch (*arg++) {
+		case 'D':		/* convert interval to days only, for checkerr.sh */
+		    if (*arg == '\0') {
+			arg = *args++;
+			panic_if_null(arg, "xD");
+		    }
+		    printf("%u\n", xivaltou(arg) / (60*60*24));
+		    exit(EX_OK);
+		    /* NOTREACHED */
+		case 'S':		/* convert interval to seconds only, for checkerr.sh */
+		    if (*arg == '\0') {
+			arg = *args++;
+			panic_if_null(arg, "xS");
+		    }
+		    printf("%u\n", xivaltou(arg));
+		    exit(EX_OK);
+		    /* NOTREACHED */
+		default:
+		    if (errfile) {
+			(void) fprintf(errfile, "%s: %s: invalid '-x' suboption\n", program, --arg);
+		    }
+		    exit(EX_USAGE);
+		    /* NOTREACHED */
+		}
+		arg = end_arg;		/* swallows complete argument */
+		break;
 	    }
 	} else {
-
 	    /*
 	     * not a flag, arg must be a recipient address so insert
 	     * it in the list.
 	     */
 	    cur = alloc_addr();
 	    cur->in_addr = escape_newline(arg);
-	    cur->uid = nobody_uid;
-	    cur->gid = nobody_gid;
+	    cur->uid = nobody_uid;	/* may be reset in compute_nobody() */
+	    cur->gid = nobody_gid;	/* may be reset in compute_nobody() */
 	    cur->succ = recipients;
 	    recipients = cur;
 	    num_recipients++;
@@ -1499,6 +1625,28 @@ process_args(args, restrict)
     }
 }
 
+static unsigned int
+xivaltou(arg)
+    char *arg;
+{
+    long l = ivaltol(arg);
+
+    if (l < 0) {
+	if (errfile) {
+	    (void) fprintf(errfile, "%s: %s: invalid interval (too large?)\n", program, arg);
+	}
+	exit(EX_USAGE);
+    }
+    if (l > UINT_MAX) {			/* if sizeof(long) != sizeof(unsigned int) */
+	if (errfile) {
+	    (void) fprintf(errfile, "%s: %s: interval is too large\n", program, arg);
+	}
+	exit(EX_USAGE);
+    }
+
+    return (unsigned int) l;
+}
+
 static char *
 escape_newline(s)
     register char *s;
