@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:main.h,v 1.7 1997/12/16 22:11:51 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:main.h,v 1.9 2000/09/03 16:29:01 woods Exp"
  */
 
 /*
@@ -19,7 +19,7 @@
 #define MAIN_H
 
 /* functions defined in main.c */
-extern int main();
+extern int main __P((int, char **));
 
 /* macros used in main.c */
 
@@ -37,6 +37,9 @@ enum op_mode {
     TEST_MODE,			/* addressing parsing tests */
     VERIFY_ADDRS,		/* verify use of addresses */
     NOOP_MODE,			/* do nothing (run queue, if applicable) */
+    PATHTO_MODE,		/* invoked as pathto, display routing path */
+    UUPATH_MODE,		/* invoked as uupath, display uucp path */
+    OPTTO_MODE,			/* invoked as optto, not yet supported */
     ROGUE_MODE,			/* display the rogue tombstone mode */
     COPYING_MODE,		/* print the COPYING file */
     PRINT_VARS_MODE		/* print configuration variables */
@@ -84,4 +87,9 @@ enum dot_usage {
     SMTP_DOTS			/* like HIDDEN_DOTS, though EOF is error */
 };
 
+/* external functions defined in main.c */
+extern void initialize_state __P((void));
+extern void reset_hit_table __P((void));
+extern void process_args __P((char **, int));
+
 #endif	/* MAIN_H */
