@@ -1,6 +1,6 @@
 #! /bin/sh
 :
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:mkdrivtab.sh,v 1.14 1997/12/16 22:25:22 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:mkdrivtab.sh,v 1.15 2000/09/03 16:35:06 woods Exp"
 
 # Script to create drivertab.c and the driver makefile directories
 # from a driver configuration file.  This script makes heavy use of
@@ -93,11 +93,7 @@ else
 fi
 
 sed -e 's/#.*//' -e '/^[ 	]*$/d' |
-    if [ -f mkdriv.perl ]; then
-	perl mkdriv.perl
-    else
-	awk -f mkdriv.awk
-    fi |
+    awk -f mkdriv.awk |
     sed	-e '/%.*%NULL/d'			\
 	-e 's/%CC%/extern void /'		\
 	-e 's/%DD%/extern struct addr */'	\
@@ -124,6 +120,14 @@ sed -e 's/#.*//' -e '/^[ 	]*$/d' |
  *	define the available director, router, transport and lookup
  *	drivers for use by smail.
  *
+ * #     #                                                   ###
+ * #  #  #    ##    #####   #    #     #    #    #   ####    ###
+ * #  #  #   #  #   #    #  ##   #     #    ##   #  #    #   ###
+ * #  #  #  #    #  #    #  # #  #     #    # #  #  #         #
+ * #  #  #  ######  #####   #  # #     #    #  # #  #  ###
+ * #  #  #  #    #  #   #   #   ##     #    #   ##  #    #   ###
+ *  ## ##   #    #  #    #  #    #     #    #    #   ####    ###
+ *
  * THIS FILE IS GENERATED AUTOMATICALLY BY THE SCRIPT $0 FROM
  * THE DRIVER CONFIGURATION FILE $driver_file
  * MAKE CHANGES TO THE DRIVER CONF FILE AND REBUILD RATHER THAN EDITING
@@ -133,6 +137,7 @@ sed -e 's/#.*//' -e '/^[ 	]*$/d' |
 #include <stdio.h>
 #include "defs.h"
 #include "smail.h"
+#include "parse.h"
 #include "addr.h"
 #include "direct.h"
 #include "route.h"
