@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:modes.c,v 1.98 2000/02/14 05:30:39 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:modes.c,v 1.106 2001/02/05 23:18:11 woods Exp"
  */
 
 /*
@@ -64,20 +64,24 @@
 
 #include "config.h"
 #include "smail.h"
-#include "alloc.h"
-#include "dys.h"
+#include "parse.h"
 #include "addr.h"
 #include "hash.h"
 #include "main.h"
 #include "log.h"
+#include "direct.h"
+#include "route.h"
 #include "transport.h"
 #include "child.h"
+#include "spool.h"
+#include "alloc.h"
+#include "dys.h"
 #include "exitcodes.h"
-#ifndef DEPEND
-# include "extern.h"
-# include "debug.h"
-# include "error.h"
-#endif
+#include "version.h"
+#include "smailconf.h"
+#include "extern.h"
+#include "debug.h"
+#include "error.h"
 
 #if defined(POSIX_OS) || defined(UNIX_BSD) || defined(WAIT_USE_UNION)
 # include <sys/wait.h>
@@ -145,19 +149,16 @@ VOLATILE int *smtp_pids;
 /* variables imported from main.c */
 extern char *smtp_service_name;
 
-/* variables imported from libc */
-extern int errno;
-
 /* functions local to this file */
-static int start_daemon();
-static void bg_run_queue();
-static void do_run_queue();
-static void process_one_spool_file();
-static void sig_unlink();
-static void sig_close();
-static void set_queue_only();
-static void do_smtp();
-static void error_resolve_timeout();
+static int start_daemon __P((void));
+static void bg_run_queue __P((int));
+static void do_run_queue __P((void));
+static void process_one_spool_file __P((char *));
+static void sig_unlink __P((int));
+static void sig_close __P((int));
+static void set_queue_only __P((int));
+static void do_smtp __P((FILE *, FILE *));
+static void error_resolve_timeout __P((struct addr **, struct addr **));
 
 /*
  * Interactive UNIX 2.2 has a bug in accept().  If accept() is
@@ -296,6 +297,19 @@ compute_nobody()
 	    nobody_gid = pw->pw_gid;
 	}
     }
+    /* reset the UIDs in any recipient address structs in case they have been
+     * changed by the above code due to a new value for the "nobody" variable
+     * being specified in a config file loaded since these were first set in
+     * process_args()
+     */
+    if (recipients) {
+	struct addr *cp;
+
+	for (cp = recipients; cp; cp = cp->succ) {
+	    cp->uid = nobody_uid;
+	    cp->gid = nobody_gid;
+	}
+    }
 }
 
 
@@ -420,10 +434,6 @@ test_addresses()
     int stdin_is_a_tty = isatty(0);
 
     X_PANIC_OKAY();
-    if (primary_name == NULL) {
-	/* setup all of the hostname information */
-	build_host_strings();
-    }
 
     while (stdin_is_a_tty && (fputs("> ", stdout) != EOF)) {
 	struct addr *cur = alloc_addr(); /* XXX memory leak.... */
@@ -504,7 +514,7 @@ test_addresses()
 	    break;
 
 	default:
-	    (void) fprintf(stderr, "%s ... internal error in resolve_addr_list\n",
+	    (void) fprintf(stderr, "%s ... internal error in route_remote_addrs\n",
 			   lp);
 	    break;
 	}
@@ -548,8 +558,16 @@ perform_deliver_mail()
     }
 
     if (read_message() == NULL) {
-	panic(EX_OSFILE, "failed to read queued message");
-	/*NOTREACHED*/
+	if (msg_size == 0) {
+	    write_log(WRITE_LOG_SYS, "discarding empty spool file!");
+	    unlink_spool();
+	    exitvalue = EX_OK;
+	} else {
+	    /* XXX should this still be a panic()? */
+	    write_log(WRITE_LOG_SYS, "failed to read queued message!");
+	    exitvalue = EX_OSFILE;
+	}
+	return;
     }
 
     /*
@@ -644,12 +662,6 @@ perform_deliver_mail()
 	panic(EX_OSFILE, "%s", error);
     }
 
-    /* setup all of the hostname information */
-    build_host_strings();
-
-    /* figure out the nobody uid/gid */
-    compute_nobody();
-
     /*
      * process the message, find all of the recipients and
      * perform delivery.
@@ -841,13 +853,13 @@ deliver_mail()
     /*
      * tidy up before going away
      */
-    if (call_defer_message) {
+    if (call_freeze_message) {
 	/*
 	 * leave a file in an error/ directory for the system
 	 * administrator to look at.  This is used for failed error
 	 * mail and for problems resulting from configuration errors.
 	 */
-	defer_message();
+	freeze_message();
     } else if (some_deferred_addrs) {
 	/*
 	 * leave the file around to be processed by a later queue run.
@@ -869,18 +881,18 @@ deliver_mail()
 }
 
 
-static void daemon_sighup();
-static void daemon_sigterm();
-static void daemon_sigalrm();
+static void daemon_sighup __P((int));
+static void daemon_sigterm __P((int));
+static void daemon_sigalrm __P((int));
 #ifdef SIGCHLD
-static void daemon_sigchld();
-static void reap_child();
+static void daemon_sigchld __P((int));
+static void reap_child __P((int));
 #endif
 static int got_sighup;
 static int got_sigalrm;
 
 #if	defined(HAVE_BSD_NETWORKING)
-static void do_daemon_accept();
+static void do_daemon_accept __P((int, int, struct sockaddr_in *));
 
 /*
  * daemon_mode - be a daemon waiting for requests
@@ -1254,8 +1266,9 @@ do_daemon_accept(ls, fd, from)
 	FILE *out;			/* output channel */
 
 	if (debug_pause_for_smtp_connections) {
-	    write_log(WRITE_LOG_SYS, "remote: pausing %d seconds for debugger startup....",
-		      debug_pause_for_smtp_connections);
+	    /* NOTE: the pid is included in the text so it can be seen with -v999 too! */
+	    write_log(WRITE_LOG_SYS, "remote: pausing %d seconds for debugger startup on PID %d ....",
+		      debug_pause_for_smtp_connections, getpid());
 	    sleep(debug_pause_for_smtp_connections); /* XXX what about interactions with SIGALRM? */
 	}
 
@@ -1527,10 +1540,6 @@ start_daemon()
 
     /* cache some interesting things */
     open_system_logs();
-    if (primary_name == NULL) {
-	build_host_strings();
-    }
-    compute_nobody();
 
     /* disconnect from the controlling terminal, if we are not debugging */
     if (debug == 0) {
@@ -1652,22 +1661,20 @@ verify_addresses()
 	    exitvalue = EX_OSFILE;
 	    return;
 	}
-	if (read_message() == NULL) {
-	    unlink_spool();
-	    (void) fprintf(errfile, "failed to read queued message\n");
-	    exitvalue = EX_OSFILE;
+	if (! read_message()) {
+	    if (msg_size == 0) {
+		write_log(WRITE_LOG_SYS, "discarding empty spool file!");
+		unlink_spool();
+		exitvalue = EX_OK;
+	    } else {
+		write_log(WRITE_LOG_SYS, "failed to read queued message!");
+		exitvalue = EX_OSFILE;
+	    }
 	    return;
 	}
 	/* don't actually need the message anymore */
 	unlink_spool();
     }
-    if (primary_name == NULL) {
-	/* setup all of the hostname information */
-	build_host_strings();
-    }
-    /* figure out who nobody is */
-    compute_nobody();
-
 
     /*
      * preparse all of the recipient addresses given as arguments.
@@ -1803,7 +1810,7 @@ do_run_queue()
 
 	/* message processed, go on to the next message */
 	work++;
-	if (debug && errfile) {
+	if (*work && debug && errfile) {
 	    (void) putc('\n', errfile);
 	}
     }
@@ -1866,8 +1873,6 @@ print_variables()
     register struct addr *cur;
     struct addr *new, *next;
 
-    build_host_strings();
-    compute_nobody();
     /* first reverse the list */
     new = NULL;
     for (cur = recipients; cur; cur = next) {
@@ -1890,6 +1895,8 @@ print_queue()
 {
     char **work;			/* vector of jobs to process */
     int col = 0;			/* current print column */
+    unsigned int numjobs = 0;
+    unsigned long jobsize = 0;
 
     X_PANIC_OKAY();
 
@@ -1914,9 +1921,12 @@ print_queue()
 	    continue;
 	}
 	sender = NULL;
-	argv = read_message();
-
-	if (argv == NULL) {
+	if (!(argv = read_message())) {
+	    if (msg_size == 0 && debug) {
+		printf("%s/%s: empty spool file, ignored...\n", spool_dir, input_spool_fn);
+	    } else if (msg_size != 0) {
+		printf("%s/%s: incomplete spool file, ignored...\n", spool_dir, input_spool_fn);
+	    }
 	    work++;
 	    continue;
 	}
@@ -1929,6 +1939,9 @@ print_queue()
 
 	(void) printf("\t\tBodySize: %ld\n", msg_body_size);
 
+	numjobs++;
+	jobsize += msg_size;
+
 	/*
 	 * print the argument vectors several to a line, trying not to
 	 * go past the 76'th column
@@ -1965,6 +1978,19 @@ print_queue()
 	    putchar('\n');
 	}
     }
+
+    if (numjobs || jobsize) {		/* pedantic, but... */
+	printf("\nSmail mail queue %s: %u job%s, %lu byte%s.\n",
+	       recipients ? "matching entries" : "stats",
+	       numjobs, numjobs > 1 ? "s" : "",
+	       jobsize, jobsize > 1 ? "s" : "");
+    } else if (recipients) {
+	printf("No matching entries in Smail mail queue.\n");
+    } else {
+	printf("Smail mail queue is empty.\n");
+    }
+
+    return;
 }
 
 
@@ -1980,11 +2006,6 @@ smtp_mode(in, out)
     FILE *out;				/* channel for responses */
 {
     open_system_logs();
-    if (primary_name == NULL) {
-	build_host_strings();
-    }
-    compute_nobody();
-
     /* do the real work */
     do_smtp(in, out);
 }
@@ -2079,9 +2100,15 @@ do_smtp(in, out)
 		    /* somebody else grabbed the lock, assume they will deliver */
 		    exit(0);
 		}
-		argv = read_message();
-
-		if (argv == NULL) {
+		if (!(argv = read_message())) {
+		    if (msg_size == 0) {
+			write_log(WRITE_LOG_SYS, "discarding empty spool file!");
+			unlink_spool();
+			exitvalue = EX_OK;
+		    } else {
+			write_log(WRITE_LOG_SYS, "failed to read queued message!");
+			exitvalue = EX_OSFILE;
+		    }
 		    exit(exitvalue);
 		}
 
@@ -2143,7 +2170,7 @@ process_spool_file(spfn)
     int pid;
     struct stat statbuf;
 
-    DEBUG1(DBG_MAIN_LO, "process_spool_file(%s): called\n", spfn);
+    DEBUG1(DBG_MAIN_MID, "process_spool_file(%s): called\n", spfn);
     /*
      * Verify the spool file exists before we try to process it
      */
@@ -2192,7 +2219,7 @@ process_one_spool_file(spfn)
     /* in child process */
     char **argv;		/* arguments from spool file */
 
-    DEBUG1(DBG_MAIN_LO, "process_one_spool_file(%s): called\n", spfn);
+    DEBUG1(DBG_MAIN_MID, "process_one_spool_file(%s): called\n", spfn);
 
     /* message grade is encoded in the last char of the filename */
     msg_grade = spfn[strlen(spfn) - 1];
@@ -2213,10 +2240,16 @@ process_one_spool_file(spfn)
     }
 
     /* read the spool file header to get the delivery args */
-    argv = read_message();
-    if (argv == NULL) {
-	panic(EX_OSFILE, "failed to read queued message in %s", spfn);
-	/* NOTREACHED */
+    if (!(argv = read_message())) {
+	if (msg_size == 0) {
+	    write_log(WRITE_LOG_SYS, "discarding empty spool file!");
+	    unlink_spool();
+	    exitvalue = EX_OK;
+	} else {
+	    write_log(WRITE_LOG_SYS, "failed to read queued message!");
+	    exitvalue = EX_OSFILE;
+	}
+	return;
     }
 
     /* process arguments from the spool file */
@@ -2251,8 +2284,9 @@ fork_wait()
 	return -1;
 
     default:
-	while ((i = wait((STATUS_TYPE *) NULL)) >= 0 && i != pid)
+	while ((i = wait((STATUS_TYPE *) NULL)) >= 0 && i != pid) {
 	    ;
+	}
 	break;
     }
     return pid;
