@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:notify.c,v 1.38 2000/02/03 03:44:36 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:notify.c,v 1.43 2001/01/18 00:31:33 woods Exp"
  */
 
 /*
@@ -19,10 +19,10 @@
  *			    error_delivery, process_msg_log, notify,
  *                          hash_predelivered_addresses
  */
-#include <stdio.h>
-#include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <stdio.h>
+#include <ctype.h>
 #include <pwd.h>
 #include <time.h>
 #include "defs.h"
@@ -30,7 +30,7 @@
 # include <stdlib.h>
 #endif
 #include "smail.h"
-#include "dys.h"
+#include "parse.h"
 #include "addr.h"
 #include "hash.h"
 #include "main.h"
@@ -38,24 +38,24 @@
 #include "direct.h"
 #include "route.h"
 #include "transport.h"
+#include "alloc.h"
+#include "dys.h"
 #include "child.h"
 #include "exitcodes.h"
-#ifndef DEPEND
-# include "extern.h"
-# include "debug.h"
-#endif
+#include "extern.h"
+#include "debug.h"
 
 /* exported variables */
 int send_to_postmaster;			/* set TRUE to mail to postmaster */
 int return_to_sender;			/* set TRUE to mail log to sender */
 
 /* functions local to this file */
-static char *decode_x_line();
-static void put_defer_error();
-static void classify_addr();
-static void notify_new_message();
-static struct addr *remove_addr();
-static void write_notify_header();
+static char *decode_x_line __P((char *, char **, char **));
+static void put_defer_error __P((struct defer_addr **, char *, char *, char *));
+static void classify_addr __P((struct addr *, struct addr **, struct addr **));
+static void notify_new_message __P((FILE *, char *, char *, char *,char *));
+static struct addr *remove_addr __P((struct addr *, char *, char *));
+static void write_notify_header __P((FILE *));
 
 /* variables local to this file */
 static char *log_banner = "\
@@ -74,23 +74,29 @@ fail_delivery(addr)
     struct addr *addr;			/* list of failed addrs */
 {
     register struct addr *cur;
+    register struct addr *top;
 
     if (addr && exitvalue == EX_OK) {
 	exitvalue = EX_UNAVAILABLE;
     }
     for (cur = addr; cur; cur = cur->succ) {
+	/* find the top parent to log the original in_addr */
+	for (top = cur; top->parent && top->parent->in_addr; top = top->parent) {
+	    ;
+	}
+	if (top == cur) {
+	    top = NULL;
+	}
 #if (SMAIL_LOG_STYLE == 2)
 	/* Default new log style (as of 3.1.29) */
 	/* This is SMAIL_LOG_STYLE=2 */
 	write_log(WRITE_LOG_SYS,
-		  "Failed TO:%s%s%s%s%s%s%s ERROR:(ERR%03ld) %s",
-		  cur->in_addr,		/* XXX this is never complete for multi-delivery msgs */
-		  cur->router? " ROUTER:": "",
-		  cur->router? cur->router->name: "",
-		  cur->director? " DIRECTOR:": "",
-		  cur->director? cur->director->name: "",
-		  cur->transport? " TRANSPORT:": "",
-		  cur->transport? cur->transport->name: "",
+		  "Failed TO:%s%s%s%s%s%s%s%s%s ERROR:(ERR%03ld) %s",
+		  cur->next_addr ? cur->next_addr : cur->in_addr,
+		  top ? " ORIG-TO:" : "", top ? top->in_addr : "",
+		  cur->router ? " ROUTER:" : "", cur->router ? cur->router->name : "",
+		  cur->director ? " DIRECTOR:" : "", cur->director ? cur->director->name : "",
+		  cur->transport ? " TRANSPORT:" : "", cur->transport ? cur->transport->name : "",
 		  cur->error->info & ERR_MASK,
 		  cur->error->message);
 #else
@@ -98,29 +104,28 @@ fail_delivery(addr)
     /* Old log style (as used by 3.1.28) */
     /* This is SMAIL_LOG_STYLE=1 */
 	write_log(WRITE_LOG_SYS,
-		  "%s ... failed: (ERR_%03ld) %s",
-		  cur->in_addr,
+		  "%s %s%s%s... failed: (ERR_%03ld) %s",
+		  cur->next_addr ? cur->next_addr : cur->in_addr,
+		  top ? "(orig-to: " : "",
+		  top ? top->in_addr : "",
+		  top ? ") " : "",
 		  cur->error->info & ERR_MASK,
 		  cur->error->message);
 # else
 #  include "ERROR - SMAIL_LOG_STYLE set to invalid value."
 # endif
 #endif
-	/* X entries in msg log will be picked up by queue run scans */
-	if (cur->parent) {
-	    write_log(WRITE_LOG_MLOG,
-		      "Xfail: <%s> parent: <%s> reason: (ERR%03ld) %s",
-		      cur->in_addr,	/* XXX this is never complete for multi-delivery msgs */
-		      cur->parent->in_addr,
-		      cur->error->info & ERR_MASK,
-		      cur->error->message);
-	} else {
-	    write_log(WRITE_LOG_MLOG,
-		      "Xfail: <%s> reason: (ERR%03ld) %s",
-		      cur->in_addr,	/* XXX this is never complete for multi-delivery msgs */
-		      cur->error->info & ERR_MASK,
-		      cur->error->message);
-	}
+	/* write MLOG X entries for failed recipients so they be ignored up by
+	 * queue run scans (see decode_x_line() below for record format details)
+	 */
+	write_log(WRITE_LOG_MLOG,
+		  "Xfail: <%s>%s%s%s reason: (ERR%03ld) %s",
+		  cur->in_addr,
+		  cur->parent ? " parent: <" : "",
+		  cur->parent ? cur->parent->in_addr : "",
+		  cur->parent ? ">" : "",
+		  cur->error->info & ERR_MASK,
+		  cur->error->message);
     }
 }
 
@@ -136,59 +141,77 @@ defer_delivery(addr, defer_errors)
     struct addr *addr;			/* list of succeeded addrs */
     struct defer_addr *defer_errors;	/* previous deferals */
 {
-    struct addr *cur;
-    struct defer_addr *link;
+    register struct addr *cur;
+    register struct addr *top;
+    struct defer_addr *defer;
 
     if (addr && exitvalue == EX_OK) {
 	exitvalue = EX_TEMPFAIL;
     }
 
     for (cur = addr; cur; cur = cur->succ) {
-
-	/* if the deferral's only cause is retry interval, don't log it */
+	/* find the top parent to log the original in_addr */
+	for (top = cur; top->parent && top->parent->in_addr; top = top->parent) {
+	    ;
+	}
+	if (top == cur) {
+	    top = NULL;
+	}
+	/* if the only cause of the deferral is that the retry interval has not
+	 * been reached we will be told not to log the cause...
+	 */
 	if (cur->error->info & ERR_DONTLOG) {
 #ifndef NODEBUG
 	    /* but, if debugging, generate a message for human reader */
-	    if (debug && errfile && debug > 1) {
+	    if (errfile && debug > 1) {
 		fprintf(errfile,
-			"%s ... deferred: (ERR_DONTLOG|ERR%03ld) %s\n",
-			cur->in_addr,
+			"%s %s%s%s... deferred: (ERR_DONTLOG|ERR%03ld) %s\n",
+			cur->next_addr ? cur->next_addr : cur->in_addr,
+			top ? "(orig-to: " : "",
+			top ? top->in_addr : "",
+			top ? ") " : "",
 			cur->error->info & ERR_MASK,
 			cur->error->message);
 	    }
 #endif	/* ! NODEBUG */
 	    continue;
 	}
-
-	/* avoid redundant messages */
-	for (link = defer_errors; link; link = link->succ) {
-	    if (EQ(link->address, cur->in_addr) ||
-		(! link->parent && ! cur->parent) ||
-		(link->parent && cur->parent && EQ(link->parent, cur->parent->in_addr)))
+	/*
+	 * avoid redundant messages
+	 *
+	 * first we search through the list of error addresses which were
+	 * recorded in the MLOG file to find one that's got the same ...
+	 */
+	for (defer = defer_errors; defer; defer = defer->succ) {
+	    if (EQ(defer->address, cur->next_addr ? cur->next_addr : cur->in_addr) ||
+		(! defer->parent && ! top) ||
+		(defer->parent && top && top->in_addr && EQ(defer->parent, top->in_addr)))
 	    {
 		break;
 	    }
 	}
-	if (link) {
-	    if (link->error == (cur->error->info & ERR_MASK) &&
-		EQ(link->message, cur->error->message))
+	/*
+	 * now if we've found a message matching the one we're about to log
+	 * then we'll skip logging this one...
+	 */
+	if (defer) {
+	    if (defer->error == (cur->error->info & ERR_MASK) &&
+		EQ(defer->message, cur->error->message))
 	    {
-		break;
+		DEBUG(DBG_NOTIFY_MID, "defer_delivery(): not logging repeated deferral message.\n");
+		continue;			/* XXX was `break'! */
 	    }
 	}
-
 #if (SMAIL_LOG_STYLE == 2)
 	/* Default new log style (as of 3.1.29) */
 	/* This is SMAIL_LOG_STYLE=2 */
 	write_log((cur->error->info & ERR_CONFERR)? (WRITE_LOG_SYS|WRITE_LOG_TTY): WRITE_LOG_SYS,
-		  "Deferred TO:%s%s%s%s%s%s%s ERROR:(ERR%03ld) %s",
-		  cur->in_addr,		/* XXX this is never complete for multi-delivery msgs */
-		  cur->router? " ROUTER:": "",
-		  cur->router? cur->router->name: "",
-		  cur->director? " DIRECTOR:": "",
-		  cur->director? cur->director->name: "",
-		  cur->transport? " TRANSPORT:": "",
-		  cur->transport? cur->transport->name: "",
+		  "Deferred TO:%s%s%s%s%s%s%s%s%s ERROR:(ERR%03ld) %s",
+		  cur->next_addr ? cur->next_addr : cur->in_addr,
+		  top ? " ORIG-TO:" : "", top ? top->in_addr : "",
+		  cur->router ? " ROUTER:" : "", cur->router ? cur->router->name : "",
+		  cur->director ? " DIRECTOR:" : "", cur->director ? cur->director->name : "",
+		  cur->transport ? " TRANSPORT:" : "", cur->transport ? cur->transport->name : "",
 		  cur->error->info & ERR_MASK,
 		  cur->error->message);
 #else
@@ -196,28 +219,30 @@ defer_delivery(addr, defer_errors)
     /* Old log style (as used by 3.1.28) */
     /* This is SMAIL_LOG_STYLE=1 */
 	write_log((cur->error->info & ERR_CONFERR)? (WRITE_LOG_SYS|WRITE_LOG_TTY): WRITE_LOG_SYS,
-                  "%s ... deferred: (ERR_%03ld) %s",
-                  cur->in_addr,
+		  "%s %s%s%s... deferred: (ERR_%03ld) %s",
+		  cur->next_addr ? cur->next_addr : cur->in_addr,
+		  top ? "(orig-to: " : "",
+		  top ? top->in_addr : "",
+		  top ? ") " : "",
                   cur->error->info & ERR_MASK,
                   cur->error->message);
 # else
 #   include "ERROR - SMAIL_LOG_STYLE set to invalid value."
 # endif
 #endif
-	if (cur->parent) {
-	    write_log(WRITE_LOG_MLOG,
-		      "Xdefer: %s parent: %s reason: (ERR%03ld) %s",
-		      cur->in_addr,	/* XXX this is never complete for multi-delivery msgs */
-		      cur->parent->in_addr,
-		      cur->error->info & ERR_MASK,
-		      cur->error->message);
-	} else {
-	    write_log(WRITE_LOG_MLOG,
-		      "Xdefer: %s reason: (ERR%03ld) %s",
-		      cur->in_addr,	/* XXX this is never complete for multi-delivery msgs */
-		      cur->error->info & ERR_MASK,
-		      cur->error->message);
-	}
+	/* write MLOG X entries for recipients experiencing deferred delivery
+	 * so they be picked up by queue run scans and used to avoid writing
+	 * duplicate deferral messages as is done above (see decode_x_line()
+	 * below for record format details)
+	 */
+	write_log(WRITE_LOG_MLOG,
+		  "Xdefer: <%s>%s%s%s reason: (ERR%03ld) %s",
+		  cur->in_addr,
+		  cur->parent ? " parent: <" : "",
+		  cur->parent ? cur->parent->in_addr : "",
+		  cur->parent ? ">" : "",
+		  cur->error->info & ERR_MASK,
+		  cur->error->message);
     }
 }
 
@@ -233,129 +258,149 @@ succeed_delivery(addr)
 
     for (cur = addr; cur; cur = cur->succ) {
 	/* find the top parent to log the original in_addr */
-	for (top = cur; top->parent; top = top->parent) {
+	for (top = cur; top->parent && top->parent->in_addr; top = top->parent) {
 	    ;
 	}
-
+	if (top == cur) {
+	    top = NULL;
+	}
 	if (dont_deliver) {
-	    write_log(WRITE_LOG_SYS, "Debugging: don't deliver %s",
-		      top->in_addr);
+	    write_log(WRITE_LOG_SYS, "Debugging: don't deliver <%s>%s%s%s",
+		      cur->next_addr,
+		      top ? " orig-to: <" : "",
+		      top ? top->in_addr : "",
+		      top ? ">" : "");
 	} else {
 #if (SMAIL_LOG_STYLE == 2)
 	    /* Default new log style (as of 3.1.29) */
 	    /* This is SMAIL_LOG_STYLE=2 */
-	    if (cur->next_host) {
-		write_log(WRITE_LOG_SYS, "Delivered%s%s%s%s%s%s%s%s%s%s",
-			  " VIA:", cur->next_host,
-			  " TO:", cur->next_addr,
-			  " ORIG-TO:", top->in_addr,
-			  " ROUTER:", cur->router->name,
-			  " TRANSPORT:", cur->transport->name);
-	    } else {
-		write_log(WRITE_LOG_SYS, "Delivered%s%s%s%s%s%s%s%s",
-			  " TO:", cur->next_addr,
-			  " ORIG-TO:", top->in_addr,
-			  " DIRECTOR:", cur->director->name,
-			  " TRANSPORT:", cur->transport->name);
-	    }
+	    write_log(WRITE_LOG_SYS, "Delivered%s%s%s%s%s%s%s%s%s%s",
+		      cur->next_host ? " VIA:" : "",
+		      cur->next_host ?  cur->next_host : "",
+		      " TO:",
+		      cur->next_addr,
+		      top ? " ORIG-TO:" : "",
+		      top ? top->in_addr: "",
+		      cur->router ? " ROUTER:" : "",
+		      cur->router ? cur->router->name : "",
+		      " TRANSPORT:",
+		      cur->transport->name);
 #else
 # if (!defined(SMAIL_LOG_STYLE)) || (SMAIL_LOG_STYLE == 1)
 	    /* Old log style (as used by 3.1.28) */
 	    /* This is SMAIL_LOG_STYLE=1 */
-	    if (cur->next_host) {
-		write_log(WRITE_LOG_SYS, "delivered%s%s%s%s%s%s%s%s%s%s",
-			  "\n|\t      via: ", cur->next_host,
-                          "\n|\t       to: ", cur->next_addr,
-                          "\n|\t  orig-to: ", top->in_addr,
-                          "\n|\t   router: ", cur->router->name,
-                          "\n|\ttransport: ", cur->transport->name);
-	    } else {
-                write_log(WRITE_LOG_SYS, "delivered%s%s%s%s%s%s%s%s",
-                          "\n|\t       to: ", cur->next_addr,
-                          "\n|\t  orig-to: ", top->in_addr,
-                          "\n|\t director: ", cur->director->name,
-                          "\n|\ttransport: ", cur->transport->name);
-
-	    }
+	    write_log(WRITE_LOG_SYS, "delivered%s%s%s%s%s%s%s%s%s%s",
+		      cur->next_host ? "\n|\t      via: " : "",
+		      cur->next_host ? cur->next_host : "",
+		      "\n|\t       to: ",
+		      cur->next_addr,
+		      top ? "\n|\t  orig-to: " : "",
+		      top ? top->in_addr : "",
+		      cur->router ? "\n|\t   router: " : "",
+		      cur->router ? cur->router->name : "",
+		      "\n|\ttransport: ",
+		      cur->transport->name);
 # else
 #  include "ERROR - SMAIL_LOG_STYLE set to invalid value."
 # endif
 #endif
 	}
-	/* X entries will be picked up by queue run scans */
-	if (cur->parent) {
-	    write_log(WRITE_LOG_MLOG, "Xsucceed: <%s> parent: <%s>",
-		      cur->in_addr, cur->parent->in_addr);
-	} else {
-	    write_log(WRITE_LOG_MLOG, "Xsucceed: <%s>", cur->in_addr);
-	}
+	/* write MLOG X entries for all recipients that have already been
+	 * delivered to so they can be ignored by queue run scans (see
+	 * decode_x_line() below for record format details)
+	 */
+	write_log(WRITE_LOG_MLOG, "Xsucceed: <%s>%s%s%s",
+		  cur->in_addr,
+		  cur->parent ? " parent: <" : "",
+		  cur->parent ? cur->parent->in_addr : "",
+		  cur->parent ? ">" : "");
     }
 }
 
 /*
- * error_delivery - log that an error was sent for this list of addrs
+ * error_delivery - log that an error notification was sent for the given list
+ * of recipients.
  *
  * detect mail sent to owners and log the original address.
  */
 void
 error_delivery(addr)
-    struct addr *addr;			/* list of error addrs */
+    struct addr *addr;		/* recipients errors have been sent for */
 {
     register struct addr *cur;
+    register struct addr *top;
 
     for (cur = addr; cur; cur = cur->succ) {
 	struct addr *log_addr = NULL;
 
-	if (cur->parent && cur->parent->true_addr) {
-	    log_addr = cur->parent->true_addr;
+	/* find the top parent to log the original in_addr */
+	for (top = cur; top->parent && top->parent->in_addr; top = top->parent) {
+	    ;
+	}
+	if (top == cur) {
+	    top = NULL;
+	}
+
+	if (top && top->true_addr) {
+	    log_addr = top->true_addr;
 	} else if (cur->true_addr) {
 	    log_addr = cur->true_addr;
 	}
 	if (log_addr) {
 #if (SMAIL_LOG_STYLE == 2)
-	    write_log(WRITE_LOG_SYS, "Error sent FOR:%s TO:%s",
-		      log_addr->in_addr, cur->in_addr);
+	    write_log(WRITE_LOG_SYS, "Error sent FOR:%s TO:%s%s%s",
+		      log_addr->in_addr,
+		      cur->next_addr ? cur->next_addr : cur->in_addr,
+		      top ? " ORIG-TO:" : "",
+		      top ? top->in_addr : "");	/* XXX may be wrong or confusing */
 #else
-	    write_log(WRITE_LOG_SYS, "%s ... error sent to %s",
-		      log_addr->in_addr, cur->in_addr);
+	    write_log(WRITE_LOG_SYS, "%s ... error sent to %s%s%s%s",
+		      log_addr->in_addr,
+		      cur->next_addr ? cur->next_addr : cur->in_addr,
+		      top ? " (orig-to " : "",
+		      top ? top->in_addr : "",
+		      top ? ")");
 #endif
 	} else if (cur->error) {
 	    if (cur->error->info & ERR_NSENDER) {
 #if (SMAIL_LOG_STYLE == 2)
 		write_log(WRITE_LOG_SYS, "Returned error FOR:%s TO:%s",
-			  cur->in_addr, sender);
+			  top ? top->in_addr : cur->in_addr, sender);
 #else
 		write_log(WRITE_LOG_SYS, "%s ... error returned to %s",
-			  cur->in_addr, sender);
+			  top ? top->in_addr : cur->in_addr, sender);
 #endif
 	    }
 	    if (cur->error->info & ERR_NPOSTMAST) {
 #if (SMAIL_LOG_STYLE == 2)
 		write_log(WRITE_LOG_SYS, "Sent error FOR:%s TO:postmaster",
-			  cur->in_addr);
+			  top ? top->in_addr : cur->in_addr);
 #else
 		write_log(WRITE_LOG_SYS, "%s ... error sent to postmaster",
-			  cur->in_addr);
+			  top ? top->in_addr : cur->in_addr);
 #endif
 	    }
-	} else {
+	} else {			/* XXX this should be impossible! */
 #if (SMAIL_LOG_STYLE == 2)
 		write_log(WRITE_LOG_SYS, "Sent error without error info FOR:%s (parent=%s)",
-			  cur->in_addr,
-			  cur->parent ? cur->parent->in_addr : "(none)");
+			  cur->next_addr ? cur->next_addr : cur->in_addr,
+			  top ? top->in_addr : "(none)");
 #else
 		write_log(WRITE_LOG_SYS, "%s ... error sent without error info (parent=%s)",
-			  cur->in_addr,
-			  cur->parent ? cur->parent->in_addr : "(none)");
+			  cur->next_addr ? cur->next_addr : cur->in_addr,
+			  top ? top->in_addr : "(none)");
 #endif
 	}
-	/* X entries will be picked up by queue run scans */
-	if (cur->parent) {
-	    write_log(WRITE_LOG_MLOG, "Xsent_error: <%s> parent: <%s>",
-		      cur->in_addr, cur->parent->in_addr);
-	} else {
-	    write_log(WRITE_LOG_MLOG, "Xsent_error: <%s>", cur->in_addr);
-	}
+	/* write MLOG X entries for recipients that error notifications have
+	 * already been sent so so they can be ignored up by queue run scans
+	 * (see decode_x_line() below for record format details)
+	 */
+	write_log(WRITE_LOG_MLOG, "Xsent_error: <%s>%s%s%s",
+		  cur->in_addr,
+		  cur->parent ? " parent: <" : "",
+		  cur->parent ? cur->parent->in_addr : "",
+		  cur->parent ? ">" : "");
+
     }
 }
 
@@ -365,12 +410,20 @@ error_delivery(addr)
  *
  * Lines beginning with X in the per-message log contain information on what
  * mail processing was completed in a previous run on the input spool file.
- * This function takes, as input, the set of addresses which were produced
- * by resolve_addr_list() for delivery and returns a list which does not
- * contain addresses which have already been delivered.
  *
- * It also returns a pointer to information which should be passed to the
- * notify() routine after all the transports have been called.
+ * This function takes, as input, the set of addresses which were produced by
+ * resolve_addr_list() for delivery and returns a list which does not contain
+ * addresses which have already been delivered, or which have already
+ * permanently failed delivery.
+ *
+ * A list of addresses for which notifications have already been sent is
+ * returned by reference through the sent_errors parameter.  This list is
+ * passed to the notify() routine so it can avoid sending duplicate
+ * notifications.
+ *
+ * A list of addresses which have been deferred is also created and returned by
+ * reference through the defer_errors parameter so that defer_delivery() can
+ * avoid recording duplicate deferral messages.
  */
 struct addr *
 process_msg_log(in, sent_errors, defer_errors)
@@ -399,8 +452,7 @@ process_msg_log(in, sent_errors, defer_errors)
 	    }
 	} else if (strncmp(s, "Xsucceed:", sizeof("Xsucceed:") - 1) == 0) {
 	    /* get the delivered address and the parent address */
-	    (void) decode_x_line(s + sizeof("Xsucceed:") - 1,
-				&address, &parent);
+	    (void) decode_x_line(s + sizeof("Xsucceed:") - 1, &address, &parent);
 
 	    /* remove any occurance from the input list */
 	    if (address) {
@@ -418,8 +470,7 @@ process_msg_log(in, sent_errors, defer_errors)
 	    register struct identify_addr *new_sent;
 
 	    /* get the sent error address and the parent address */
-	    (void) decode_x_line(s + sizeof("Xsent_error:") - 1,
-				 &address, &parent);
+	    (void) decode_x_line(s + sizeof("Xsent_error:") - 1, &address, &parent);
 
 	    if (! address) {
 		continue;
@@ -445,11 +496,17 @@ process_msg_log(in, sent_errors, defer_errors)
  *
  * The input should be of the form:
  *
- *	<address> parent: <parent_address> garbage...
- * or	<address> garbage...
+ *	<address> orig-to: <parent_address> message...
+ *	[more message...]
  *
- * this routine returns the address and the parent_address.  It also
- * returns a pointer to the garbage.
+ * or:
+ *
+ *	<address> message...
+ *	[more message...]
+ *
+ * this routine returns the address and the parent_address.  It also returns a
+ * pointer to the message (which may include the reason code which will be
+ * parsed by put_defer_error() below).
  */
 static char *
 decode_x_line(line, address, parent)
@@ -461,10 +518,12 @@ decode_x_line(line, address, parent)
     register char *p = line;		/* point to parts of the line */
 
     /* skip initial white-space */
-    while (isspace(*p)) p++;
-
+    while (isspace(*p)) {
+	p++;
+    }
     /* ignore anything not of the proper form */
     if (*p != '<') {
+	DEBUG1(DBG_NOTIFY_LO, "decode_x_line(): WARNING: no '<' for address: %s\n", line);
 	*address = NULL;
 	return NULL;
     }
@@ -481,6 +540,7 @@ decode_x_line(line, address, parent)
     }
     while (level > 0) {
 	if (!(p = address_token(p))) {
+	    DEBUG1(DBG_NOTIFY_LO, "decode_x_line(): WARNING: no address token in address: %s\n", *address);
 	    *address = NULL;		/* bad input form */
 	    return NULL;
 	}
@@ -498,7 +558,7 @@ decode_x_line(line, address, parent)
     }
     /* is there a parent address? */
     if (strncmp(p, "parent:", sizeof("parent:") - 1) != 0) {
-	/* no */
+	DEBUG1(DBG_NOTIFY_HI, "decode_x_line(): no parent for: %s\n", *address);
 	*parent = NULL;
 	return p;
     }
@@ -511,6 +571,7 @@ decode_x_line(line, address, parent)
 
     /* ignore anything not of the proper form */
     if (*p != '<') {
+	DEBUG1(DBG_NOTIFY_MID, "decode_x_line(): WARNING: no '<' for parent: %s\n", *parent);
 	*parent = NULL;
 	return NULL;
     }
@@ -527,7 +588,8 @@ decode_x_line(line, address, parent)
     }
     while (level > 0){
 	if (!(p = address_token(p))) {
-	    *address = NULL;
+	    DEBUG1(DBG_NOTIFY_MID, "decode_x_line(): WARNING: no address token in parent: %s\n", *parent);
+	    *parent = NULL;
 	    return NULL;
 	}
 	if (*p == '<') {
@@ -548,6 +610,10 @@ decode_x_line(line, address, parent)
 /*
  * put_defer_error - put a deferal message for an address in a list
  *
+ * The reason code and error message text are parsed:
+ *
+ *	reason: (ERRnnn) messge_text
+ *
  * remove previous entries for this address from the list.
  */
 static void
@@ -560,10 +626,11 @@ put_defer_error(list, message, address, parent)
     int error;
     static char reason_pfx[] = "reason: (ERR";
     char *p;
-    struct defer_addr *link, *next_link, *new_list;
+    struct defer_addr *defaddr, *next_link, *new_list;
 
     if (strncmp(message, reason_pfx, sizeof(reason_pfx) - 1) != 0) {
 	/* doesn't match the correct format */
+	DEBUG(DBG_NOTIFY_HI, "put_defer_error(): did not find error code.\n");
 	return;
     }
     message += sizeof(reason_pfx) - 1;
@@ -573,6 +640,7 @@ put_defer_error(list, message, address, parent)
     }
     if (*message != ')') {
 	/* doesn't match the correct format */
+	DEBUG(DBG_NOTIFY_HI, "put_defer_error(): did not find trailing ')' after error code.\n");
 	return;
     }
     *message = 0;
@@ -586,27 +654,32 @@ put_defer_error(list, message, address, parent)
     new_list->error = error;
     new_list->message = COPY_STRING(message);
     new_list->address = COPY_STRING(address);
-    new_list->parent = parent? COPY_STRING(parent): NULL;
+    new_list->parent = parent ? COPY_STRING(parent) : NULL;
 
-    for (link = *list; link; link = next_link) {
-	next_link = link->succ;
-	if (EQ(link->address, address) ||
-	    (! link->parent && ! parent) ||
-	    (link->parent && parent && EQ(link->parent, parent)))
+    DEBUG3(DBG_NOTIFY_MID, "put_defer_error(): found defer error for <%s> (ERR%d): '%s'.\n",
+	   address, error, message);
+
+    for (defaddr = *list; defaddr; defaddr = next_link) {
+	next_link = defaddr->succ;
+	if (EQ(defaddr->address, address) ||
+	    (! defaddr->parent && ! parent) ||
+	    (defaddr->parent && parent && EQ(defaddr->parent, parent)))
 	{
 	    continue;
 	}
-	link->succ = new_list;
-	new_list = link;
+	defaddr->succ = new_list;
+	new_list = defaddr;
     }
     *list = new_list;
+
+    return;
 }
 
 
 /*
  * notify - notify all users who are to receive a note about errors
  *
- * also, note any configuration errors and set call_defer_message
+ * also, note any configuration errors and set call_freeze_message
  * if any are found.  If any addrs exist in the defer list, set
  * some_deferred_addrs to prevent main from calling unlink_spool().
  */
@@ -624,6 +697,7 @@ notify(defer, fail, sent_errors)
     static char *mailargs[] = {		/* setup argv for exec("smail") */
 	NULL,
 	"-bS",				/* use Batched-SMTP mode */
+	NULL,				/* will be -v$debug if in debug mode */
 	NULL,
     };
     FILE *f;				/* stdin to child process */
@@ -635,6 +709,9 @@ notify(defer, fail, sent_errors)
 
     /* set the exec name to the smail binary */
     mailargs[0] = smail;
+    if (debug >= DBG_NOTIFY_HI) {
+	mailargs[2] = xprintf("-v%d", debug); /* XXX this memory will leak! */
+    }
 
     /* remove any address which have already been taken care of */
     while (sent_errors) {
@@ -653,7 +730,7 @@ notify(defer, fail, sent_errors)
 	    panic(EX_SOFTWARE, "notify(): called for deferred message without error info (in_addr=%s)", cur->in_addr);
 	}
 	if (cur->error->info & ERR_CONFERR) {
-	    call_defer_message = TRUE;	/* config error, defer whole message */
+	    call_freeze_message = TRUE;	/* config error, freeze the message in error/ */
 	}
     }
 
@@ -672,21 +749,21 @@ notify(defer, fail, sent_errors)
     }
 
     /*
-     * Move entries from the verify list to the to_owner list if they have a
-     * valid owner address.  If some don't verify, retry classify_addr() until
-     * they do.
+     * Move entries from the verify list to the to_owner list if they are valid
+     * owner address.  If some don't verify, retry classify_addr() until they
+     * do...
      */
     while (verify) {
-	struct addr *new_list = NULL;	/* unverified addrs */
+	struct addr *new_list = NULL;	/* unverified owner addrs */
 
-	DEBUG1(DBG_NOTIFY_HI, "notify(): verifying address %s.\n", verify->work_addr);
+	DEBUG1(DBG_NOTIFY_MID, "notify(): verifying owner address(es) (first is %s).\n", verify->work_addr);
 	/*
 	 * treat defer and fail lists as identical, since we cannot
 	 * reasonably handle defer addresses here.
 	 */
 	resolve_addr_list(verify, &to_owner, &new_list, &new_list, FALSE);
 
-	/* take any new addrs and send through classify_addr */
+	/* take any failed/deferred addrs and send through classify_addr */
 	verify = NULL;
 	for (cur = new_list; cur; cur = next) {
 	    DEBUG1(DBG_NOTIFY_MID, "notify(): re-classifying %s.\n", cur->in_addr)
@@ -694,6 +771,7 @@ notify(defer, fail, sent_errors)
 	    if (!cur->error) {
 		panic(EX_SOFTWARE, "notify(): attempt to re-classify address without error info (in_addr=%s)", cur->in_addr);
 	    }
+	    cur->flags |= ADDR_VRFY_ONLY;
 	    classify_addr(cur, &to_other, &verify);
 	}
     }
@@ -729,7 +807,7 @@ notify(defer, fail, sent_errors)
      */
     if (error_sender) {
 	if (return_to_sender || send_to_postmaster || to_owner) {
-	    call_defer_message = TRUE;
+	    call_freeze_message = TRUE;
 	    return;
 	}
     }
@@ -752,7 +830,7 @@ notify(defer, fail, sent_errors)
     if (pid < 0) {
 	DEBUG(DBG_NOTIFY_LO, "notify: open_child failed, cannot notify\n");
 	/* if we can't notify, let humans interfere later */
-	call_defer_message = TRUE;
+	call_freeze_message = TRUE;
 	return;
     }
 
@@ -767,16 +845,19 @@ notify(defer, fail, sent_errors)
     while (cur) {
 	char *last_owner;
 
+	DEBUG1(DBG_NOTIFY_LO, "notify(): sending to address owner: %s\n", cur->in_addr);
+
 	/* initiate a new message */
 	cc1 = cc2 = NULL;
 	if (send_to_postmaster) {
+	    DEBUG(DBG_NOTIFY_LO, "      ... also sending to postmaster\n");
 	    cc1 = "postmaster";
 	}
 	if (return_to_sender) {
+	    DEBUG1(DBG_NOTIFY_LO, "     ... also returning to sender: %s\n", sender);
 	    cc2 = sender;
 	}
-	notify_new_message(f, cur->in_addr, cc1, cc2,
-			   "sending to address owner");
+	notify_new_message(f, cur->in_addr, cc1, cc2, "sending to address owner");
 	send_log(f, FALSE, log_banner);	/* don't include X lines from log */
 	(void) fputs(addr_error_banner, f);
 	last_owner = cur->in_addr;
@@ -827,8 +908,11 @@ notify(defer, fail, sent_errors)
 
     /* if the postmaster is to receive mail, then send a copy */
     if (send_to_postmaster) {
+	DEBUG(DBG_NOTIFY_LO, "notify(): sending to postmaster\n");
+
 	cc1 = cc2 = NULL;
 	if (return_to_sender) {
+	    DEBUG1(DBG_NOTIFY_LO, "     ... also returning to sender: %s\n", sender);
 	    cc2 = sender;
 	}
 	notify_new_message(f, "postmaster", cc1, cc2, "sending to postmaster");
@@ -866,8 +950,11 @@ notify(defer, fail, sent_errors)
 
     /* if the sender is to receive mail, then send a copy */
     if (return_to_sender) {
+	DEBUG1(DBG_NOTIFY_LO, "notify(): returning to sender: %s\n", sender);
+
 	cc1 = cc2 = NULL;
 	if (send_to_postmaster) {
+	    DEBUG(DBG_NOTIFY_LO, "      ... also sending to postmaster\n");
 	    cc1 = "postmaster";
 	}
 	notify_new_message(f, sender, cc1, cc2, "returning to sender");
@@ -933,7 +1020,7 @@ notify(defer, fail, sent_errors)
  *
  * If postmaster or the sender is to receive an error, link into the
  * to_other list, setting send_to_postmaster and/or return_to_sender
- * appropriately.  If a config error is found, set call_defer_message.
+ * appropriately.  If a config error is found, set call_freeze_message.
  *
  * If we need to send to an owner, determine if an owner string is
  * defined and link an addr structure for that owner into the to_owner
@@ -962,17 +1049,19 @@ classify_addr(addr, to_other, to_owner)
 	    send_to_postmaster = TRUE;
 	}
     }
+    /*
+     * try bouncing to an address owner, if one is defined (and valid)
+     */
     if (addr->error->info & (ERR_NSOWNER|ERR_NPOWNER)) {
-	/* send to an address owner, if one is defined and expandable */
 	register char *s = NULL;	/* expanded owner string */
-	register struct addr *cur;	/* addr for scanning parents */
+	register struct addr *cur;	/* addr ptr for scanning parents */
 	char *work_addr = NULL;		/* output from preparse_address() */
 
 	/*
-	 * find the closest parent associated with a director that
-	 * defines an owner string.
+	 * first find the closest parent associated with a director that
+	 * defines an owner string...
 	 */
-	for (cur = addr; cur; cur = cur->parent) {
+	for (cur = addr->parent; cur; cur = cur->parent) {
 	    char *error;		/* temp for storing parse error */
 
 	    /*
@@ -986,15 +1075,17 @@ classify_addr(addr, to_other, to_owner)
 				   cur->remainder)) &&
 		(work_addr = preparse_address(s, &error)))
 	    {
+		DEBUG2(DBG_NOTIFY_MID, "classify_addr(): for %s: found a valid owner address: %s.\n", addr->in_addr, cur->director->owner);
 		break;
 	    }
 	}
+	/*
+	 * if we didn't walk off the end of the parent list then an owner was
+	 * found for the address, so build a new entry in to_owner list...
+	 */
 	if (cur) {
-	    /* owner found for the address, build entry in to_owner list */
 	    struct addr *new = alloc_addr();
 
-	    new->succ = *to_owner;		/* link */
-	    *to_owner = new;
 	    new->in_addr = COPY_STRING(s);	/* copy owner address */
 	    /*
 	     * if this addr has already been seen by classify_addr() use
@@ -1007,15 +1098,22 @@ classify_addr(addr, to_other, to_owner)
 		new->true_addr = addr;		/* keep track of true addr */
 	    }
 	    new->work_addr = work_addr;		/* use the preparsed address */
-	    /* disable use of smartuser driver */
-	    new->flags = ADDR_SMARTUSER;
+	    /* disable use of smartuser driver, and only allow verify */
+	    new->flags = ADDR_SMARTUSER | ADDR_VRFY_ONLY;
 	    /*
-	     * if resolve_addr_list() fails to verify this list look higher
-	     * up for a parent that has an associated owner.
+	     * if resolve_addr_list() fails to verify this list then prepare to
+	     * look higher up for a parent that has an associated owner when
+	     * re-classifying...
 	     */
 	    new->parent = cur->parent;
+	    /* finally link it into the to_owner list */
+	    new->succ = *to_owner;
+	    *to_owner = new;
 	} else {
-	    /* no owner for the address, send to sender and/or postmaster */
+	    /*
+	     * otherwise no owner was found for the address, bounce to the
+	     * sender and/or postmaster...
+	     */
 	    if (addr->true_addr) {
 		int perhaps_link = FALSE; /* true to see link, if not linked */
 		struct error *true_error = addr->true_addr->error;
@@ -1172,14 +1270,19 @@ write_notify_header(f)
 
 
 /*
- * This code attempts to optimise the reprocessing of queued mail
- * by putting sucessfully delivered addresses into the hash table
- * to prevent them being put through the routers.
- * This cuts down the time taken doing DNS lookups dramatically
- * especially for mailing list mail.
+ * This code attempts to optimise the reprocessing of queued mail by putting
+ * sucessfully delivered remote addresses into the hash table to prevent them
+ * being put through the routers.  (We can't effectively pre-hash locally
+ * deliverable addresses because of course they are transformed into something
+ * else before they are added to the hash table.)
  *
- * It is also in the wrong place - its just that I use decode_x_line()
- * which is local to this file...!  (sorry)
+ * This cuts down the time taken doing DNS lookups dramatically especially for
+ * mailing list mail -- it is often far cheaper to read the MLOG file twice
+ * (which doing this will indeed cause to happen) than it is to do the
+ * additional DNS queries.
+ *
+ * It is also in the wrong place - its just that I use decode_x_line() which is
+ * local to this file...!  (sorry)
  */
 void
 hash_predelivered_addresses()
@@ -1206,8 +1309,8 @@ hash_predelivered_addresses()
 	    offset = 0;
 	}
 	if (offset) {
-	    /* get the failed address and the parent address */
-	    (void) decode_x_line(s + offset , &address, &parent); 
+	    /* get the address (and the parent address) */
+	    (void) decode_x_line(s + offset, &address, &parent); 
 	  
 	    /* Strip off any <> etc. */
 	    preparsed_address = preparse_address(address, &error);
