@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:parse.c,v 1.8 2000/02/17 04:22:54 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:parse.c,v 1.10 2000/11/03 23:28:36 woods Exp"
  */
 
 /*
@@ -31,28 +31,29 @@
 
 #include <sys/types.h>
 #include <stdio.h>
+#include <errno.h>
 #include <ctype.h>
 #include "defs.h"
 #include "smail.h"
+#include "main.h"
+#include "parse.h"
+#include "addr.h"
 #include "log.h"
 #include "alloc.h"
-#include "parse.h"
 #include "dys.h"
 #include "exitcodes.h"
-#ifndef DEPEND
-# include "debug.h"
-# include "extern.h"
-#endif
+#include "debug.h"
+#include "extern.h"
 
 /* variables exported from this file */
 char *on = "1";				/* boolean on attribute value */
 char *off = "0";			/* boolean off attribute value */
 
 /* functions local to this file */
-static char *c_quote();
-static char *eof_error();
-static char *finish_parse();
-static char *skip_space();
+static char *c_quote __P((char *, int *));
+static char *eof_error __P((char *));
+static char *finish_parse __P((char *, struct attribute **, struct attribute **));
+static char *skip_space __P((char *));
 
 
 /*
@@ -460,12 +461,12 @@ parse_config(entry, error)
 
 	return attr;
     } else if (*entry != '=') {
-	*error = "expected `=' after attribute name";
+	*error = xprintf("expected `=' after attribute name: %s", STR(&str));
 	return NULL;
     }
 
     if (attr_type != ' ') {
-	*error = "unexpected pattern:  `= value' follows boolean attribute";
+	*error = xprintf("unexpected pattern:  `= value' follows boolean attribute: %s", STR(&str));
 	return NULL;
     }
 
@@ -493,7 +494,7 @@ parse_config(entry, error)
 	     * make sure that the string doesn't suddenly come
 	     * to an end at a funny spot
 	     */
-	    *error = "unexpected end of attribute";
+	    *error = xprintf("end of quoted attribute found before closing double-quote: %s", STR(&str));
 	    return NULL;
 	}
 	entry++;			/* skip the " */
@@ -511,7 +512,7 @@ parse_config(entry, error)
 		entry++;
 		if (*entry == '\0') {
 		    /* must have something after \ */
-		    *error = "unexpected end of attribute";
+		    *error = xprintf("unexpected end of attribute after backslash: %s", STR(&str));
 		    return NULL;
 		}
 	    }
@@ -525,7 +526,7 @@ parse_config(entry, error)
      * make sure this is really the end of the entry
      */
     if (*entry != '\0') {
-	*error = "expected end of entry";
+	*error = xprintf("unexpected data after end of entry: %s", STR(&str));
 	return NULL;
     }
 
