@@ -1,5 +1,5 @@
 /*
-#ident "@(#)smail/src:$Name $:parse.h,v 1.4 2000/02/17 04:22:55 woods Exp"
+#ident "@(#)smail/src:$Name $:parse.h,v 1.5 2000/09/03 16:29:02 woods Exp"
  */
 
 /*
@@ -31,3 +31,9 @@ struct attribute {
     char *name;				/* name of attribute */
     char *value;			/* value of the attribute */
 };
+
+/* external functions defined in parse.c */
+extern char *parse_entry __P((char *, struct attribute **, struct attribute **, char **));
+extern struct attribute *parse_config __P((char *, char **));
+extern struct attribute *parse_table __P((char *, char **));
+extern char *read_entry __P((FILE *));
