@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:pathto.c,v 1.12 1999/09/19 18:48:01 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:pathto.c,v 1.15 2000/09/03 17:00:15 woods Exp"
  */
 
 /*
@@ -24,15 +24,24 @@
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
 #endif
+#if defined(POSIX_OS)
+# include <unistd.h>
+#else /* not POSIX_OS */
+# if defined (HAVE_UNISTD_H)
+#  include <unistd.h>
+# endif
+#endif	/* not POSIX_OS */
 #include "smail.h"
 #include "main.h"
-#include "transport.h"
+#include "parse.h"
 #include "addr.h"
+#include "route.h"
+#include "transport.h"
+#include "alloc.h"
+#include "dys.h"
 #include "exitcodes.h"
-#ifndef DEPEND
-# include "error.h"
-# include "extern.h"
-#endif
+#include "error.h"
+#include "extern.h"
 
 /* variables local to this file */
 static int bang_paths_only;
@@ -41,24 +50,25 @@ static int direct_and_route;
 static int show_transport;
 
 /* functions local to this file */
-static void print_pathto_addr();
-static void pathto_direct_and_route();
-static char **pathto_setup();
-static struct addr *pathto_route();
-static void pathto_usage();
-static void uupath_usage();
+static void print_pathto_addr __P((struct addr *));
+static void pathto_direct_and_route __P((char **, struct addr **, struct addr **, struct addr **));
+static char **pathto_setup __P((int, char **, char *, void(*)(void)));
+static struct addr *pathto_route __P((char *, char **));
+static void pathto_usage __P((void));
+static void uupath_usage __P((void));
 
 /*
  * pathto - return paths to the given addresses.
  */
 void
-pathto(argv)
+pathto(argc, argv)
+    int argc;
     char **argv;			/* generally, argv from main */
 {
     struct addr *addr;
     char *error;
 
-    argv = pathto_setup(argv, "vdnstD:", pathto_usage);
+    argv = pathto_setup(argc, argv, "vdnstD:", pathto_usage);
     if (direct_and_route) {
 	struct addr *okay;		/* list of routed addrs */
 	struct addr *defer;		/* list of temp failed addrs */
@@ -189,7 +199,8 @@ print_pathto_addr(addr)
  */
 /*ARGSUSED*/
 void
-optto(argv)
+optto(argc, argv)
+    int argc;
     char **argv;			/* generally, argv from main */
 {
     (void) fprintf(stderr, "%s: optto not currently supported\n", program);
@@ -200,13 +211,14 @@ optto(argv)
  * uupath - return paths to the given host.
  */
 void
-uupath(argv)
+uupath(argc, argv)
+    int argc;
     char **argv;			/* generally, argv from main */
 {
     struct addr *addr;
     char *error;
 
-    for (argv = pathto_setup(argv, "vstD:", uupath_usage); *argv; argv++) {
+    for (argv = pathto_setup(argc, argv, "vstD:", uupath_usage); *argv; argv++) {
 	char *host_path = build_uucp_route(*argv, &error, 0);
 	char *in_addr;
 
@@ -254,53 +266,19 @@ uupath(argv)
  * pathto_setup - standard setup for the various pathto routines
  */
 static char **
-pathto_setup(argv, oargs, usage)
+pathto_setup(argc, argv, oargs, usage)
+    int argc;
     char **argv;
     char *oargs;
-    void (*usage)();
+    void (*usage) __P((void));
 {
-    char *error;
-    int argc;
     extern char *optarg;
     extern int optind;
     int c;
     char **argp;
 
-    /* handle relative configuration file names */
-    config_file = make_lib_fn(config_file);
-
-    /* read in the config and secondary config files */
-    if ((error = read_config_file(config_file)) ||
-	(second_config_file &&
-	   (error = read_config_file(make_lib_fn(second_config_file)))))
-    {
-	panic(EX_OSFILE, "%s", error);
-    }
-
-    /* do the same thing with other variables */
-    director_file = make_lib_fn(director_file);
-    router_file = make_lib_fn(router_file);
-    transport_file = make_lib_fn(transport_file);
-    method_dir = make_lib_fn(method_dir);
-    copying_file = make_lib_fn(copying_file);
-    smail = make_lib_fn(smail);
-
-    /* read the other configuration files */
-    if ((error = read_transport_file()) ||
-	(error = read_router_file()) ||
-	(error = read_director_file()))
-    {
-	panic(EX_OSFILE, "%s", error);
-    }
-
-    /* All errors go to the terminal */
-    error_processing = TERMINAL;
 
     --argv;				/* go back to program name */
-    initialize_state();
-
-    /* count arguments, why does getopt really need this? */
-    for (argc = 0; argv[argc]; argc++) ;
 
     disable_smarthost = TRUE;
     bang_paths_only = TRUE;
