@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:pwcache.c,v 1.12 1998/11/06 20:15:11 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:pwcache.c,v 1.14 2000/09/11 01:17:15 woods Exp"
  */
 
 /*
@@ -27,23 +27,17 @@
 #include <ctype.h>
 #include "defs.h"
 #include "smail.h"
+#include "main.h"
+#include "addr.h"
 #include "hash.h"
 #include "alloc.h"
-#ifndef DEPEND
-# include "extern.h"
-#endif
+#include "dys.h"
+#include "extern.h"
 
-/* functions imported from the C library */
-#ifndef ANSI_C
-extern struct passwd *getpwnam();
-extern struct passwd *getpwuid();
-extern struct group *getgrnam();
-extern struct group *getgrgid();
-#endif
 
 /* functions local to this file */
-static void fill_pw_cache();
-static void fill_gr_cache();
+static void fill_pw_cache __P((struct passwd *));
+static void fill_gr_cache __P((struct group *));
 
 /*
  * There are separate caches for four mappings:
@@ -94,7 +88,7 @@ static struct hash_table *group_cache = NULL; /* groupname->gid cache */
  */
 struct passwd *
 getpwbyuid(uid)
-    int uid;
+    unsigned int uid;
 {
     struct pw_cache *entp, ent;
     static struct passwd ret;
@@ -145,7 +139,7 @@ getpwbyuid(uid)
  */
 struct group *
 getgrbygid(gid)
-    int gid;
+    unsigned int gid;
 {
     struct gr_cache *entp, ent;
     static struct group ret;
@@ -315,7 +309,7 @@ fill_pw_cache(pw)
     struct pw_cache ent;		/* fill this with pw info */
     char uid_buf[50];			/* big enough to hold a number */
     register char *p;
-    int size_pw_dir;
+    unsigned int size_pw_dir;
 
     (void) strncpy(ent.pw_name, pw->pw_name, PW_SIZE - 1);
     ent.pw_name[PW_SIZE - 1] = '\0';
@@ -327,7 +321,7 @@ fill_pw_cache(pw)
     size_pw_dir = strlen(pw->pw_dir) + 1;
     ent.pw_dir = xmalloc(size_pw_dir);
     (void) strcpy(ent.pw_dir, pw->pw_dir);
-    (void) sprintf(uid_buf, "%d", pw->pw_uid);
+    (void) sprintf(uid_buf, "%lu", (long unsigned int) pw->pw_uid);
     (void) add_to_hash(uid_buf, (char *)&ent, sizeof(ent), uid_cache);
     (void) add_to_hash(ent.pw_name, (char *)&ent, sizeof(ent), user_cache);
 }
@@ -347,7 +341,7 @@ fill_gr_cache(gr)
 	*p = lowercase((unsigned char) *p);
     }
     ent.gr_gid = gr->gr_gid;
-    (void)sprintf(gid_buf, "%d", gr->gr_gid);
+    (void)sprintf(gid_buf, "%lu", (long unsigned int) gr->gr_gid);
     (void) add_to_hash(gid_buf, (char *)&ent, sizeof(ent), gid_cache);
     (void) add_to_hash(ent.gr_name, (char *)&ent, sizeof(ent), group_cache);
 }
