@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:qualify.c,v 1.6 1997/03/07 03:39:03 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:qualify.c,v 1.7 2000/09/03 17:00:15 woods Exp"
  */
 
 /*
@@ -19,17 +19,19 @@
  *	chip@ateng.ateng.com.  It has been modified.
  */
 
-#include <stdio.h>
-#include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <stdio.h>
+#include <errno.h>
 #include "defs.h"
 #include "smail.h"
+#include "main.h"
 #include "parse.h"
+#include "addr.h"
+#include "alloc.h"
 #include "dys.h"
-#ifndef DEPEND
-# include "extern.h"
-#endif
+#include "smailconf.h"
+#include "extern.h"
 
 struct qualify {
     char    *host;                      /* host name */
@@ -40,9 +42,6 @@ static struct qualify *domains = NULL;  /* Array of host/domain pairs */
 static int domain_count = 0;            /* Count of valid pairs */
 static int domain_max = 0;              /* Count of allocated pairs */
 
-/* variables imported from libc */
-extern int errno;
-
 char *
 read_qualify_file()
 {
