@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:queue.c,v 1.44 2000/02/04 22:14:58 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:queue.c,v 1.46 2000/09/03 17:00:15 woods Exp"
  */
 
 /*
@@ -22,6 +22,7 @@
 
 #include <sys/types.h>
 #include <stdio.h>
+#include <errno.h>
 #include <ctype.h>
 #include <pwd.h>
 #include <grp.h>
@@ -38,18 +39,17 @@
 #endif	/* not POSIX_OS */
 
 #include "smail.h"
-#include "alloc.h"
-#include "spool.h"
-#include "addr.h"
 #include "main.h"
+#include "parse.h"
+#include "addr.h"
 #include "log.h"
 #include "field.h"
+#include "alloc.h"
 #include "dys.h"
+#include "spool.h"
 #include "exitcodes.h"
-#ifndef DEPEND
-# include "debug.h"
-# include "extern.h"
-#endif
+#include "debug.h"
+#include "extern.h"
 
 /* whence values for lseek(2) */
 #ifndef SEEK_SET
@@ -72,20 +72,17 @@ long msg_body_size;			/* size of message body */
 int msg_grade;				/* grade level for this message */
 
 /* functions local to this file */
-static int queue_envelope();
-static int queue_message_internal();
-static int queue_message_with_dots();
-static int queue_message_simple();
-static int fixup_login_user();
-static int put_line();
-static int read_from_lines();
-static char *forward_token();
-static char *backward_token();
-static void finish_sender();
-static int queue_compare();
-
-char *getenv();
-
+static int queue_envelope __P((struct addr *));
+static int queue_message_internal __P((FILE *, enum dot_usage, char **));
+static int queue_message_with_dots __P((FILE *, enum dot_usage, char **));
+static int queue_message_simple __P((FILE *));
+static int fixup_login_user __P((void));
+static int put_line __P((char *));
+static int read_from_lines __P((void));
+static char *forward_token __P((char *));
+static char *backward_token __P((char *, char *));
+static void finish_sender __P((char *, char *));
+static int queue_compare __P((const void *, const void *));
 
 /*
  * queue_message - spool a message.
@@ -503,7 +500,6 @@ static int
 fixup_login_user()
 {
     register int i;
-    extern off_t lseek();
 
     if (local_sender)
 	return SUCCEED;
@@ -1276,7 +1272,7 @@ char **
 scan_spool_dirs()
 {
     static char **mv = NULL;		/* vector of messages */
-    int mc = 0;				/* count of messages */
+    unsigned int mc = 0;		/* count of messages */
     static int mv_size = 32;		/* allocated vector entries */
     char *dn;				/* current directory to scan */
     static struct str str;		/* storage for filenames */
@@ -1326,8 +1322,10 @@ scan_spool_dirs()
 		    if ((seen = EQ(fn, msgid)))
 			break;
 		}
-		if (!seen)
+		if (!seen) {
+		    DEBUG1(DBG_QUEUE_HI, "scan_spool_dirs() skipping %s (not in cmd-line list)\n", fn);
 		    continue;
+		}
 	    }
 
 	    /* See if the grade is within the range being processed */
@@ -1335,6 +1333,8 @@ scan_spool_dirs()
 	    if (operation_mode != PRINT_QUEUE) {
 		entry_grade =  fn[strlen(fn) - 1];
 		if ((entry_grade < min_runq_grade) || (entry_grade > max_runq_grade)) {
+		    DEBUG4(DBG_QUEUE_HI, "scan_spool_dirs() skipping %s (grade '%c' not in runq range '%c'-'%c')\n",
+			   fn, entry_grade, min_runq_grade, max_runq_grade);
 		    continue;	/* outside grade range */
 		}
 	    }
@@ -1370,16 +1370,18 @@ scan_spool_dirs()
  * queue_compare - compare two filenames which represent queue files
  */
 static int
-queue_compare(a, b)
-    char **a;
-    char **b;
+queue_compare(x, y)
+    const void *x;
+    const void *y;
 {
-    int a_grade = (*a)[strlen(*a) - 1];
-    int b_grade = (*b)[strlen(*b) - 1];
+    char *a = *((char **) x);
+    char *b = *((char **) y);
+    int a_grade = a[strlen(a) - 1];
+    int b_grade = b[strlen(b) - 1];
 
     if (a_grade != b_grade) {
 	return a_grade - b_grade;
     }
 
-    return strcmpic(rindex(*a, '/'), rindex(*b, '/'));
+    return strcmpic(strrchr(a, '/'), strrchr(b, '/'));
 }
