@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:resolve.c,v 1.23 2000/02/17 04:39:55 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:resolve.c,v 1.27 2000/10/02 02:06:41 woods Exp"
  */
 
 /*
@@ -21,18 +21,20 @@
 #define NEED_SOCKETS			/* grumble */
 #include "defs.h"
 #include "smail.h"
-#include "dys.h"
-#include "hash.h"
-#include "log.h"
+#include "main.h"
+#include "parse.h"
 #include "addr.h"
+#include "hash.h"
+#include "direct.h"
 #include "route.h"
 #include "transport.h"
+#include "alloc.h"
+#include "dys.h"
+#include "log.h"
 #include "exitcodes.h"
-#ifndef DEPEND
-# include "extern.h"
-# include "debug.h"
-# include "error.h"
-#endif
+#include "extern.h"
+#include "debug.h"
+#include "error.h"
 
 #ifndef INADDR_NONE
 # define INADDR_NONE		((unsigned long) (-1)) /* XXX 64bit too??? */
@@ -55,6 +57,8 @@ struct block *hit_table_block = NULL;	/* block() for bmalloc() et al */
  * inputs:
  *	in	- the list of input address structures
  *
+ *	hash_addrs - set to TRUE to ignore duplicate addresses
+ *
  * outputs:
  *	out	- the list of completely resolved address structures.
  *		  transport, next_host and next_addr will be properly
@@ -90,7 +94,7 @@ resolve_addr_list(in, out, defer, fail, hash_addrs)
     remote = NULL;
     local = NULL;
 
-    DEBUG(DBG_RESOLVE_HI, "resolve_addr_list called\n");
+    DEBUG1(DBG_RESOLVE_HI, "resolve_addr_list(): called [will%s check for dups]\n", hash_addrs ? "" : " NOT");
 
     /*
      * Resolve all addresses in our input queue.
@@ -110,7 +114,7 @@ resolve_addr_list(in, out, defer, fail, hash_addrs)
 	for (cur = in, in = NULL; cur; cur = next) {
 	    int form;			/* address form from parse_address() */
 
-	    DEBUG2(DBG_RESOLVE_HI, "resolve_addr_list working on %s (will%s check dups).\n", cur->work_addr, hash_addrs ? "" : " NOT");
+	    DEBUG1(DBG_RESOLVE_HI, "resolve_addr_list working on %s.\n", cur->work_addr);
 	    next = cur->succ;
 
 	    /* First, if allowed, we try adding the current address to the
@@ -200,7 +204,6 @@ resolve_addr_list(in, out, defer, fail, hash_addrs)
 	struct addr *prev;
 	struct hash_table *hit_out_table; /* table to recognize address hits */
 	struct block *hit_out_table_block; /* block for bmalloc() et al */
-	struct hash *curh;		/* pointer to walk the table */
 	char *hashval;
 
 	hit_out_table_block = malloc_block();
@@ -218,7 +221,7 @@ resolve_addr_list(in, out, defer, fail, hash_addrs)
 			      cur->next_host ? cur->next_host : "(localhost)",
 			      cur->transport->name);
 	    if (add_to_hash(hashval, (char *) NULL, 0, hit_out_table) == ALREADY_HASHED) {
-		DEBUG1(DBG_RESOLVE_LO, "%s: has already been seen -- skipping.\n", hashval);
+		DEBUG1(DBG_RESOLVE_LO, "%s: has already been seen -- skipping during rescan.\n", hashval);
 		/* we always get around the loop at least once without entering
 		 * this side of the 'if', so 'prev' will never be NULL
 		 */
@@ -259,9 +262,6 @@ islocalhost(target)
     char *hostip;
     struct in_addr inet_s;
     struct hostent *hostentp;
-# ifndef INET_ADDR_USE_STRUCT
-    unsigned long inet;			/* internet address */
-# endif
 #endif /* HAVE_BSD_NETWORKING */
 
     if ((uucp_name && EQIC(target, uucp_name)) ||
@@ -280,31 +280,17 @@ islocalhost(target)
         hostip = COPY_STRING(target+1); /* make a copy so we don't have to modify target. */
 	len = strlen(hostip);
 	hostip[len-1] = '\0';
-# ifdef HAVE_INET_ATON
 	if (!inet_aton(hostip, &inet_s)) {
 	    xfree(hostip);
 	    return FALSE;		/* not a valid IP number.... */
 	}
-# else
-#  ifdef INET_ADDR_USE_STRUCT
-        inet_s = inet_addr(hostip);
-#  else
-	inet = inet_addr(hostip);
-	inet_s.s_addr = inet;
-#  endif
-# endif
 	xfree(hostip);
-	if (inet_s.s_addr == INADDR_NONE) {
-	    return FALSE;		/* not a valid IP number.... */
-	}
 	if (inet_s.s_addr == htonl(INADDR_LOOPBACK)) {
 	    return TRUE;		/* obviously us.... */
 	}
 	/*
 	 * Do a reverse query to get the hostname.  This is a bit bogus as it
 	 * requires that the lookup work for the check to function properly.
-	 * Better would be to look at the interfaces directly but doing that in
-	 * a host-independent manner is nearly impossible.
 	 */
 	if ((hostentp = gethostbyaddr((char *) &(inet_s.s_addr), sizeof(inet_s.s_addr), AF_INET))) {
 	    if (islocalhost(hostentp->h_name)) {
