@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:retry.c,v 1.41 2000/02/17 04:29:52 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:retry.c,v 1.47 2000/10/02 02:03:19 woods Exp"
  */
 
 /*
@@ -40,19 +40,22 @@
 #endif
 
 #include "smail.h"
-#include "alloc.h"
+#include "main.h"
+#include "parse.h"
 #include "addr.h"
+#include "direct.h"
+#include "route.h"
 #include "transport.h"
 #include "log.h"
-#include "parse.h"
+#include "alloc.h"
 #include "dys.h"
 #include "lookup.h"
 #include "jump.h"
-#ifndef DEPEND
-# include "extern.h"
-# include "debug.h"
-# include "error.h"
-#endif
+#include "spool.h"
+#include "smailconf.h"
+#include "extern.h"
+#include "debug.h"
+#include "error.h"
 
 #define RETRY_MODE   0644		/* file creation modes */
 
@@ -81,9 +84,6 @@
 # define VOLATILE	/* most compilers won't optimize global variables */
 #endif
 
-/* variables imported from libc */
-extern int errno;
-
 struct retry {
     struct retry *succ;			/* next in chain */
     char    *domains;			/* target domain, or "*" */
@@ -106,10 +106,11 @@ static int allow_later P_((time_t, time_t, struct retry *, struct error **));
 static char *tryfile_name P_((struct transport *, char *, int));
 static int tryfile_cat P_((struct str *, char *, unsigned, int, int, int));
 static int tryfile_mkdir P_((struct str *));
-static int tryfile_lock P_((int, long));
+static int tryfile_lock_wait P_((int, long));
+static int tryfile_lock P_((int));
 static void tryfile_unlock P_((int));
 static void tryfile_alarm P_((int));
-static void write_error P_((int, char *, struct error *));
+static void write_error_or_unlink P_((int, char *, struct error *));
 static void read_error P_((int, char *, struct error *));
 
 /*
@@ -162,19 +163,29 @@ read_retry_file()
     while ((entry = read_entry(f))) {
 	struct attribute *new;
 	char *p;
+	int len;
 	time_t interval, duration;
 
 	new = parse_table(entry, &error);
 	if (new == NULL) {
 	    break;
 	}
+	/* trim any comment for the error messages */
+	if ((p = strchr(entry, '#'))) {
+	    *p = '\0';
+	}
+	/* trim the trailing newline, if any, for error messages */
+	len = strlen(entry);
+	if (entry[len - 1] == '\n') {
+	    entry[len - 1] = '\0';
+	}
 
 	/* 
 	 * split the string into two parts on the '/'
 	 */
 	p = strchr(new->value, '/');
 	if (p == NULL) {
-	    error = xprintf("%s: %s: invalid entry, no '/': %s.", retry_file, new->name, new->value);
+	    error = xprintf("%s: %s: invalid entry, no '/': \"%s\".", retry_file, new->name, entry);
 	    xfree(new->name);
 	    /* note that new->value points to the same storage as new->name */
 	    xfree((char *) new);
@@ -188,7 +199,7 @@ read_retry_file()
 	if (*(new->value)) {
 	    interval = ivaltol(new->value);
 	    if (interval < 0) {
-		error = xprintf("%s: %s: invalid retry interval: %s.", retry_file, new->name, new->value);
+		error = xprintf("%s: %s: invalid retry interval: %s (from entry \"%s\").", retry_file, new->name, new->value, entry);
 		xfree(new->name);
 		xfree((char *) new);
 		break;
@@ -201,7 +212,7 @@ read_retry_file()
 	if (*p) {
 	    duration = ivaltol(p);
 	    if (duration < 0) {
-		error = xprintf("%s: %s: invalid retry duration: %s.", retry_file, new->name, p);
+		error = xprintf("%s: %s: invalid retry duration: %s (from entry \"%s\").", retry_file, new->name, p, entry);
 		xfree(new->name);
 		xfree((char *) new);
 		break;
@@ -215,7 +226,7 @@ read_retry_file()
 
 	/* assign new structure's domain */
 	/* shrink the storage from parse_table() to toss the "value" part */
-	r->domains = xrealloc(new->name, strlen(new->name));
+	r->domains = xrealloc(new->name, (unsigned) (strlen(new->name) + 1));
 
 	/* assign new structure's limits */
 	r->interval = interval;
@@ -235,7 +246,7 @@ read_retry_file()
 	 */
 	xfree((char *) new);
 
-	/* a lone wildcard record obviates need to read further */
+	/* a lone wildcard record obviates any need to read further */
 	if (EQ(nr->domains, "*")) {
 	    break;
 	}
@@ -243,10 +254,11 @@ read_retry_file()
     (void) fclose(f);
 
     /*
-     * if no errors, and there is no default record, and if
-     * global configuration requires a default record, add it.
+     * if there were no errors reading the retry file, and there is still no
+     * default record, and if global configuration requires a default record,
+     * add it.
      */
-    if (!error && nr && EQ(nr->domains, "*") && (r = default_retry())) {
+    if (!error && nr && !EQ(nr->domains, "*") && (r = default_retry())) {
 	if (nr == NULL) {
 	    retries = r;
 	} else {
@@ -269,22 +281,25 @@ dump_retry_config(f)
 
     fputs("#\n# -- retry configuration\n#\n", f);
     for (rp = retries; (rp); rp = rp->succ) {
-	char *interval, *duration;
+	char *value;
+	char *comment = NULL;
 
 	/* note also that ltoival() uses static memory which means that it
 	 * cannot be called twice in the same expression even if it didn't
 	 * return NULL....
 	 */
+	fprintf(f, "%s\t", rp->domains);
 	if (rp->interval == 0) {
 	    /* queue_interval might not be set yet so just print it symbolically */
-	    interval = "0(queue_interval*2)"; 
-	} else if (!(interval = ltoival(rp->interval))) {
-	    interval = "-1(invalid)";
+	    fputs("0/", f);
+	    comment = "#\tnote retry_interval of zero uses runtime (queue_interval * 2)\n";
+	} else {
+	    fprintf(f, "%s/", (value = ltoival(rp->interval)) ? value : (comment = "#\tinvalid interval in entry above\n", "-1"));
 	}
-	if (!(duration = ltoival(rp->duration))) {
-	    duration = "-1(invalid)";
+	fprintf(f, "%s\n", (value = ltoival(rp->duration)) ? value : (comment = "#\tinvalid duration in entry above\n", "-1"));
+	if (comment) {
+	    fputs(comment, f);
 	}
-	fprintf(f, "%s\t%s/%s\n", rp->domains, interval, duration);
     }
     fputs("#\n# -- end of retry configuration\n#\n", f);
 }
@@ -296,22 +311,23 @@ dump_retry_config(f)
 static struct retry *
 default_retry()
 {
-    struct retry *r;
+    struct retry *r = NULL;
 
-    if (!retry_interval && !retry_duration) {
-	return NULL;
+    if (retry_duration >= 0) {
+	r = (struct retry *) xmalloc(sizeof(struct retry));
+	r->domains = COPY_STRING("*");
+	r->interval = retry_interval;
+	r->duration = retry_duration;
+	r->succ = NULL;
     }
-
-    r = (struct retry *) xmalloc(sizeof(struct retry));
-    r->domains = COPY_STRING("*");
-    r->interval = retry_interval;
-    r->duration = retry_duration;
-    r->succ = NULL;
     return r;
 }
 
 /*
  * retry_addr_before - retry processing before transport attempt
+ *
+ * called from transport drivers that do local or external delivery
+ * (eg. appendfile and pipe).
  */
 struct addr *
 retry_addr_before(in, defer, fail)
@@ -351,7 +367,10 @@ retry_addr_before(in, defer, fail)
 
 		DEBUG1(DBG_RETRY_LO, "examine %s\n", tryname);
 
-		/* examine the retry file: lock; ignore empties */
+		/*
+		 * examine the retry file: lock and call allow_now() and then unlock;
+		 * ignore empties or missing files
+		 */
 		for (;;) {
 		    int lk;
 
@@ -360,7 +379,15 @@ retry_addr_before(in, defer, fail)
 			may_deliver = TRUE;
 			break;
 		    }
-		    lk = tryfile_lock(fd, 0L);
+		    lk = tryfile_lock_wait(fd, 0L);
+		    if (lk == DB_FAIL) {
+			DEBUG1(DBG_RETRY_LO, "tryfile_lock_wait() failed: %s", strerror(errno));
+			/* XXX should this be written to the system log instead? */
+			(void) close(fd);
+			(void) unlink(tryname);	/* XXX really??? */
+			where = defer;
+			break;
+		    }
 		    if (lk == DB_AGAIN) {
 			(void) close(fd);
 			continue;
@@ -451,7 +478,9 @@ retry_addr_finished(addr)
     struct addr *cur;
 
     for (cur = addr; cur; cur = cur->succ) {
-	if (cur->flags & ADDR_RETRY_FILE) {
+
+	if (cur->flags & ADDR_RETRY_FILE) {	/* only if we're supposed to
+						 * touch the retry file */
 	    struct error *error = cur->error;
 	    char *tryname;
 	    int fd;
@@ -468,8 +497,8 @@ retry_addr_finished(addr)
 		continue;
 	    }
 
-	    DEBUG2(DBG_RETRY_LO, "%s error message in %s\n",
-		   error ? "writing" : "removing", tryname);
+	    DEBUG2(DBG_RETRY_LO, "%s %s\n",
+		   error ? "writing error message in" : "removing", tryname);
 
 	    /* write to, or remove, retry file */
 	    for (;;) {
@@ -478,17 +507,34 @@ retry_addr_finished(addr)
 		if ((fd = open(tryname, O_WRONLY | (error ? O_CREAT : 0),
 			       RETRY_MODE)) == -1)
 		{
+		    if (error) {
+		        write_log(WRITE_LOG_SYS, "tryfile_lock_wait(): open(%s) failed: %s",
+				  tryname, strerror(errno));
+		    }
 		    break;
 		}
-		lk = tryfile_lock(fd, 0L);
-		if (lk == DB_AGAIN) {
+		lk = tryfile_lock_wait(fd, 0L);
+		if (lk == DB_FAIL) {
+		    write_log(WRITE_LOG_SYS, "tryfile_lock_wait() lock of %s failed: %s %s",
+			      tryname,
+			      strerror(errno),
+			      error ? error->message : "(would have removed it)");
 		    (void) close(fd);
-		    continue;
+		    break;
+		}
+		if (lk == DB_AGAIN) {	/* it was removed from under us but... */
+		    (void) close(fd);
+		    if (error) {
+			continue;	/* ...we still have something to say! */
+		    } else {
+			break;		/* ...or maybe we don't care */
+		    }
 		}
 
 		/* the rest happens only once */
 		if (lk == DB_SUCCEED) {
-		    write_error(fd, tryname, error);
+		    /* write the error, or unlink if none */
+		    write_error_or_unlink(fd, tryname, error);
 		    tryfile_unlock(fd);
 		}
 		(void) close(fd);
@@ -538,34 +584,49 @@ retry_host_lock(transport, host, defer_failure, error)
 	return FAIL;
     }
 
-    DEBUG1(DBG_RETRY_LO, "retry host lock %s\n", tryname);
-
     /* open and lock retry file */
     for (;;) {
 	int lk;
+	char *errstr;
 
 	if ((fd = open(tryname, O_RDWR|O_CREAT|O_SWYM, RETRY_MODE)) == -1) {
 	    if (!*error) {
-		*error = note_error(ERR_175, xprintf("cannot open host retry file %s: %s", tryname, strerror(errno)));
+		*error = note_error(ERR_175,
+				    xprintf("cannot open host retry file %s: %s",
+					    tryname, strerror(errno)));
 	    }
 	    *defer_failure = TRUE;
 	    return FAIL;
 	}
-	lk = tryfile_lock(fd, host_lock_timeout);
+	if (process_queue) {
+	    lk = tryfile_lock(fd);	/* don't bother waiting! */
+	} else {
+	    lk = tryfile_lock_wait(fd, host_lock_timeout);
+	}
 	if (lk == DB_SUCCEED) {
 	    break;
 	}
-	if (lk == DB_AGAIN) {
+	if (lk == DB_AGAIN) {		/* lock was removed from under us... */
 	    (void) close(fd);
-	    continue;
+	    continue;			/* ... so try again */
 	}
 
-	DEBUG1(DBG_RETRY_LO, "retry host lock %s TIMED OUT\n", tryname);
+	/* else DB_FAIL */
+	errstr = strerror(errno);
+	DEBUG2(DBG_RETRY_LO, "retry_host_lock(): %s: %s.\n",
+	       process_queue ? "already locked" : "timed out", errstr);
 	(void) close(fd);
 	if (!*error) {
-	    *error = note_error(ERR_DONTLOG | ERR_175, "host retry file locked");
+	    *error = note_error(ERR_DONTLOG | ERR_175,
+				xprintf("host retry file %s lock failed: %s (%s)",
+					tryname,
+					process_queue ? "already locked?" : "timed out?",
+					errstr));
 	}
 	*defer_failure = TRUE;
+
+	/* XXX should we unlink(tryname) here? */
+
 	return FAIL;
     }
 
@@ -592,10 +653,12 @@ retry_host_lock(transport, host, defer_failure, error)
     }
 
     /* keep human informed */
-    DEBUG3(DBG_RETRY_LO, "retry host lock %s (%s) %s\n",
+    DEBUG5(DBG_RETRY_LO, "lock of %s %s (%s)%s%s\n",
+	   tryname,
 	   (ret == SUCCEED) ? "succeeded" : "failed",
-	   dfok ? "will defer failure" : "will not defer failure",
-	   tryname);
+	   dfok ? "will defer if delivery fails" : "will NOT defer if delivery fails",
+	   (error && *error) ? ": " : "",
+	   (error && *error) ? (*error)->message : "");
 
     /* if retry control allows deferral, inform caller */
     *defer_failure = (dfok) ? TRUE : FALSE;
@@ -634,11 +697,13 @@ retry_host_unlock(started, error)
 	return;
     }
 
-    DEBUG2(DBG_RETRY_LO, "retry host unlock (%s) %s\n",
-	   error ? "error" : "success", locked_path);
+    DEBUG3(DBG_RETRY_LO, "unlocking %s and %s%s.\n",
+	   locked_path,
+	   error ? "storing error: " : "unlinking",
+	   error ? error->message : "");
 
-    /* write the error */
-    write_error(locked_fd, locked_path, error);
+    /* write the error, or unlink if none */
+    write_error_or_unlink(locked_fd, locked_path, error);
 
     /* unlock and close the retry file */
     tryfile_unlock(locked_fd);
@@ -700,12 +765,7 @@ allow_now(fd, tryname, spooled, started, r, error)
 	interval = r->interval;
     }
     if (last && last <= started && (started - last) < interval) {
-
-	/* retry interval has not passed, so we won't deliver */
-	DEBUG1(DBG_RETRY_LO, "retry interval not reached for %s.\n", tryname);
-
 	if ((started - spooled) > r->duration) {
-	    DEBUG1(DBG_RETRY_MID, "retry duration for %s exceeded.\n", tryname);
 	    /* your time is up */
 	    if (*error == NULL) {
 		struct error err;
@@ -737,7 +797,6 @@ allow_later(spooled, started, r, error)
 {
     /* if this was the last try... */
     if ((started - spooled) > r->duration) {
-	DEBUG(DBG_RETRY_LO, "allow_later(): retry duration exceeded.\n");
 	if (*error == NULL) {
 	    *error = note_error(ERR_173, "retry duration exceeded");
 	}
@@ -897,16 +956,17 @@ tryfile_mkdir(sp)
 }
 
 VOLATILE JUMPSIG old_sigalrm;
-VOLATILE int old_alarm;
+VOLATILE unsigned int old_alarm;
 
 /*
- * tryfile_lock - lock an open retry file; optionally, time out
+ * tryfile_lock_wait - lock an open retry file, waiting until an exclusive lock
+ * can be obtained; optionally, time out
  *
  * Returns DB_SUCCESS (good), DB_FAIL (bad),
- * or DB_AGAIN (if file was removed).
+ * or DB_AGAIN (if file was removed from under us).
  */
 static int
-tryfile_lock(fd, timeout)
+tryfile_lock_wait(fd, timeout)
     int fd;
     long timeout;
 {
@@ -942,6 +1002,29 @@ tryfile_lock(fd, timeout)
     return ret;
 }
 
+/*
+ * tryfile_lock - lock an open retry file, non-blocking mode
+ *
+ * Returns DB_SUCCESS (good), DB_FAIL (bad),
+ * or DB_AGAIN (if file was removed from under us).
+ */
+static int
+tryfile_lock(fd)
+    int fd;
+{
+    int ret;
+
+    if (lock_fd(fd) == FAIL) {
+	ret = DB_FAIL;
+    } else {
+	struct stat st;
+
+	ret = (fstat(fd, &st) == 0 && st.st_nlink > 0) ? DB_SUCCEED : DB_AGAIN;
+    }
+
+    return ret;
+}
+
 /*
  * tryfile_unlock - unlock a retry file locked with timeout
  */
@@ -964,10 +1047,10 @@ tryfile_alarm(sig)
 }
 
 /*
- * write_error - write an error description to a retry file, or remove it
+ * write_error_or_unlink - write an error description to a retry file, or remove it
  */
 static void
-write_error(fd, tryfile, error)
+write_error_or_unlink(fd, tryfile, error)
     int fd;
     char *tryfile;
     struct error *error;
@@ -1009,7 +1092,7 @@ read_error(fd, tryfile, error)
     struct stat statbuf;
     char *buf, *errmsg, *p;
     long errcode;
-    int len;
+    unsigned int len;
 
     /* assume that we can't read the file */
     error->info = ERR_173;
