@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:route.c,v 1.37 2000/02/03 03:42:16 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:route.c,v 1.38 2000/09/03 17:00:17 woods Exp"
  */
 
 /*
@@ -23,47 +23,44 @@
  */
 
 #define NEED_SOCKETS
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include "defs.h"
 #include "smail.h"
+#include "main.h"
+#include "parse.h"
+#include "addr.h"
 #include "route.h"
 #ifdef HAVE_BSD_NETWORKING
 #  define USE_MX_HINTS
 #  include "bindsmtpth.h"
 #endif /* HAVE_BSD_NETWORKING */
 #include "transport.h"
-#include "addr.h"
 #include "field.h"
+#include "alloc.h"
 #include "dys.h"
 #include "log.h"
 #include "exitcodes.h"
-#include "parse.h"
 #include "smailconf.h"
-#ifndef DEPEND
-# include "extern.h"
-# include "debug.h"
-# include "error.h"
-#endif
+#include "extern.h"
+#include "debug.h"
+#include "error.h"
 
 /* variables exported from this file */
 int cached_routers = FALSE;		/* TRUE if cache_routers() called */
 
-/* variables imported from libc */
-extern int errno;
-
 /* functions local to this file */
-static void premunge_remote_addrs();
-static void compute_transport();
-static void compute_next_addr();
-static void next_addr_basic();
-static void next_addr_uucp();
-static char *route_to_route_addr();
-static char *router_read_method();
-static char *router_driv_function();
+static void premunge_remote_addrs __P((struct addr *));
+static void compute_transport __P((struct addr *, char *));
+static void compute_next_addr __P((struct addr *));
+static void next_addr_basic __P((struct addr *, int));
+static void next_addr_uucp __P((struct addr *));
+static char *route_to_route_addr __P((char *));
+static char *router_read_method __P((char *, struct attribute *));
+static char *router_driv_function __P((char *, struct attribute *));
 
 /* Config table */
 static struct attr_table router_generic[] = {
@@ -226,7 +223,7 @@ route_remote_addrs(in, out, retry, defer, fail)
 		cur->succ = *fail;
 		*fail = cur;
 		if (*retry) {
-		    xfree(*retry);
+		    xfree((char *) *retry);
 		}
 		*retry = NULL;
 	    } else {
