@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:route.h,v 1.9 1999/10/27 19:03:53 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:route.h,v 1.10 2000/09/03 16:29:02 woods Exp"
  */
 
 /*
@@ -41,12 +41,16 @@ struct method {
 
 /* compiled in route drivers */
 struct route_driver {
-    char *name;				/* name of route driver */
-    void (*cache)();			/* function to cache routing info */
-    void (*driver)();			/* function to perform routing */
-    void (*finish)();			/* function to free resources */
-    char *(*builder)();			/* fun to read from router file */
-    void (*dumper)();			/* function to dump config */
+    char *name;					/* name of route driver */
+    void (*cache) __P((struct router *));	/* function to cache routing info */
+    void (*driver) __P((struct router *,
+			struct addr *,
+			struct addr **,
+			struct addr **,
+			struct addr **));	/* function to perform routing */
+    void (*finish) __P((struct router *));	/* function to free resources */
+    char *(*builder) __P((struct router *, struct attribute *)); /* func to read from router file */
+    void (*dumper) __P((FILE *, struct router *)); /* function to dump config */
 };
 
 /*
@@ -66,4 +70,17 @@ struct rt_info {
 #define USE_ALWAYS	0x0001		/* if match, don't use next router */
 #define RT_AFFECTS_USER 0x0002		/* This router can affect the user part */
 
+/* external functions defined in route.c */
+extern void route_remote_addrs __P((struct addr *, struct addr **, struct addr **, struct addr **, struct addr **));
+extern char *match_end_domain __P((char *, char *));
+extern char *is_suffix __P((char *, char *, int));
+extern void route_driver_finish __P((struct router *, struct addr *, int, char *, char *, struct transport *, struct transport_hints *));
+extern struct router *find_router __P((char *));
+extern struct route_driver *find_route_driver __P((char *));
+extern void cache_routers __P((void));
+extern void finish_routers __P((void));
+extern char *read_router_file __P((void));
+extern void dump_router_config __P((FILE *));
+extern struct method *read_method_file __P((char *, char **));
+
 #endif	/* ROUTE_H */
