@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/routers:RELEASE-3_2_0_111:bind.c,v 1.28 1998/11/20 01:02:25 woods Exp"
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_112:bind.c,v 1.29 2000/09/03 17:00:22 woods Exp"
  */
 
 /*
@@ -26,38 +26,32 @@
  * Specifications for this router driver can be found in "bindlib.c".
  *
  */
-#include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/param.h>
+#include <stdio.h>
+#include <errno.h>
 #include "defs.h"
 #include "../smail.h"
-#include "../smailconf.h"
+#include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../route.h"
 #include "../transport.h"
 #include "../lookup.h"
+#include "../alloc.h"
 #include "../dys.h"
+#include "../smailconf.h"
 #include "rtlib.h"
 #include "bind.h"
-#ifndef DEPEND
-# include "../extern.h"
-# include "../debug.h"
-# include "../error.h"
-#endif
+#include "../extern.h"
+#include "../debug.h"
+#include "../error.h"
 
 /* functions local to this file */
 
-#ifdef ANSI_C
-# define P_(x) x
-#else
-# define P_(x) ()
-#endif
-
-static int bind_lookup P_((struct router*,struct addr*,int,struct rt_info*,
-			   struct error **));
-#undef P_
+static int bind_lookup __P((struct router*,struct addr*,int,struct rt_info*,
+			    struct error **));
 
 static struct attr_table bind_attributes[] = {
     BIND_ATTRIBUTES(bind_private, bindlib_attr),
