@@ -1,4 +1,6 @@
-/* @(#) bind.h,v 1.9 1992/07/11 11:50:21 tron Exp */
+/*
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_112:bind.h,v 1.10 2000/09/03 16:26:33 woods Exp"
+ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -19,3 +21,11 @@
 struct bind_private {
     struct bindlib_private bindlib_attr;
 };
+
+extern void rtd_bind __P((struct router *,
+			  struct addr *,
+			  struct addr **,
+			  struct addr **,
+			  struct addr **));
+extern char *rtb_bind __P((struct router *, struct attribute *));
+extern void rtp_bind __P((FILE *, struct router *));
