@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/routers:RELEASE-3_2_0_111:gethost.c,v 1.20 2000/02/05 19:11:21 woods Exp"
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_112:gethost.c,v 1.22 2000/10/02 02:06:42 woods Exp"
  */
 
 /*
@@ -76,25 +76,27 @@
  */
 
 #define NEED_SOCKETS
-#include <stdio.h>
-#include <ctype.h>
 #include <sys/types.h>
+#include <sys/stat.h>
+#include <stdio.h>
+#include <errno.h>
+#include <ctype.h>
 #include "defs.h"
 #include "../smail.h"
-#include "../smailconf.h"
+#include "../main.h"
 #include "../parse.h"
-#include "../dys.h"
 #include "../addr.h"
 #include "../route.h"
 #include "../transport.h"
 #include "../lookup.h"
+#include "../alloc.h"
+#include "../dys.h"
+#include "../smailconf.h"
 #include "rtlib.h"
 #include "gethost.h"
-#ifndef DEPEND
-# include "../extern.h"
-# include "../debug.h"
-# include "../error.h"
-#endif
+#include "../extern.h"
+#include "../debug.h"
+#include "../error.h"
 
 #ifndef INADDR_NONE
 # define INADDR_NONE		((unsigned long) (-1)) /* XXX 64bit too??? */
@@ -112,9 +114,9 @@ extern int h_errno;
 #endif
 
 /* functions local to this file */
-static int gethostbyname_lookup();
-static int gethostbyaddr_lookup();
-static int bad_form();
+static int gethostbyname_lookup __P((struct router *, struct addr *, int, struct rt_info *, struct error **));
+static int gethostbyaddr_lookup __P((struct router *, struct addr *, int, struct rt_info *, struct error **));
+static int bad_form __P((struct router *, char *, struct error **));
 
 static struct attr_table gethostbyname_attributes[] = {
     { "domain", t_string, NULL, NULL, OFFSET(gethostbyname_private, domain) },
@@ -356,18 +358,15 @@ gethostbyaddr_lookup(rp, addr, fl, rt_info, error_p)
 {
     register char *p;			/* temp */
     struct in_addr inet_s;		/* internet address */
-#if !defined(HAVE_INET_ATON) && !defined(INET_ADDR_USE_STRUCT)
-    unsigned long inet;			/* internet address */
-#endif
 
     p = addr->target;
 
-    DEBUG2(DBG_DRIVER_HI, "router %s: gethostbyaddr_lookup called for %s.\n", rp->name, addr->target);
+    DEBUG2(DBG_DRIVER_HI, "router %s: gethostbyaddr_lookup called for %s.\n", rp->name, p);
 
     /* check for a domain literal form */
     if (!strchr(p, '[') && !strchr(p, ']')) {
 	/* definitely not a domain literal */
-	DEBUG2(DBG_DRIVER_HI, "router %s: gethostbyaddr_lookup not a domain literal %s.\n", rp->name, addr->target);
+	DEBUG2(DBG_DRIVER_HI, "router %s: gethostbyaddr_lookup(%s) not a literal IP address.\n", rp->name, p);
 	return DB_NOMATCH;
     }
     /*
@@ -386,23 +385,10 @@ gethostbyaddr_lookup(rp, addr, fl, rt_info, error_p)
 
     /* see if the inet library likes the address */
     *p = '\0';				/* chop off ending `]', for now */
-#ifdef HAVE_INET_ATON
     if (!inet_aton(addr->target + 1, &inet_s)) {
 	*p = ']';			/* put it back */
 	return bad_form(rp, addr->target, error_p);
     }
-#else
-# ifdef INET_ADDR_USE_STRUCT
-    inet_s = inet_addr(addr->target + 1);
-# else
-    inet = inet_addr(addr->target + 1);
-    inet_s.s_addr = inet;
-# endif
-#endif
-    if (inet_s.s_addr == INADDR_NONE) {
-	*p = ']';			/* put it back */
-	return bad_form(rp, addr->target, error_p);
-    }
     *p = ']';				/* put it back */
 
     /*
