@@ -1,4 +1,6 @@
-/* @(#) gethost.h,v 1.4 1992/07/11 11:50:26 tron Exp */
+/*
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_112:gethost.h,v 1.5 2000/09/03 16:26:33 woods Exp"
+ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -27,3 +29,19 @@ struct gethostbyname_private {
     char *domain;			/* optional domain names */
     char *required;			/* required domain names */
 };
+
+extern void rtd_gethostbyname __P((struct router *,
+				   struct addr *,
+				   struct addr **,
+				   struct addr **,
+				   struct addr **));
+extern char *rtb_gethostbyname __P((struct router *, struct attribute *));
+extern void rtp_gethostbyname __P((FILE *, struct router *));
+
+extern void rtd_gethostbyaddr __P((struct router *,
+				   struct addr *,
+				   struct addr **,
+				   struct addr **,
+				   struct addr **));
+extern char *rtb_gethostbyaddr __P((struct router *, struct attribute *));
+extern void rtp_gethostbyaddr __P((FILE *, struct router *));
