@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/routers:RELEASE-3_2_0_111:pathalias.c,v 1.12 1998/11/20 01:07:54 woods Exp"
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_112:pathalias.c,v 1.14 2001/02/05 23:27:11 woods Exp"
  */
 
 /*
@@ -81,35 +81,36 @@
  *		   Also remember that the target string passed must
  *		   be left intact for higher-level software.
  */
-#include <stdio.h>
-#include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <stdio.h>
+#include <errno.h>
+#include <ctype.h>
 #include "defs.h"
 #include "../smail.h"
-#include "../smailconf.h"
+#include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../route.h"
 #include "../transport.h"
 #include "../lookup.h"
+#include "../alloc.h"
 #include "../dys.h"
+#include "../smailconf.h"
 #include "rtlib.h"
 #include "pathalias.h"
-#ifndef DEPEND
-# include "../extern.h"
-# include "../debug.h"
-# include "../error.h"
-#endif
+#include "../extern.h"
+#include "../debug.h"
+#include "../error.h"
 
 /* functions local to this file */
-static int pathalias_lookup();
-static void close_if_reopen();
-static int find_domain();
-static char *strip_costs();
-static struct error *bad_entry();
-static struct error *open_failed();
-static struct error *lookup_error();
+static int pathalias_lookup __P((struct router *, struct addr *, int, struct rt_info *, struct error **));
+static void close_if_reopen __P((struct router *));
+static int find_domain __P((struct router *, struct addr *, char **, char **, struct error **));
+static char *strip_costs __P((char *));
+static struct error *bad_entry __P((struct router *, char *));
+static struct error *open_failed __P((struct router *, char *, char *));
+static struct error *lookup_error __P((struct router *, char *, char *));
 
 static struct attr_table pathalias_attributes[] = {
     { "file", t_string, NULL, NULL, OFFSET(pathalias_private, file) },
@@ -122,7 +123,6 @@ static struct attr_table pathalias_attributes[] = {
     { "reopen", t_boolean, NULL, NULL, PA_REOPEN },
     { "tryagain", t_boolean, NULL, NULL, PA_TRYAGAIN },
     { "optional", t_boolean, NULL, NULL, PA_OPTIONAL },
-    { "cacheopen", t_boolean, NULL, NULL, PA_CACHEOPEN },
 };
 static struct attr_table *end_pathalias_attributes =
     ENDTABLE(pathalias_attributes);
@@ -147,67 +147,6 @@ rtd_pathalias(rp, in, out, defer, fail)
     close_if_reopen(rp);
 }
 
-#ifdef notyet
-/*
- * rtc_pathalias - open a pathalias database in the server, if so requested
- */
-void
-rtc_pathalias(rp)
-    struct router *rp;			/* router entry */
-{
-    struct pathalias_private *priv;
-    int success;
-    char *error_text;
-
-    priv = (struct pathalias_private *)rp->private;
-    if (rp->flags & PA_CACHEOPEN) {
-	rp->flags &= ~(PA_OPENFAIL | PA_OPENAGAIN | PA_OPENNOMATCH);
-	if (priv->error_text) {
-	    xfree(priv->error_text);
-	}
-	success = cache_database(priv->file, priv->proto,
-				 priv->retries, priv->interval,
-				 (struct stat *)NULL,
-				 &priv->database, &error_text);
-	switch (success) {
-	case FILE_FAIL:
-	    rp->flags |= PA_OPENFAIL;
-	    error_text = COPY_STRING(error_text);
-	    break;
-
-	case FILE_AGAIN:
-	    rp->flags |= PA_OPENAGAIN;
-	    error_text = COPY_STRING(error_text);
-	    break;
-
-	case FILE_NOMATCH:
-	    rp->flags |= PA_OPENNOMATCH;
-	}
-    }
-}
-#endif
-
-#ifdef notyet
-/*
- * rtf_pathalias - close a pathalias database when smail is done with router
- */
-void
-rtf_pathalias(rp)
-    struct router *rp;
-{
-    struct pathalias_private *priv;
-
-    priv = (struct pathalias_private *)rp->private;
-    rp->flags &= ~(PA_OPENFAIL | PA_OPENAGAIN | PA_OPENNOMATCH);
-    if (priv->database) {
-	close_database(priv->database);
-    }
-    if (priv->error_text) {
-	xfree(priv->error_text);
-    }
-}
-#endif
-
 /*
  * rtb_pathalias - read the configuration file attributes
  */
@@ -456,7 +395,7 @@ find_domain(rp, addr, raw_path, match, error_p)
     struct pathalias_private *priv;	/* private data */
     char *savedomain = NULL;		/* saved position of removed domain */
     char *target = addr->target;	/* target being searched for */
-    int len = strlen(target);		/* total length of target */
+    unsigned int len = strlen(target);	/* total length of target */
     int success;			/* return from function calls */
     char *error_text = NULL;		/* error messages from subroutines */
 
@@ -476,56 +415,34 @@ find_domain(rp, addr, raw_path, match, error_p)
 
     /* open the database if it is not already open */
     if (priv->database == NULL) {
-	error_text = priv->error_text;
-	if (rp->flags & PA_OPENFAIL) {
-	    success = FILE_FAIL;
-	} else if (rp->flags & PA_OPENAGAIN) {
-	    success = FILE_AGAIN;
-	} else if (rp->flags & PA_OPENNOMATCH) {
-	    success = FILE_NOMATCH;
-	} else {
 #ifdef USE_TARGET_DOMAIN
-	    char *fstr = expand_string(priv->file, addr, "", "");
+	char *fstr = expand_string(priv->file, addr, "", "");
 
-	    if (fstr && *fstr) {
-		tdbname = COPY_STRING(fstr);
-		success = open_database(tdbname, priv->proto, priv->retries,
-					priv->interval, (struct stat *) NULL,
-					&priv->database, &error_text);
-		/* XXX could check if expand_string() did anything since
-		 * XXX we may not have to close the file if it didn't
-		 * XXX contain a variable to expand.  HOWEVER, since
-		 * XXX tdbname is a dynamic pointer to allocated data that
-		 * XXX we free before returning, we cannot *not* reopen.
-		 */
-		rp->flags |= PA_REOPEN;
-	    } else {
-		DEBUG1(DBG_DRIVER_LO,
-		       "pathalias: find_domain: error expanding dbname %s.",
-		       priv->file);
-		success = FILE_FAIL;
-		error_text = "Database file name not valid";
-	    }
-#else
-	    success = open_database(priv->file, priv->proto, priv->retries,
+	if (fstr && *fstr) {
+	    tdbname = COPY_STRING(fstr);
+	    success = open_database(tdbname, priv->proto, priv->retries,
 				    priv->interval, (struct stat *) NULL,
 				    &priv->database, &error_text);
-#endif
+	    /* XXX could check if expand_string() did anything since we may not
+	     * XXX have to close the file if it didn't contain a variable to
+	     * XXX expand.  HOWEVER, since tdbname is a dynamic pointer to
+	     * XXX allocated data that we free before returning, we cannot
+	     * XXX *not* reopen.
+	     */
+	    rp->flags |= PA_REOPEN;
+	} else {
+	    DEBUG1(DBG_DRIVER_LO,
+		   "pathalias: find_domain: error expanding dbname %s.",
+		   priv->file);
+	    success = FILE_FAIL;
+	    error_text = "Database file name not valid";
 	}
+#else
+	success = open_database(priv->file, priv->proto, priv->retries,
+				priv->interval, (struct stat *) NULL,
+				&priv->database, &error_text);
+#endif
 	if (success != FILE_SUCCEED) {
-	    switch (success) {
-	    case FILE_NOMATCH:
-		rp->flags |= PA_OPENNOMATCH;
-		return FILE_NOMATCH;
-
-	    case FILE_FAIL:
-		rp->flags |= PA_OPENFAIL;
-		break;
-
-	    case FILE_AGAIN:
-		rp->flags |= PA_OPENAGAIN;
-		break;
-	    }
 	    if (priv->error_text) {
 		xfree(priv->error_text);
 	    }
