@@ -1,4 +1,6 @@
-/* @(#) pathalias.h,v 1.3 1992/07/11 11:50:32 tron Exp */
+/*
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_112:pathalias.h,v 1.5 2001/02/05 23:27:11 woods Exp"
+ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -19,10 +21,6 @@
 #define PA_REOPEN	0x00010000	/* always reopen database to search */
 #define PA_OPTIONAL	0x00020000	/* the paths file is optional */
 #define PA_TRYAGAIN	0x00040000	/* defer address on open failure */
-#define PA_CACHEOPEN	0x00080000	/* cache open file descriptor */
-#define PA_OPENFAIL	0x00100000	/* internal - open got FILE_FAIL */
-#define PA_OPENAGAIN	0x00200000	/* internal - open got FILE_AGAIN */
-#define PA_OPENNOMATCH	0x00400000	/* internal - open got FILE_NOMATCH */
 
 /* private information stored per router file entry */
 struct pathalias_private {
@@ -31,7 +29,15 @@ struct pathalias_private {
     char *domain;			/* optional domain names */
     char *required;			/* required domain names */
     int retries;			/* max count of retries */
-    int interval;			/* sleep interval between retries */
+    unsigned int interval;		/* sleep interval between retries */
     char *database;			/* internal - open database */
     char *error_text;			/* internal - error text from open */
 };
+
+extern void rtd_pathalias __P((struct router *,
+			       struct addr *,
+			       struct addr **,
+			       struct addr **,
+			       struct addr **));
+extern char *rtb_pathalias __P((struct router *, struct attribute *));
+extern void rtp_pathalias __P((FILE *, struct router *));
