@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/routers:RELEASE-3_2_0_111:queryprog.c,v 1.9 1998/11/20 01:02:27 woods Exp"
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_112:queryprog.c,v 1.10 2000/09/03 17:00:23 woods Exp"
  */
 
 /*
@@ -41,12 +41,13 @@
  *	    The match status values are cached.
  */
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <stdio.h>
-#include <ctype.h>
 #include <errno.h>
+#include <ctype.h>
 #include "defs.h"
 #include "../smail.h"
-#include "../smailconf.h"
+#include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../route.h"
@@ -54,24 +55,21 @@
 #include "../hash.h"
 #include "../child.h"
 #include "../lookup.h"
+#include "../alloc.h"
 #include "../dys.h"
+#include "../smailconf.h"
 #include "rtlib.h"
 #include "queryprog.h"
-#ifndef DEPEND
-# include "../extern.h"
-# include "../debug.h"
-# include "../error.h"
-#endif
-
-/* variables imported from libc */
-extern int errno;
+#include "../extern.h"
+#include "../debug.h"
+#include "../error.h"
 
 /* functions local to this file */
-static int queryprogram_lookup();
-static int decode_query_output();
-static struct error *parse_error();
-static struct error *temp_command_error();
-static struct error *output_parse_error();
+static int queryprogram_lookup __P((struct router *, struct addr *, int, struct rt_info *, struct error **));
+static int decode_query_output __P((struct router *, char *, struct rt_info *, char **));
+static struct error *parse_error __P((struct router *, char *, char *));
+static struct error *temp_command_error __P((struct router *));
+static struct error *output_parse_error __P((struct router *, char *));
 
 static struct attr_table queryprogram_attributes[] = {
     { "cmd", t_string, NULL, NULL, OFFSET(queryprogram_private, cmd) },
