@@ -1,4 +1,6 @@
-/* @(#) queryprog.h,v 1.3 1992/07/11 11:50:38 tron Exp */
+/*
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_112:queryprog.h,v 1.4 2000/09/03 16:26:34 woods Exp"
+ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -24,3 +26,11 @@ struct queryprogram_private {
 
 #define QP_READ_PATH	  0x0001000	/* read path from prog output */
 #define QP_READ_TRANSPORT 0x0002000	/* read transport from prog output */
+
+extern void rtd_queryprogram __P((struct router *,
+				  struct addr *,
+				  struct addr **,
+				  struct addr **,
+				  struct addr **));
+extern char *rtb_queryprogram __P((struct router *, struct attribute *));
+extern void rtp_queryprogram __P((FILE *, struct router *));
