@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/routers:RELEASE-3_2_0_111:reroute.c,v 1.11 1998/11/20 01:07:55 woods Exp"
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_112:reroute.c,v 1.13 2001/02/05 23:27:11 woods Exp"
  */
 
 /*
@@ -194,34 +194,34 @@
  *	    needs lots of time and determination.  Please restrain yourself,
  *	    for the sake of fast and flawless mail delivery.
  */
-#include <stdio.h>
-#include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <stdio.h>
+#include <ctype.h>
 #include "defs.h"
 #include "../smail.h"
-#include "../smailconf.h"
+#include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../route.h"
 #include "../transport.h"
 #include "../lookup.h"
+#include "../alloc.h"
 #include "../dys.h"
+#include "../smailconf.h"
 #include "rtlib.h"
 #include "reroute.h"
-#ifndef DEPEND
-# include "../extern.h"
-# include "../debug.h"
-# include "../error.h"
-#endif
+#include "../extern.h"
+#include "../debug.h"
+#include "../error.h"
 
 /* functions local to this file */
-static int reroute_lookup();
-static void close_if_reopen();
-static int find_domain();
-static struct error *bad_entry();
-static struct error *open_failed();
-static struct error *lookup_error();
+static int reroute_lookup __P((struct router *, struct addr *, int, struct rt_info *, struct error **));
+static void close_if_reopen __P((struct router *));
+static int find_domain __P((struct router *, struct addr *, char **, char **, struct error **));
+static struct error *bad_entry __P((struct router *, char *));
+static struct error *open_failed __P((struct router *, char *, char *));
+static struct error *lookup_error __P((struct router *, char *, char *));
 
 static struct attr_table reroute_attributes[] = {
     { "file", t_string, NULL, NULL, OFFSET(reroute_private, file) },
@@ -238,7 +238,6 @@ static struct attr_table reroute_attributes[] = {
     { "matchlocal", t_boolean, NULL, NULL, RR_MATCHLOCAL },
     { "matchdb", t_boolean, NULL, NULL, RR_MATCHDB },
     { "bounceonly", t_boolean, NULL, NULL, RR_BOUNCEONLY },
-    { "cacheopen", t_boolean, NULL, NULL, RR_CACHEOPEN },
 };
 static struct attr_table *end_reroute_attributes =
     ENDTABLE(reroute_attributes);
@@ -263,67 +262,6 @@ rtd_reroute(rp, in, out, defer, fail)
     close_if_reopen(rp);
 }
 
-#ifdef notyet
-/*
- * rtc_reroute - open a reroute database in the server, if so requested
- */
-void
-rtc_reroute(rp)
-    struct router *rp;			/* router entry */
-{
-    struct reroute_private *priv;
-    int success;
-    char *error_text;
-
-    priv = (struct reroute_private *)rp->private;
-    if (rp->flags & RR_CACHEOPEN) {
-	rp->flags &= ~(RR_OPENFAIL | RR_OPENAGAIN | RR_OPENNOMATCH);
-	if (priv->error_text) {
-	    xfree(priv->error_text);
-	}
-	success = cache_database(priv->file, priv->proto,
-				 priv->retries, priv->interval,
-				 (struct stat *)NULL,
-				 &priv->database, &error_text);
-	switch (success) {
-	case FILE_FAIL:
-	    rp->flags |= RR_OPENFAIL;
-	    error_text = COPY_STRING(error_text);
-	    break;
-
-	case FILE_AGAIN:
-	    rp->flags |= RR_OPENAGAIN;
-	    error_text = COPY_STRING(error_text);
-	    break;
-
-	case FILE_NOMATCH:
-	    rp->flags |= RR_OPENNOMATCH;
-	}
-    }
-}
-#endif
-
-#ifdef notyet
-/*
- * rtf_reroute - close a reroute database when smail is done with router
- */
-void
-rtf_reroute(rp)
-    struct router *rp;
-{
-    struct reroute_private *priv;
-
-    priv = (struct reroute_private *)rp->private;
-    rp->flags &= ~(RR_OPENFAIL | RR_OPENAGAIN | RR_OPENNOMATCH);
-    if (priv->database) {
-	close_database(priv->database);
-    }
-    if (priv->error_text) {
-	xfree(priv->error_text);
-    }
-}
-#endif
-
 /*
  * rtb_reroute - read the configuration file attributes
  */
@@ -689,60 +627,41 @@ find_domain(rp, addr, new_path, flag, error_p)
 
 	/* open the database if it is not already open */
 	if (priv->database == NULL) {
-	    error_text = priv->error_text;
-	    if (rp->flags & RR_OPENFAIL) {
-		success = FILE_FAIL;
-	    } else if (rp->flags & RR_OPENAGAIN) {
-		success = FILE_AGAIN;
-	    } else if (rp->flags & RR_OPENNOMATCH) {
-		success = FILE_NOMATCH;
-	    } else {
 #ifdef USE_TARGET_DOMAIN
-		char *fstr = expand_string(priv->file, addr, "", "");
+	    char *fstr = expand_string(priv->file, addr, "", "");
 
-		if (fstr && *fstr) {
-		    tdbname = COPY_STRING(fstr);
-		    success = open_database(tdbname, priv->proto, priv->retries,
-					    priv->interval, (struct stat *) NULL,
-					    &priv->database, &error_text);
-		    /* XXX could check if expand_string() did anything since
-		     * XXX we may not have to close the file if it didn't
-		     * XXX contain a variable to expand.  HOWEVER, since
-		     * XXX tdbname is a dynamic pointer to allocated data that
-		     * XXX we free before returning, we cannot *not* reopen.
-		     */
-		    rp->flags |= RR_REOPEN;
-		} else {
-			DEBUG1(DBG_DRIVER_LO,
-			       "reroute: find_domain: error expanding dbname %s.",
-			       priv->file);
-			success = FILE_FAIL;
-			error_text = "Database file name not valid";
-		}
-#else
-		success = open_database(priv->file, priv->proto,
-					priv->retries, priv->interval,
-					(struct stat *)NULL, &priv->database,
-					&error_text);
-#endif
+	    if (fstr && *fstr) {
+		tdbname = COPY_STRING(fstr);
+		success = open_database(tdbname, priv->proto, priv->retries,
+					priv->interval, (struct stat *) NULL,
+					&priv->database, &error_text);
+		/* XXX could check if expand_string() did anything since we may
+		 * XXX not have to close the file if it didn't contain a
+		 * XXX variable to expand.  HOWEVER, since tdbname is a dynamic
+		 * XXX pointer to allocated data that we free before returning,
+		 * XXX we cannot *not* reopen.
+		 */
+		rp->flags |= RR_REOPEN;
+	    } else {
+		DEBUG1(DBG_DRIVER_LO,
+		       "reroute: find_domain: error expanding dbname %s.",
+		       priv->file);
+		success = FILE_FAIL;
+		error_text = "Database file name not valid";
 	    }
+#else
+	    success = open_database(priv->file, priv->proto,
+				    priv->retries, priv->interval,
+				    (struct stat *)NULL, &priv->database,
+				    &error_text);
+#endif
 	    if (success != FILE_SUCCEED) {
 		*savebang = '!';		/* restore the target */
 		switch (success) {
 		case FILE_NOMATCH:
-		    rp->flags |= RR_OPENNOMATCH;
-		    if ((rp->flags & (RR_OPTIONAL | RR_MATCHLOCAL))
-					== (RR_OPTIONAL | RR_MATCHLOCAL)) {
+		    if ((rp->flags & (RR_OPTIONAL | RR_MATCHLOCAL)) == (RR_OPTIONAL | RR_MATCHLOCAL)) {
 			    continue;
 		    }
-		    return FILE_NOMATCH;
-
-		case FILE_FAIL:
-		    rp->flags |= RR_OPENFAIL;
-		    break;
-
-		case FILE_AGAIN:
-		    rp->flags |= RR_OPENAGAIN;
 		    break;
 		}
 		if (priv->error_text) {
