@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/routers:RELEASE-3_2_0_111:rewrite.c,v 1.16 1999/11/26 04:12:40 woods Exp"
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_112:rewrite.c,v 1.18 2001/02/05 23:27:12 woods Exp"
  */
 /*
  * rewrite.c
@@ -122,26 +122,27 @@
  *			XXX: There should probably be a way to fine-tune
  *			the error handling.
  */
-#include <stdio.h>
-#include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <stdio.h>
+#include <errno.h>
+#include <ctype.h>
 #include "defs.h"
 #include "../smail.h"
-#include "../smailconf.h"
+#include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../route.h"
 #include "../transport.h"
 #include "../lookup.h"
+#include "../alloc.h"
 #include "../dys.h"
+#include "../smailconf.h"
 #include "rtlib.h"
 #include "rewrite.h"
-#ifndef DEPEND
-# include "../extern.h"
-# include "../debug.h"
-# include "../error.h"
-#endif
+#include "../extern.h"
+#include "../debug.h"
+#include "../error.h"
 
 /*
  * XXX: these error codes should go in error.h, or should be shared with
@@ -154,26 +155,25 @@
 #define ERR_1205 1205L	/* preparse error on rewritten address */
 
 /* functions local to this file */
-static int rewrite_lookup();
-static void close_if_reopen();
-static int find_domain();
-static struct error *bad_entry();
-static struct error *open_failed();
-static struct error *lookup_error();
-static struct error *expand_error();
+static int rewrite_lookup __P((struct router *, struct addr *, int, struct rt_info *, struct error **));
+static void close_if_reopen __P((struct router *));
+static int find_domain __P((struct router *, struct addr *, char **, char **, struct error **));
+static struct error *bad_entry __P((struct router *, char *));
+static struct error *open_failed __P((struct router *, char *, char *));
+static struct error *lookup_error __P((struct router *, char *, char *));
+static struct error *expand_error __P((struct router *, char *, char *));
 
 static struct attr_table rewrite_attributes[] = {
     /* note that many of these are shared with pathalias */
-    { "file", t_string, NULL, NULL, OFFSET(pathalias_private, file) },
-    { "proto", t_string, NULL, NULL, OFFSET(pathalias_private, proto) },
-    { "domain", t_string, NULL, NULL, OFFSET(pathalias_private, domain) },
-    { "required", t_string, NULL, NULL, OFFSET(pathalias_private, required) },
-    { "retries", t_int, NULL, NULL, OFFSET(pathalias_private, retries) },
-    { "interval", t_int, NULL, NULL, OFFSET(pathalias_private, interval) },
-    { "reopen", t_boolean, NULL, NULL, PA_REOPEN },
-    { "tryagain", t_boolean, NULL, NULL, PA_TRYAGAIN },
-    { "optional", t_boolean, NULL, NULL, PA_OPTIONAL },
-    { "cacheopen", t_boolean, NULL, NULL, PA_CACHEOPEN },
+    { "file", t_string, NULL, NULL, OFFSET(rewrite_private, file) },
+    { "proto", t_string, NULL, NULL, OFFSET(rewrite_private, proto) },
+    { "domain", t_string, NULL, NULL, OFFSET(rewrite_private, domain) },
+    { "required", t_string, NULL, NULL, OFFSET(rewrite_private, required) },
+    { "retries", t_int, NULL, NULL, OFFSET(rewrite_private, retries) },
+    { "interval", t_int, NULL, NULL, OFFSET(rewrite_private, interval) },
+    { "reopen", t_boolean, NULL, NULL, RW_REOPEN },
+    { "tryagain", t_boolean, NULL, NULL, RW_TRYAGAIN },
+    { "optional", t_boolean, NULL, NULL, RW_OPTIONAL },
 };
 static struct attr_table *end_rewrite_attributes =
     ENDTABLE(rewrite_attributes);
@@ -206,67 +206,6 @@ rtd_rewrite(rp, in, out, defer, fail)
     close_if_reopen(rp);
 }
 
-#ifdef notyet
-/*
- * rtc_rewrite - open a rewrite database in the server, if so requested
- */
-void
-rtc_rewrite(rp)
-    struct router *rp;			/* router entry */
-{
-    struct pathalias_private *priv;
-    int result;
-    char *error_text;
-
-    priv = (struct pathalias_private *)rp->private;
-    if (rp->flags & PA_CACHEOPEN) {
-	rp->flags &= ~(PA_OPENFAIL | PA_OPENAGAIN | PA_OPENNOMATCH);
-	if (priv->error_text) {
-	    xfree(priv->error_text);
-	}
-	result = cache_database(priv->file, priv->proto,
-				priv->retries, priv->interval,
-				(struct stat *)NULL,
-				&priv->database, &error_text);
-	switch (result) {
-	case FILE_FAIL:
-	    rp->flags |= PA_OPENFAIL;
-	    error_text = COPY_STRING(error_text);
-	    break;
-
-	case FILE_AGAIN:
-	    rp->flags |= PA_OPENAGAIN;
-	    error_text = COPY_STRING(error_text);
-	    break;
-
-	case FILE_NOMATCH:
-	    rp->flags |= PA_OPENNOMATCH;
-	}
-    }
-}
-#endif
-
-#ifdef notyet
-/*
- * rtf_rewrite - close a rewrite database when smail is done with router
- */
-void
-rtf_rewrite(rp)
-    struct router *rp;
-{
-    struct pathalias_private *priv;
-
-    priv = (struct pathalias_private *)rp->private;
-    rp->flags &= ~(PA_OPENFAIL | PA_OPENAGAIN | PA_OPENNOMATCH);
-    if (priv->database) {
-	close_database(priv->database);
-    }
-    if (priv->error_text) {
-	xfree(priv->error_text);
-    }
-}
-#endif
-
 /*
  * rtb_rewrite - read the configuration file attributes
  */
@@ -277,8 +216,6 @@ rtb_rewrite(rp, attrs)
 {
     char *error;
     static struct rewrite_private rewrite_template = {
-	{ /* XXX: keep this in synch with pathalias driver.
-	   * should really do this the way bindlib does it. */
 	NULL,				/* file */
 	"lsearch",			/* proto */
 	NULL,				/* domains */
@@ -290,7 +227,6 @@ rtb_rewrite(rp, attrs)
 #endif	/* not HAVE_RENAME */
 	2,				/* interval */
 	NULL,				/* database -- for internal use */
-	},
     };
     struct rewrite_private *priv;	/* new rewrite_private structure */
 
@@ -371,7 +307,6 @@ rewrite_lookup(rp, addr, fl, rt_info, error_p)
     char *match;	/* substring of target that was matched in database */
     char *flag;		/* flag field from raw_data */
     char *format;	/* format field from raw_data */
-    int form;		/* form of remainder */
     char *error = NULL;	/* error from some lower-level functions */
     char *new_address;	/* rewritten address */
     char *new_target = NULL; /* target of rewritten address */
@@ -382,10 +317,10 @@ rewrite_lookup(rp, addr, fl, rt_info, error_p)
 
     switch (result) {
     case FILE_NOMATCH:
-	if ((rp->flags & PA_OPTIONAL) == 0) {
-	    struct pathalias_private *priv;
+	if ((rp->flags & RW_OPTIONAL) == 0) {
+	    struct rewrite_private *priv;
 
-	    priv = (struct pathalias_private *)rp->private;
+	    priv = (struct rewrite_private *)rp->private;
 	    *error_p = open_failed(rp, priv->file, "Database not found");
 	    result = FILE_FAIL;
 	}
@@ -527,10 +462,10 @@ static void
 close_if_reopen(rp)
     struct router *rp;
 {
-    register struct pathalias_private *priv;
+    register struct rewrite_private *priv;
 
-    priv = (struct pathalias_private *)rp->private;
-    if (priv->database && (rp->flags&PA_REOPEN)) {
+    priv = (struct rewrite_private *)rp->private;
+    if (priv->database && (rp->flags & RW_REOPEN)) {
 	/*
 	 * close the database if it was open and the
 	 * reopen attribute is on stating that the database
@@ -570,14 +505,14 @@ find_domain(rp, addr, raw_data, match, error_p)
     char **match;			/* store matched target here */
     struct error **error_p;		/* lookup error */
 {
-    struct pathalias_private *priv;	/* private data */
+    struct rewrite_private *priv;	/* private data */
     char *savedomain = NULL;		/* saved position of removed domain */
     char *target = addr->target;	/* target being searched for */
-    int len = strlen(target);		/* total length of target */
+    unsigned int len = strlen(target);	/* total length of target */
     int result;				/* return from function calls */
     char *error_text = NULL;		/* error messages from subroutines */
 
-    priv = (struct pathalias_private *)rp->private;
+    priv = (struct rewrite_private *)rp->private;
 
     /*
      * check for a required domain.  If none from the list of required
@@ -593,56 +528,34 @@ find_domain(rp, addr, raw_data, match, error_p)
 
     /* open the database if it is not already open */
     if (priv->database == NULL) {
-	error_text = priv->error_text;
-	if (rp->flags & PA_OPENFAIL) {
-	    result = FILE_FAIL;
-	} else if (rp->flags & PA_OPENAGAIN) {
-	    result = FILE_AGAIN;
-	} else if (rp->flags & PA_OPENNOMATCH) {
-	    result = FILE_NOMATCH;
-	} else {
 #ifdef USE_TARGET_DOMAIN
-	    char *fstr = expand_string(priv->file, addr, "", "");
+	char *fstr = expand_string(priv->file, addr, "", "");
 
-	    if (fstr && *fstr) {
-		tdbname = COPY_STRING(fstr);
-		result = open_database(tdbname, priv->proto, priv->retries,
-				       priv->interval, (struct stat *) NULL,
-				       &priv->database, &error_text);
-		/* XXX could check if expand_string() did anything since
-		 * XXX we may not have to close the file if it didn't
-		 * XXX contain a variable to expand.  HOWEVER, since
-		 * XXX tdbname is a dynamic pointer to allocated data that
-		 * XXX we free before returning, we cannot *not* reopen.
-		 */
-		rp->flags |= PA_REOPEN;
-	    } else {
-		DEBUG1(DBG_DRIVER_LO,
-		       "rewrite: find_domain: error expanding dbname %s.",
-		       priv->file);
-		result = FILE_FAIL;
-		error_text = "Database file name not valid";
-	    }
-#else
-	    result = open_database(priv->file, priv->proto, priv->retries,
+	if (fstr && *fstr) {
+	    tdbname = COPY_STRING(fstr);
+	    result = open_database(tdbname, priv->proto, priv->retries,
 				   priv->interval, (struct stat *) NULL,
 				   &priv->database, &error_text);
-#endif
+	    /* XXX could check if expand_string() did anything since we may not
+	     * XXX have to close the file if it didn't contain a variable to
+	     * XXX expand.  HOWEVER, since tdbname is a dynamic pointer to
+	     * XXX allocated data that we free before returning, we cannot
+	     * XXX *not* reopen.
+	     */
+	    rp->flags |= RW_REOPEN;
+	} else {
+	    DEBUG1(DBG_DRIVER_LO,
+		   "rewrite: find_domain: error expanding dbname %s.",
+		   priv->file);
+	    result = FILE_FAIL;
+	    error_text = "Database file name not valid";
 	}
+#else
+	result = open_database(priv->file, priv->proto, priv->retries,
+			       priv->interval, (struct stat *) NULL,
+			       &priv->database, &error_text);
+#endif
 	if (result != FILE_SUCCEED) {
-	    switch (result) {
-	    case FILE_NOMATCH:
-		rp->flags |= PA_OPENNOMATCH;
-		return FILE_NOMATCH;
-
-	    case FILE_FAIL:
-		rp->flags |= PA_OPENFAIL;
-		break;
-
-	    case FILE_AGAIN:
-		rp->flags |= PA_OPENAGAIN;
-		break;
-	    }
 	    if (priv->error_text) {
 		xfree(priv->error_text);
 	    }
@@ -811,7 +724,8 @@ open_failed(rp, file, open_error)
      *      error encountered should be stored in errno.
      *
      * ACTIONS
-     *      Defer all of the input addresses as configuration errors.
+     *      Defer all of the input addresses as configuration errors, but do
+     *      not call it a configuration error if `tryagain' is set.
      *
      * RESOLUTION
      *      The postmaster should check the router entry against the
@@ -819,9 +733,10 @@ open_failed(rp, file, open_error)
      */
     error_text = xprintf("router %s: rewrite database %s, open failed: %s",
 			 rp->name, file, open_error);
-    DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
+    DEBUG2(DBG_DRIVER_LO, "%s: %s\n", error_text, (rp->flags & RW_TRYAGAIN) ? "(will defer)" : "Configuration error!");
 
-    return note_error(ERR_1202, error_text);
+    return note_error((rp->flags & RW_TRYAGAIN) ? ERR_1202 : ERR_CONFERR|ERR_1202,
+		      error_text);
 }
 
 static struct error *
