@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/routers:RELEASE-3_2_0_111:rtlib.c,v 1.12 2000/02/03 03:42:24 woods Exp"
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_112:rtlib.c,v 1.13 2000/09/03 17:00:23 woods Exp"
  */
 
 /*
@@ -17,19 +17,23 @@
  *	this intelligence.
  */
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <stdio.h>
+#include <errno.h>
 #include "defs.h"
 #include "../smail.h"
+#include "../main.h"
+#include "../parse.h"
 #include "../addr.h"
 #include "../route.h"
 #include "../transport.h"
 #include "../lookup.h"
+#include "../alloc.h"
+#include "../dys.h"
 #include "rtlib.h"
-#ifndef DEPEND
-# include "../extern.h"
-# include "../debug.h"
-# include "../error.h"
-#endif
+#include "../extern.h"
+#include "../debug.h"
+#include "../error.h"
 
 /*
  * rtd_standard - standard function for routing to a target
@@ -116,7 +120,11 @@ rtd_standard(rp, in, out, defer, fail, lookup)
     struct addr **out;			/* non-failed addr structures */
     struct addr **defer;		/* addrs to defer to a later time */
     struct addr **fail;			/* unresolvable addrs */
-    int (*lookup)();			/* function to actually do routing */
+    int (*lookup) __P((struct router *,
+		  struct addr *,
+		  int,
+		  struct rt_info *,
+		  struct error **));
 {
     register struct addr *cur;		/* current addr being processed */
     struct addr *next;			/* next addr to process */
