@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/routers:RELEASE-3_2_0_111:smarthost.c,v 1.12 1998/11/20 01:02:29 woods Exp"
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_112:smarthost.c,v 1.13 2000/09/03 17:00:24 woods Exp"
  */
 
 /*
@@ -52,26 +52,28 @@
  *	  through the routers to resolve the transport to the smarthost.
  */
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <stdio.h>
+#include <errno.h>
 #include "defs.h"
 #include "../smail.h"
-#include "../smailconf.h"
+#include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../route.h"
 #include "../transport.h"
+#include "../alloc.h"
 #include "../dys.h"
+#include "../smailconf.h"
 #include "smarthost.h"
-#ifndef DEPEND
-# include "../extern.h"
-# include "../debug.h"
-# include "../error.h"
-#endif
+#include "../extern.h"
+#include "../debug.h"
+#include "../error.h"
 
 /* functions local to this file */
-static struct error *no_smarthost_info();
-static struct error *smarthost_parse_error();
-static int match_auth_domains();
+static struct error *no_smarthost_info __P((struct router *));
+static struct error *smarthost_parse_error __P((struct router *, char *, char *));
+static int match_auth_domains __P((char *));
 
 static struct attr_table smarthost_attributes[] = {
     { "path", t_string, NULL, NULL, OFFSET(smarthost_private, path) },
