@@ -1,4 +1,6 @@
-/* @(#) smarthost.h,v 1.3 1992/07/11 11:50:48 tron Exp */
+/*
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_112:smarthost.h,v 1.4 2000/09/03 16:29:20 woods Exp"
+ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -19,3 +21,11 @@ struct smarthost_private {
     char *host;				/* next_host to get to smarthost */
     char *route;			/* route from next_host to smarthost */
 };
+
+extern void rtd_smarthost __P((struct router *,
+			       struct addr *,
+			       struct addr **,
+			       struct addr **,
+			       struct addr **));
+extern char *rtb_smarthost __P((struct router *, struct attribute *));
+extern void rtp_smarthost __P((FILE *, struct router *));
