@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/routers:RELEASE-3_2_0_111:uuname.c,v 1.9 1999/11/26 03:38:52 woods Exp"
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_112:uuname.c,v 1.10 2000/09/03 17:00:24 woods Exp"
  */
 
 /*
@@ -43,37 +43,34 @@
  *	notes:
  *	    The output of the command is cached.
  */
-#include <stdio.h>
-#include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <stdio.h>
+#include <errno.h>
 #include "defs.h"
 #include "../smail.h"
-#include "../smailconf.h"
+#include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../route.h"
 #include "../transport.h"
+#include "../alloc.h"
 #include "../dys.h"
 #include "../lookup.h"
 #include "../child.h"
+#include "../smailconf.h"
 #include "rtlib.h"
 #include "uuname.h"
-#ifndef DEPEND
-# include "../extern.h"
-# include "../debug.h"
-# include "../error.h"
-#endif
-
-/* variables imported from libc */
-extern int errno;
+#include "../extern.h"
+#include "../debug.h"
+#include "../error.h"
 
 /* functions local to this file */
-static int uuname_lookup();
-static int scan_uuname();
-static int read_command();
-static struct error *open_error();
-static struct error *scan_error();
+static int uuname_lookup __P((struct router *, struct addr *, int, struct rt_info *, struct error **));
+static int scan_uuname __P((struct uuname_private *, char *, char **, char **));
+static int read_command __P((struct uuname_private *, char **));
+static struct error *open_error __P((struct router *, char *));
+static struct error *scan_error __P((struct router *, char *));
 
 static struct attr_table uuname_attributes[] = {
     { "cmd", t_string, NULL, NULL, OFFSET(uuname_private, cmd) },
