@@ -1,4 +1,6 @@
-/* @(#) uuname.h,v 1.3 1992/07/11 11:50:52 tron Exp */
+/*
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_112:uuname.h,v 1.4 2000/09/03 16:26:34 woods Exp"
+ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -30,3 +32,13 @@ struct uuname_private {
     int stat_ino;			/* internal - st_ino from stat */
     int stat_mtime;			/* internal - st_mtime from stat */
 };
+
+extern void rtd_uuname __P((struct router *,
+			    struct addr *,
+			    struct addr **,
+			    struct addr **,
+			    struct addr **));
+extern void rtf_uuname __P((struct router *));
+extern void rtc_uuname __P((struct router *));
+extern char *rtb_uuname __P((struct router *, struct attribute *));
+extern void rtp_uuname __P((FILE *, struct router *));
