@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_111:silly.c,v 1.8 1997/06/25 04:48:46 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_112:silly.c,v 1.10 2000/09/09 20:52:31 woods Exp"
  */
 
 /*
@@ -36,12 +36,16 @@
 # endif
 #endif  /* not POSIX_OS */
 
+#include "smail.h"
+#include "main.h"
+#include "addr.h"
+#include "alloc.h"
+#include "dys.h"
+
 #ifdef STANDALONE
 char *local_sender;
 #else /* !STANDALONE */
-#ifndef DEPEND
 # include "extern.h"
-#endif
 extern char *local_sender;
 #endif /* !STANDALONE */
 
@@ -57,7 +61,7 @@ void
 silly()
 {
 	char name[TOMBNAME+1];		/* name to put on tombstone */
-	int sender_len;			/* length of the sender string */
+	unsigned int sender_len;	/* length of the sender string */
 	int pad_len;			/* spaces on left for centering */
 	int i;				/* index */
 
@@ -83,7 +87,7 @@ silly()
 	printf("\n\nThe route-addr hits...");
 	fflush(stdout);
 	sleep(1);
-	printf("\rYour mailer feels weeker");
+	printf("\rYour mailer feels weaker");
 	fflush(stdout);
 	sleep(1);
 	printf("\rThe route-addr hits...     ");
